#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Dener Rodrigues'

EXCEPTIONS = []
global RELOAD
RELOAD = False
terminal_keystr = "<Ctrl>T"
from sigpdv.tools.log import traced as Log


def valida_versao_alchemy(versao):
    if float(versao[0] + '.' + versao[1:].replace('.', '') + '0') < 0.711:
        msg_erro = 'Sistema não pode inicializar devido a versão do SQLAlchemy estar desatualizada.' \
                   '\n\nVersão deve ser superior ou igual a 0.7.11' \
                   '\n Versão atual %s' % versao
        app = QtGui.QApplication(sys.argv)
        Log.error(msg_erro)
        message_dlg(None, MSG_ERROR, msg_erro)
        os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
        sys.exit()


def valida_versao_sigutils(versao):
    if len(versao.split('.')[2]) == 1:
        versao += '0'

    if int(versao.replace('.', '')) < 2199:
        msg_erro = 'Sistema não pode inicializar devido a versão do SigUtils estar desatualizada.' \
                   '\n\nVersão deve ser superior ou igual a 2.1.22 ' \
                   '\n Versão atual %s' % versao
        app = QtGui.QApplication(sys.argv)
        Log.error(msg_erro)
        message_dlg(None, MSG_ERROR, msg_erro)
        os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
        sys.exit()

def valida_versao_nfe():
    from nfe import __version__ as versao_nfe
    Log.info('Versão python-NFe: %s' % versao_nfe)
    n_versao = ''
    for i in versao_nfe.split('.'):
        n_versao += i.rjust(3, '0')

    if n_versao < '004051013':
        raise Exception('Atualize o pacote Python-nfe\n'
                        'Versão requerida: 4.51.13 ou superior\n'
                        'Versão atual: %s' % versao_nfe)

@Log
def atualiza_ip_no_supervisor(config):
    import requests
    # parametros da aba 'SigERP'
    server = config.get('erp_sup_host')
    bd = config.get('erp_sup_name')
    porta = config.get('erp_sup_port')
    pwd_sup = config.get('erp_sup_pass')
    username_sup = config.get('erp_sup_user')
    caixa = config.get('caixa')
    loja = config.get('loja')
    try:
        res = requests.post('http://%s:%s/atualizar_ip_caixa' % (server, porta),
                            auth=(username_sup, pwd_sup),
                            data={'caixa': caixa,
                                  'loja': loja,
                                  'bd': bd,
                                  'senha':pwd_sup,
                                  'usuario':username_sup,
                                  'bd_caixa': config['db_pdv_name'],
                                  'usuario_bd_caixa': config['db_pdv_user']},
                            timeout=10000)
    except Exception as e:
        Log.warning("Erro ao atualizar ip no supervisor: %s" % e)
        res = False

    return res and res.content

try:
    import os
    import subprocess
    import commands
    from PySide import QtCore, QtGui
    Log.info("Frente Aberto(Pid: %s)" % os.getpid())
    sub_process = subprocess.Popen(['ps', '-ho', 'pid,ppid,args'], stdout=subprocess.PIPE)
    out, err = sub_process.communicate()
    # caso o frente levante duas vezes, a segunda istancia fecha automaticamente.
    for line in out.splitlines():
        dados_linha = line.split()
        if ('./sigfrente' == dados_linha[-1] or
            'sigfrente' == dados_linha[-1] or
            'pos.py' in dados_linha[-1] ) and not str(os.getpid()) in line:
                Log.info('Dois Frentes abertos!')
                cmd = 'kill -9 %s' % os.getpid()
                Log.info(cmd)
                os.system(cmd)

    import re
    import sys
    import traceback

    import sqlalchemy
    from sigpdv.ui.messagebox import YES_ROLE, NO_ROLE, MSG_ERROR, MSG_QUESTION, MSG_INFO, MSG_NONE, \
        message_dlg

    valida_versao_alchemy(sqlalchemy.__version__)

    import SigUtils
    Log.info('Versão SigUtils: %s' % SigUtils.__version__)

    valida_versao_sigutils(SigUtils.__version__)

    from nfe.nfc_e import nfce as nfc_e
    valida_versao_nfe()

    from SigUtils.calcDigito import ean, baseModulo, modulo10
    from SigUtils.biometria.fingertech.fingertech import Fingertech
    from exceptions import Exception
    from sqlalchemy.exc import SQLAlchemyError
    from psycopg2._psycopg import IntegrityError
    from sigpdv.models.supervisor import SupervisorCfgConfiguracao, SupervisorCadSerie, SupervisorCadItens, \
        SupervisorDavCapa, SupervisorDavCorpo, SupervisorDavFechamento, SupervisorCadVendedor, \
        SupervisorCdaCapa, SupervisorCdaCorpo, SupervisorCdaItensFavoritos, SupervisorPveCapa, SupervisorPveCorpo, \
        SupervisorDavVencimentos
    from sigpdv.models.db_utils import sess, b_sess, exec_sql, testa_conexao_rpc, dict_2_sql, \
                                       testa_conexao_pg, modulo_instalado_fichas

    from sigpdv.tools.config import config, _OPTIONS
    from sigpdv.models.cad_cfop_data import CaixaCadNfeCfop
    from sigpdv.models.cad_clientes_data import CaixaCadClientes
    from sigpdv.models.cad_finalizadora_data import CaixaCadFinalizadora
    from sigpdv.models.cad_bandeiras_data import CaixaCadBandeiras
    from sigpdv.models.cad_finzalizadora_na_ecf_data import CaixaCadFinalizadoraNaEcf
    from sigpdv.models.cad_tributacao_nfce_data import CaixaCadTributacaoNfce
    from sigpdv.models.cad_itens_data import CaixaCadItens
    from sigpdv.models.cad_restricao_item_data import CaixaCadResticaoItem
    from sigpdv.models.cad_kits_data import CaixaCadKits
    from sigpdv.models.cad_mensagens_clientes_data import CaixaCadMensagensClientes
    from sigpdv.models.cad_mesas_data import CaixaCadMesas
    from sigpdv.models.cad_modelos_nfe_data import CaixaCadModelosNfe
    from sigpdv.models.cad_municipios_data import CaixaCadMunicipios
    from sigpdv.models.cad_operadores_data import CaixaCadOperadores
    from sigpdv.models.cad_promo_capa_data import CaixaCadPromoCapa
    from sigpdv.models.cad_promo1_data import CaixaCadPromo1
    from sigpdv.models.cad_promo2_data import CaixaCadPromo2
    from sigpdv.models.cad_promo3f_data import CaixaCadPromo3f
    from sigpdv.models.cad_promo3p_data import CaixaCadPromo3p
    from sigpdv.models.cad_promo4_data import CaixaCadPromo4
    from sigpdv.models.cad_promo5f_data import CaixaCadPromo5f
    from sigpdv.models.cad_promo5p_data import CaixaCadPromo5p
    from sigpdv.models.cad_promo6_data import CaixaCadPromo6
    from sigpdv.models.cad_promo7_data import CaixaCadPromo7
    from sigpdv.models.cad_comandos_impressoras_data import CaixaCadComandosImpressoras
    from sigpdv.models.cad_lista_clientes_data import CaixaCadListaClientes
    from sigpdv.models.cad_tributacao_data import CaixaCadTributacao
    from sigpdv.models.cad_tributacao_simples_nacional_data import CaixaCadTributacaoSimplesNacional
    from sigpdv.models.cad_sorteio_data import CaixaCadSorteio
    from sigpdv.models.cad_vendedor_data import CaixaCadVendedor
    from sigpdv.models.cfg_configuracao_data import CaixaCfgConfiguracao
    from sigpdv.models.cfg_ecf_autorizada_data import CaixaCfgEcfAutorizada
    from sigpdv.models.cfg_mensagens_data import CaixaCfgMensagens
    from sigpdv.models.cfg_parametros_data import CaixaCfgParametros
    from sigpdv.models.cfg_versao_data import CaixaCfgVersao
    from sigpdv.models.cor_nsu_data import CaixaCorNsu
    from sigpdv.models.mov_caixa_data import CaixaMovCaixa
    from sigpdv.models.mov_capa_data import CaixaMovCapa
    from sigpdv.models.mov_corpo_xz_data import CaixaMovCorpoXZ
    from sigpdv.models.mov_corpos_data import CaixaMovCorpos
    from sigpdv.models.mov_cupom_servidor_data import CaixaMovCupomServidor
    from sigpdv.control.mov_cupom_servidor import MovCupomServidor
    from sigpdv.models.mov_fechamentos_data import CaixaMovFechamentos
    from sigpdv.models.mov_itens_mesa_data import CaixaMovItensMesa
    from sigpdv.models.mov_meios_pagamentos_data import CaixaMovMeiosPagamentos
    from sigpdv.models.mov_mesas_data import CaixaMovMesas
    from sigpdv.models.mov_saldo_data import CaixaMovSaldo
    from sigpdv.models.mov_totalizadores_xz_data import CaixaMovTotalizadoresXZ
    from sigpdv.models.mov_vencimentos_data import CaixaMovVencimentos
    from sigpdv.models.mov_kits_vendidos_data import CaixaMovKitsVendidos
    from sigpdv.models.nfe_capa_data import CaixaNfeCapa
    from sigpdv.models.nfe_itens_data import CaixaNfeItens
    from sigpdv.models.mov_autorizacoes_data import CaixaMovAutorizacoes
    from sigpdv.models.cad_operada_cartao_data import CaixaCadOperadoraCartao
    if config['usar_ficha']:
        try:
            if not modulo_instalado_fichas:
                msg = "O parâmetro 'Ficha' no PdvConfig de ser desmarcado ou o módulo\n" \
                      "Supervisor Fichas deve ser instalado no Supervisor!"
                Log.error(msg)
                app = QtGui.QApplication(sys.argv)
                message_dlg(None, MSG_ERROR, 'A Aplicação será encerrada', msg)
                os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
                sys.exit()
        except Exception as e:
            Log.error(str(e))

        from sigpdv.models.cad_ficha_data import CaixaCadFicha
        from sigpdv.ui.busca_obs_produto import BuscaObsProduto

    from sigpdv.web_sig.websig_sql_capa import CaixaWebsig
    from elixir import create_all, setup_all
    # setup_all  e create_all criam as tabelas que nao existem no banco de dados
    # dentro de cada arquivo do model tinha uma chamada dessas funcoes, foram retiradas pois quando o supervisor não está
    # na mesma loja ou o ip trocou, demora muito para inicializar o frente.
    try:
        setup_all()
        create_all()
    except Exception as e:
        Log.error(str(e))
    from sigpdv.control.mov_capa import MovCapa
    from sigpdv.control.cupom_fiscal import CupomFiscal
    from sigpdv.control.movimento_ecf import MovimentoEcf
    from sigpdv.control.mov_corpos import MovCorpos
    from sigpdv.control.mov_fechamentos import MovFechamentos
    from sigpdv.control.mov_vencimentos import MovVencimentos
    from sigpdv.control.cad_promo_capa import CadPromoCapa
    from sigpdv.control.cad_promo1 import CadPromo1
    from sigpdv.control.cad_promo2 import CadPromo2
    from sigpdv.control.cad_promo3p import CadPromo3p
    from sigpdv.control.cad_promo3f import CadPromo3f
    from sigpdv.control.cad_promo4 import CadPromo4
    from sigpdv.control.cad_promo5p import CadPromo5p
    from sigpdv.control.cad_promo5f import CadPromo5f
    from sigpdv.control.cad_promo6 import CadPromo6
    from sigpdv.control.cad_promo7 import CadPromo7
    from sigpdv.control.cad_funcionario import Cadfuncionarios
    from sigpdv.control.cad_comandos_impressoras import CadComandosImpressoras
    from sigpdv import get_version
    try:
        atualiza_ip_no_supervisor(config)
    except:
        pass
    from sigpdv.dav_pve.dav_erp import import_dav, get_state_pedido_dav, \
        ErrConnRPC, cancelar_pedidos
    from sigpdv.dav_pve.dav_gestao import importar_davs_gestao, DavGestao
    from sigpdv.tools.cardinals import NumeroPorExtenso
    from sigpdv.tools.crypto import assinatura_ead
    from datetime import timedelta, time, date
    from dateutil.parser import parse
    from decimal import Decimal, ROUND_DOWN, ROUND_UP
    from ui.dlpPwd import password_dlg
    from ui.info_dialog import InfoDialog
    from sigtef.interativo import SigTefI
    from stoqdrivers.enum import TaxType, UnitType
    from stoqdrivers.kbd.teclado import Teclado
    from stoqdrivers.exceptions import DriverError, CommandError, \
        CouponOpenError, PendingReduceZ, HardwareFailure, \
        PrinterOfflineError, ItemAdditionError, CancelItemError, \
        PdvConnectionError, PrinterGTError, InvalidState, PrinterSerialError, \
        PrinterFileError, CommError, PaymentAdditionError, CodigoInvalido, AlreadyTotalized, \
        EcfDateError, CouponNotOpenError, OutofPaperError, TimeoutError, AlmostOutofPaper, \
        SummerTime, CouponNoFiscalOpenError
    from stoqdrivers.readers.barcode.reader import BarcodeReader
    from subprocess import call
    from time import sleep
    from gui.ui.ui_sigpdv import Ui_Dialog as Ui_PdvWindow
    from gui.widgets.pixmap import Animation
    from sigpdv.control.utils import gerar_registro_ident_estab, gerar_registro_relacao_merc, \
        gerar_registro_totalizacao, relatorio_gerencial_meios_de_pgto, \
        relatorio_gerencial_parametros_cfg, relatorio_gerencial_dav_emitidos, \
        gerar_relacao_dav_emitidos, gerar_itens_dav_emitidos, \
        gerar_cotep_09_08, get_info_pafecf, gerar_ac_09_08, \
        exportacao, checar_conexao, update_lista_app, \
        relatorio_identificacao_pafecf, rel_mesas_abertas, rel_transf_mesas, \
        atualizar_saldo_inicial_supervisor, \
        rel_gerencial_conf_mesas, gerar_sped_df, gerar_indice_tecnico, config_dados_cliente, \
        verificar_itens_vendidos_em_promocao, ler_arq_socio_torcedor, \
        get_balanca, reload_app, remove_accents, converte_tz
    from sigpdv.control.cad_tributacao import CadTributacao
    from sigpdv.control.cad_kits import CadKits
    from sigpdv.control.importacao_automatica import ImportacaoAutomatica
    from sigpdv.control.fiscal_printer import FiscalPrinterHelper
    from sigpdv.control.relatorio_gerencial import GerencialReport
    from sigpdv.control.mov_meios_pagamentos import MovMeiosPagamentos
    from sigpdv.control.sangria import Sangria
    from sigpdv.control.suprimento import Suprimento
    from sigpdv.control.doacao import Doacao
    from sigpdv.control.ficha import Ficha
    from sigpdv.control.abertura_fechamento_turno import AberturaFechamentoTurno
    from sigpdv.control.reducao_xz import ReducaoXZ
    from sigpdv.control.mov_caixa import MovCaixa
    from sigpdv import root_path as ROOT_PATH
    from datetime import datetime
    from ctypes import CDLL
    from sigpdv.tools.socat import Socat
    from sigpdv.thread.th import BackgroundTask
    from sigpdv.tools.network import get_ip
    Log.info('IP Caixa: %s' % get_ip())
    from sigpdv.tools.validar import validar_cpf_cnpj
    from sigpdv.tools.encoding import smart_str
    from sigpdv import LISTA_APPS, MAIN_APP
    from sigpdv.update.update import SigFrenteUp
    from sigpdv.sigrestaurante.controle_mesa import ControleMesa
    from sigpdv.nepos.nepos import Nepos
    from sigpdv.web_sig.websig_sql import CadWebsig
    from sigpdv.pdv_config import TIPOS_TECLADO
    os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'modprobe pcspkr'))
    CadWebsig()
except Exception as e:
    Log.error(str(e))
    app = QtGui.QApplication(sys.argv)
    message_dlg(None, MSG_ERROR, 'A Aplicação será encerrada', str(e))
    os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
    sys.exit()

K1 = ord('1') if not config['usarcodificado'] else config['bt_num_1']
K2 = ord('2') if not config['usarcodificado'] else config['bt_num_2']
K3 = ord('3') if not config['usarcodificado'] else config['bt_num_3']
K4 = ord('4') if not config['usarcodificado'] else config['bt_num_4']
K5 = ord('5') if not config['usarcodificado'] else config['bt_num_5']
K6 = ord('6') if not config['usarcodificado'] else config['bt_num_6']
K7 = ord('7') if not config['usarcodificado'] else config['bt_num_7']
K8 = ord('8') if not config['usarcodificado'] else config['bt_num_8']
K9 = ord('9') if not config['usarcodificado'] else config['bt_num_9']
K0 = ord('0') if not config['usarcodificado'] else config['bt_num_0']
KDICT = {
    K1: '1',
    K2: '2',
    K3: '3',
    K4: '4',
    K5: '5',
    K6: '6',
    K7: '7',
    K8: '8',
    K9: '9',
    K0: '0'
}
TWOPLACES = Decimal(10) ** -2
TREEPLACES = Decimal(10) ** -3

# STATUS DO CUPOM
coupon_closed, coupon_opened, coupon_add_discount, coupon_add_surchage, coupon_add_payment, \
coupon_finish, mesa_closed, mesa_opened, mesa_transf, itens_mesa_transf, coupon_ficha, \
coupon_troca_ficha = range(12)

STATE_CUPOM_NAMES = {
    coupon_closed: 'CupomFechado',
    coupon_opened: 'CupomAberto',
    coupon_add_discount: 'CupomAddDesc',
    coupon_add_surchage: 'CupomAddAcresc',
    coupon_add_payment: 'CupomAddPgto',
    coupon_finish: 'CupomFim',
    mesa_closed: 'MesaFechada',
    mesa_opened: 'MesaAberta',
    mesa_transf: 'MesaTransf',
    itens_mesa_transf: 'ItensMesaTransf',
    coupon_ficha: 'FichaAberta'
}
# STATUS DO ITEM
item_none, item_add_quantity, item_add_discount, item_add_surchage, item_cancel, \
item_balanca = range(6)
STATE_ITEM_NAMES = {
    item_none: 'NoItem',
    item_add_quantity: 'ItemAddQtda',
    item_add_discount: 'ItemAddDesc',
    item_add_surchage: 'ItemAddAcresc',
    item_cancel: 'ItemCancel',
    item_balanca: 'ItemBalanca'
}
# STATUS DA TELA
in_login, in_menu, in_menu_fiscal, in_coupon, in_ficha, in_close_turn, in_ss, in_search_client, \
in_add_payment, in_dav, in_pve, in_importacao, in_item_menu_fiscal, in_end_sale, \
in_restaurante, in_update, in_troca_ficha = range(17)

STATE_SCREEN_NAMES = {in_login: 'EmLogin',
                      in_menu: 'EmMenu',
                      in_coupon: 'EmCupom',
                      in_ficha: 'EmFicha',
                      in_troca_ficha: 'EmTrocaFicha',
                      in_close_turn: 'EmFecharTurno',
                      in_ss: 'EmSangSup',
                      in_search_client: 'EmBuscaCliProd',
                      in_add_payment: 'EmPgto',
                      in_dav: 'EmDAV',
                      in_pve: 'EmPVE',
                      in_importacao: 'ExcImportacao',
                      in_end_sale: 'EmVendaFinalizada',
                      in_update: 'EmAtualizacao'}

# Stacked Actions Pages
stk_page_menu, stk_page_login, stk_page_coupom, stk_page_ficha, stk_page_payments, stk_page_ss, \
    stk_search_client, stk_dav_pre, stk_tef, stk_fiscal, stk_importacao, stk_restaurant, \
    stk_consulta_ch, stk_update = range(14)

# Items do menu
imenu_login, imenu_turno, imenu_venda, imenu_cancelar, imenu_suprimento, imenu_sangria, imenu_x, imenu_tef, \
imenu_atualizar_sis, imenu_faturar_dav, imenu_reload, imenu_turnoff = range(12)

# Metodos de Pagamento
pm_dinheiro, pm_cheque, pm_cartao, pm_ticket, pm_outros = range(5)

# Constantes Libera Restrições
LIB_LIMITE_CREDITO, LIB_MENSAGEM_CLIENTE, LIB_CHEQUE_VISTA = range(3)

try:
    if config['virtual_printer_active']:
        obj_socat = Socat()
        obj_socat.run(config['virtual_printer_srv'], config['ecf_port_tcp'])

except PrinterFileError as e:
    pass

except Exception as e:
    app = QtGui.QApplication(sys.argv)
    message_dlg(None, MSG_ERROR, str(e))
    sys.exit(-1)


@Log
def get_loja():
    res = CaixaCfgConfiguracao.get_by()
    if not res:
        class NoLoja(object):
            supcfc_loja = 0
            supcfc_cnpj = ''
            supcfc_cliente = ''
            supcfc_telefone = ''
            supcfc_inscricao_estadual = ''
            supcfc_numero = ''
            supcfc_certificado_conteudo = ''
            supcfc_chave_conteudo = ''
            supcfc_data_validade = ''
            supcfc_proprietario_certificado = ''
            supcfc_csc = ''
            supcfc_regime_tributario = ''
            supcfc_estado = ''
            supcfc_endereco = ''
            supcfc_qtd_conexoes_nfce = 20
            supcfc_timeout_conexoes_nfce = 10
            supcfc_incio_contingencia = ''
            supcfc_fim_contingencia = ''

        res = NoLoja()

    return res


class NotClient(object):
    name = ''
    supcli_tipo_pessoa = 'F'
    supcli_cnpj_cpf = ''
    supcli_endereco = ''
    supcli_cidade = ''
    supcli_status = ''
    supcli_codigo_interno = ''
    supcli_insc_estadual = ''
    supcli_total_em_aberto = 0
    supcli_limite_compras = 0
    supcli_cod_municipio = 0
    supcli_insc_estadual = ''
    supcli_bairro = ''
    supcli_cidade = ''
    supcli_estado = ''
    supcli_endereco_numero = ''
    supcli_complemento = ''
    supcli_cep = ''
    supcli_telefone = ''
    supcli_rota = 0
    supcli_email = ''


class PdvWindow(QtGui.QDialog):
    AddItensMode, PaymentMode, CouponClosed, CouponOpened = range(4)
    def get_data_atualizacao(self):
        try:
            arquivo = open('zdata.txt', 'r')
            self.data_atualizacao = arquivo.readline()
            arquivo.close()
        except:
            arquivo = open('zdata.txt', 'wr')
            arquivo.write('00/00/0000 00:00')
            arquivo.close()
            self.data_atualizacao = '00/00/0000 00:00'

    def set_data_atualizacao(self):
        data = datetime.now()
        data = '%02d/%02d/%d %02d:%02d' % (data.day, data.month, data.year, data.hour, data.minute)
        arquivo = open('zdata.txt', 'wr')
        arquivo.write(data)
        arquivo.close()
        self.data_atualizacao = data


    def __init__(self, parent=None, init_msg=[], status_conexao=False):
        self.QtGui = QtGui
        self.QtGui.QDialog.__init__(self, parent)
        self.ui = Ui_PdvWindow()
        self.ui.setupUi(self)
        self.conectado = status_conexao
        self.remove_accents = remove_accents # declarado para o gertec consegui usar esta func.
        self.verifica_hora_servidor_sefaz = verifica_hora_servidor_sefaz
        self.teclado = False
        self.linhas_display = 0 # numero de linha que possui o display do teclado
        self.vr_doacao = Decimal(0)
        self.vr_ficha = Decimal(0)
        self.docao_bloqueio = False # bloqueia os botoes ao confirmar a doacao na tela de pagamento
        if config['usarcodificado'] and config['usar_display'] and config['teclado_selecionado']:
            try:
                self.teclado = Teclado(self, TIPOS_TECLADO.get(config['teclado_selecionado']))
                self.teclado.limpar_display() # limpa display
            except Exception as e:
                self.teclado = False
                Log.info('Erro ao iniciar teclado reduzido: %s' % str(e))

        self.nfce_qr_code = ''
        self.limpa_display = False
        self.caixa_id = config['caixa']
        self.obj_mov_capa = MovCapa(self)
        self.verifica_z_emitido()
        self.estacionamento = False
        self.msg_estacionamento = ''
        self.codigo_estacionamento = ''
        self.atualizando_total_em_aberto = False
        self.consulta_saldo = False
        self.get_data_atualizacao()
        self.desconto_total_itens = Decimal(0).quantize(TWOPLACES)
        self._capas_gerencial = [] # registra os cupons gerenciais da nota(DAV)
        # desconto promocoes
        self.desconto_promo = Decimal(0).quantize(TWOPLACES)
        self.list_itens_desconto_promo = []
        self.dict_mov_corpos_socio = {}
        self.dict_itens_desconto_funcionario = {}
        self.desconto_funcionario = Decimal(0)
        self.desconto_socio_torcedor = Decimal(0)
        self.registrei_desconto = False # controle para add_payment
        self.ler_arq_socio_torcedor = ler_arq_socio_torcedor
        self.ficha = False
        self.obs_produto = '' # observacao dos itens de producao propria na ficha
        self.ui.lbl_data_entrega_ficha.setVisible(False)
        self.ui.lbl_cliente_ficha.setVisible(False)
        self.ui.edtDataEntregaFicha.setVisible(False)
        self.ui.edtClienteFicha.setVisible(False)

        self.ui.lblCliente.setText('Cliente')
        self.ui.lblImgCliente.setPixmap(QtGui.QPixmap(
                     ":/16x16/art/scalable/actions/contact-new.svg"))

        if config['virtual_printer_active'] and not obj_socat.checkProcess():
            obj_socat.run(config['virtual_printer_srv'], config['ecf_port_tcp'])

        # ATALHOS
        @Log
        def get_key_name(key):
            try:
                name = QtCore.Qt.Key(key).name[4:]
            except Exception as e:
                message_dlg(self, MSG_ERROR, 'Verifique no PDVCONFIG se todas as teclas do '
                                           + 'teclado codificado estao configuradas corretamente')
                return ''

            if name == 'Comma':
                name = ','
            if name == 'Period':
                name = '.'
            return name

        # definicao dos objetos
        self.obj_cupom_fiscal = CupomFiscal(self)
        self.obj_cor_nsu = CaixaCorNsu
        self.obj_abertura_fechamento_turno = AberturaFechamentoTurno(self)
        self.obj_cad_promo_capa = CadPromoCapa(self)
        self.obj_cad_promo1 = CadPromo1(self)
        self.obj_cad_promo2 = CadPromo2(self)
        self.obj_cad_promo3p = CadPromo3p(self)
        self.obj_cad_promo3f = CadPromo3f(self)
        self.obj_cad_promo4 = CadPromo4(self)
        self.obj_cad_promo5p = CadPromo5p(self)
        self.obj_cad_promo5f = CadPromo5f(self)
        self.obj_cad_promo6 = CadPromo6(self)
        self.obj_cad_promo7 = CadPromo7(self)
        self.obj_cad_funcionarios = Cadfuncionarios(self)
        self.obj_nepos = Nepos(self)
        self.obj_fiscal_printer = FiscalPrinterHelper()
        self.obj_fiscal_printer.parent = self
        self.obj_restricao_item = CaixaCadResticaoItem
        self.obj_fingertech = Fingertech()
        self.obj_fingertech.arq_log = '/tmp/log_biometria.txt'
        self.obj_fingertech.arq_biometria = '/tmp/biometria.txt'
        self.obj_doacao = Doacao(self)
        self.obj_ficha = Ficha(self)
        self.obj_parametros = CaixaCfgParametros
        self.obj_impressoras = CadComandosImpressoras(self)

        self.loja = get_loja()
        self.get_loja = get_loja
        self.emitir_contingencia = False

        self.get_key_name = get_key_name
        self.exportando_nota_nro = False  # Controle na exportacao e cacelar a nota no mesmo momento

        usar_codificado = config.get('usarcodificado', False)
        self.setWindowFlags(QtCore.Qt.FramelessWindowHint)


        # RESOLUÇÃO DA TELA
        if self.loja.supcfc_estado not in ('DF', 'df'):
            self.ui.mainMenu.takeItem(12)
            # esconde o menu Faturar DAV quando nao etiver usando "Imprimir DAV"
            if not config['imprimir_dav']:
                self.ui.mainMenu.item(imenu_faturar_dav).setHidden(True)

        if QtGui.QApplication.desktop().screenGeometry().width() <= 1024:
            self.ui.frmLeft.setVisible(False)
            self.ui.logo_foot.setVisible(True)
            self.ui.frmSubtotal_2.setVisible(True)
            self.ui.lblSubTotal_2.setVisible(True)
        else:
            self.ui.frmLeft.setVisible(True)
            self.ui.frmSubtotal_2.setVisible(False)
            self.ui.lblSubTotal_2.setVisible(False)
        if usar_codificado:
            self.ui.frameKeyboard.setMaximumSize(QtCore.QSize(0, 0))

        # acidiona atalhos aos botoes
        self.set_btn_shortcut(usar_codificado)
        if config['usar_ficha']:
            self.ui.edtCodigo.setMaxLength(15)

        #-------------------------------------------------------------------------------
        # PARAMETROS DO INI
        #-------------------------------------------------------------------------------
        self._font = QtGui.QFont()
        self._font.setPointSize(config['font_grid'])
        self._font.setKerning(True)
        self.cnpj_cpf_cheque = ''
        self.get_cnpj_empresa()
        self._balanca = get_balanca()
        self._habilitar_mnu_fiscal = False
        self.z_pendente = False
        # Tempo Ocioso
        self._limpar_info = 0
        self._tempo_ocioso = 0
        self._time_offline = 0
        self._tot_icms = Decimal('0')
        self._tot_isqn = Decimal('0')
        self.barcode_active = config['itens_scan_active']
        scanner_timer = QtCore.QTimer(self)
        scanner_timer.timeout.connect(self._active_scanner_timer)
        self._timer = scanner_timer
        self.desbloqueia_importacao()
        self.tdm = False # indica se está emitindo um arquivo TDM

        # controla a abertura de turno, caso aperte enter rapidamente varias vezes no login
        self.bloqueia_campo_senha = False

        # Variaveis utilizadas para controlar os itens que são vendido em promocao por cupom
        self._itens_promocao = False
        self.promocoes_aplicadas = {}
        self._id_promocao = None
        self._primeiro_desconto_qtd = False
        self._vlr_desc_promocao = None

        if self.barcode_active:
            try:
                self.barcode = BarcodeReader(brand='metrologic', model='MC630',
                                             device=config['scanner'])
                self.barcode._port.timeout = 0.001
            except:
                init_msg.append(Exception('Verifique a conexão com o Leitor de Código de Barras'))
                self.barcode_active = False

        self._recuperar_cupom = False

        sair = False
        if init_msg:
            str_msg = ''
            for msg in init_msg:
                Log.info('%r' % msg)
                str_msg += '-> %s\n' % msg[0]
                if type(msg) == PendingReduceZ:
                    self.z_pendente = True
                if type(msg) == CouponOpenError:
                    Log.info('Cupom Aberto')
                if type(msg) == EcfDateError:
                    sair = False
            message_dlg(self, MSG_INFO, 'Avisos sobre a inicialização do sistema.', str_msg)

        self.par_sangria_x_z_suprimento = config['sangria_x_z_suprimento']
        # TIMERS
        timer_end_sale = QtCore.QTimer(self)
        timer_end_sale.timeout.connect(self._timer_end_sale_exec)
        self._timer_end_sale = timer_end_sale
        #-------------------------------------------------------------------------------
        # ******* NOVAS ****************************************************************
        self.KTEXT = ''
        self.info_anim = Animation(self.ui.lblInfo, 'pos')  # Animação do Texto de Informação
        self.font = QtGui.QFont("Ubuntu", 18, QtGui.QFont.Bold)  # Fonte usada em alguns widgets da
        # Aplicação
        self.ui.lblInfo.setFont(self.font)  # Defini a fonte padrão p/ o Label de Informação
        self._keyboard_lock = False
        self.bloqueia_tecla = False # controle de bloqueio de teclas
        self.bloqueia_retorno = False # Controle de bloqueio do ESC no fim da venda, estava sendo chamado varias vezes pelo operador
        self._executando_importacao = False
        paf_ecf = get_info_pafecf('sw.ini')
        self.ui.lblInfoVersao.setText(u'SigPDV: %s' % get_version())

        self._close_window = True
        self.ui.stackedMenus.setCurrentIndex(0)
        self.emitindo_dav = False # controle para nao deixar emitir dois davs ao mesmo tempo
        self._IdDav = False
        self._IdPve = False
        self.comanda_id = False
        self._IdsComandas = []
        self._consulta_cheque = False
        self._pves_list = False
        self._IdMesa = False
        self.vlr_item_dav = False
        self._PveCancelamento = False
        self._DavPgtoManual = False
        self._DavFechamentosIter = False
        self._count = 10
        self._wait_logoff = False
        self._vlrReceber = Decimal(0)  # valor a receber
        self.EOL_DELIMIT = '\r'
        self.out = ''
        self.a = 0
        self.retries = 10
        self._tab_order = []
        self._set_controls_default_values(ss_visible=True)
        self._CONTROL_FOCUS = None
        self.cliente = NotClient
        self.cliente_total_aberto = 0.0 # total em aberto atulizado em tempo real durante as vendas
        self._coupon = False
        self._info_ticket = Decimal(0)
        self._info_dinheiro = Decimal(0)
        self._info_outros = Decimal(0)
        self._info_cartao = Decimal(0)
        self._info_desc_total = Decimal(0)
        self._info_cheque = Decimal(0)
        self._info_desc_cupom = Decimal(0)
        self._payment_receipt = []
        self._payment_receipt_g = []
        self._libera_restricao = False # id do operador que liberou a restricao
        self._libera_desconto = False # id do operador que liberou o desconto
        self._close_window = True
        self.cancel_pinpad = False
        self.btn_cancel_info_dialog = False
        self._info_dialog = InfoDialog(parent=self)
        self._indicador_desc_acre = ''
        self._comanda_ativa = False
        self._flag_saldo_super = False
        self._state_coupon_old = False

        # Controle socio torcedor
        self.socio_torcedor = False
        self.tela_socio_torcedor = False
        self._tef_nsu_socio_torcedor = False

        # controla o cancelmento do tef para nao cancelar uma transacao concluida e o Socio Torc.
        self.controle_cancelamento_tef = False

        self._usa_serie = False

        self._aceita_turno = False

        self.mov_caixa = False  # ABERTURA DO CAIXA

        self.ui.stackedActions.setCurrentIndex(stk_page_login)
        self.user_logged = False
        self._operador_codigo = None
        self._operador_id = None
        self._message_in_coupon = None
        self._nome_vendedor = ''
        self.vendedor_codigo = 0
        self.vendedor_comanda_ids = []
        self._state_screen = in_login
        self._previous_state = in_login
        self._conn = sess
        self._state_coupon = coupon_closed
        self.consulta_produto = False

        self.ui.dateChVcto.textChanged.connect(self.on_change_dateChVcto)
        self._date_coupon = date.today()

        self._state_item = item_none
        self._item_discount = Decimal(0)
        self._item_quantity = Decimal(1)
        self._coupon_discount = Decimal(0)
        self._coupon_surchage = Decimal(0)
        self.last_item = 0
        self._coupon_amount = Decimal(0)
        self._coupon_subtotal = Decimal(0)
        self._coupon_payment_method = False
        self._coupon_totalize = None
        self._coupon_id = 0
        self._payment_amount = Decimal(0)

        self._menu_atualiza = False

        self.cliente_identificado = False
        self.acao_pendente_z = False

        # LBL INFO
        self.ui.lblNumEmpresa.setText(
            '%.4d - %s' % (self.loja.supcfc_loja, self.loja.supcfc_cliente))
        self.ui.lblNumCaixa.setText(u'%03d <b>Fechado' % self.caixa_id)
        #-------------------------------------------------------------------------------
        # Cupom
        #-------------------------------------------------------------------------------
        self.mov_capa = False
        self.mov_corpos = []
        self.mov_fechamentos = False
        self._TOTALIZE_OPEN = None

        self._desc_item = Decimal('0.0')
        self._item_as_desc = False
        self._qtda_item = Decimal('1')

        self.ui.tableItens.itemPressed.connect(self._returnPressed_tableItens)
        self.ui.tableItensFicha.itemPressed.connect(self._returnPressed_tableItensFicha)
        self.ui.tableBuscaProduto.itemPressed.connect(self._returnPressed_tableBuscaProduto)
        self.ui.tableBuscaCliente.itemPressed.connect(self._returnPressed_tableBuscaCliente)
        self.ui.tableBuscaDav.itemPressed.connect(self._returnPressed_tableBuscaDav)
        self.ui.tableBuscaPve.cellEntered.connect(self._returnPressed_tableBuscaPve)

        # RESTAURANTE
        self.obj_controle_mesa = ControleMesa()
        self.ui.tableMesa.itemPressed.connect(self.obj_controle_mesa._returnPressed_tableMesa)
        self.ui.tableItensMesa.itemPressed.connect(self.obj_controle_mesa._returnPressed_tableItensMesa)
        self.ui.tableMesa.currentCellChanged.connect(self.obj_controle_mesa._cell_activated_tableMesa)

        # SIGTEF
        self._texto_comprovante = None

        self.ui.tableSigTefMenu.cellEntered.connect(self._returnPressed_tableSigTefMenu)
        self.sig_tef = SigTefI(self)
        self.desconto_socio_calculado = False
        self.socio_tef = False
        self.cartao_tef = False
        self.ticket_tef = False
        self.recarga_tef = False

        # MENUS
        self.ui.mainMenu.entered.connect(self._returnPressed_mainmenu)
        self.ui.menuTef.entered.connect(self._returnPressed_menutef)

        self.ui.mainMenu.itemDoubleClicked.connect(self._returnPressed_mainmenu)
        self.ui.menuTef.itemDoubleClicked.connect(self._returnPressed_menutef)

        # KeysNumPad
        self.ui.btnKEY_0.clicked.connect(self.KEY_0_Click)
        self.ui.btnKEY_1.clicked.connect(self.KEY_1_Click)
        self.ui.btnKEY_2.clicked.connect(self.KEY_2_Click)
        self.ui.btnKEY_3.clicked.connect(self.KEY_3_Click)
        self.ui.btnKEY_4.clicked.connect(self.KEY_4_Click)
        self.ui.btnKEY_5.clicked.connect(self.KEY_5_Click)
        self.ui.btnKEY_6.clicked.connect(self.KEY_6_Click)
        self.ui.btnKEY_7.clicked.connect(self.KEY_7_Click)
        self.ui.btnKEY_8.clicked.connect(self.KEY_8_Click)
        self.ui.btnKEY_9.clicked.connect(self.KEY_9_Click)
        self.ui.btnKEY_COMMA.clicked.connect(self.KEY_COMMA_Click)
        self.ui.btnKEY_BACKSPACE.clicked.connect(self.KEY_BACKSPACE_Click)
        ## KeysItens
        self.ui.btnKEY_QTDA.clicked.connect(self.KEY_QTDA_Click)
        self.ui.btnKEY_QTDA.clicked.connect(self.KEY_QTDA_Click)
        self.ui.btnKEY_SUPORTE.clicked.connect(self.KEY_SUPORTE_Click)
        self.ui.btnKEY_CANCEL_ITENS.clicked.connect(self.KEY_CANCEL_ITENS_Click)
        ## KeysActions
        self.ui.btnKEY_ESCAPE.clicked.connect(self.KEY_ESCAPE_Click)
        self.ui.btnKEY_RETURN.clicked.connect(self.KEY_RETURN_Click)
        self.ui.btnKEY_LEFT.clicked.connect(self.KEY_LEFT_Click)
        self.ui.btnKEY_UP.clicked.connect(self.KEY_UP_Click)
        self.ui.btnKEY_RIGHT.clicked.connect(self.KEY_RIGHT_Click)
        self.ui.btnKEY_DOWN.clicked.connect(self.KEY_DOWN_Click)
        ## KeysClienteVendedor
        self.ui.btnKEY_CLIENTE.clicked.connect(self.KEY_CLIENTE_Click)
        self.ui.btnKEY_VENDEDOR.clicked.connect(self.KEY_VENDEDOR_Click)
        ## KeysFinalizadoras
        self.ui.btnKEY_DINHEIRO.clicked.connect(self.KEY_DINHEIRO_Click)
        self.ui.btnKEY_CHEQUE.clicked.connect(self.KEY_CHEQUE_Click)
        self.ui.btnKEY_CARTAO.clicked.connect(self.KEY_CARTAO_Click)
        self.ui.btnKEY_TICKET.clicked.connect(self.KEY_TICKET_Click)
        self.ui.btnKEY_ESTACIONAMENTO.clicked.connect(self.KEY_ESTACIONAMENTO_Click)
        self.ui.btnKEY_DOACAO.clicked.connect(self.KEY_DOACAO)
        self.ui.btnKEY_OUTROS.clicked.connect(self.KEY_OUTROS_Click)
        ## KeysAcrescimoDesconto
        self.ui.btnKEY_DESCONTO.clicked.connect(self.KEY_DESCONTO_Click)
        self.ui.btnKEY_ACRESCIMO.clicked.connect(self.KEY_ACRESCIMO_Click)
        ## KeysOutras
        self.ui.btnKEY_COMANDA.clicked.connect(self.KEY_COMANDA_Click)
        self.ui.btnKEY_RECARGA_CEL.clicked.connect(self.KEY_RECARGA_CEL_Click)
        self.ui.btnKEY_CONSULTA.clicked.connect(self.KEY_CONSULTA_Click)
        self.ui.btnKEY_MENSAGEM.clicked.connect(self.KEY_MENSAGEM_Click)
        self.ui.btnKEY_ImprimeDav.clicked.connect(self.KEY_IMPRIME_DAV)
        self.ui.btnKEY_SOCIO_TORCEDOR.clicked.connect(self.KEY_SOCIO_TORCEDOR_Click)
        self.ui.btnKEY_CANCELAR_NOTA.clicked.connect(self.KEY_CANCELAR_NOTA)
        self.ui.btnKEY_MENU.clicked.connect(self.KEY_MENU_Click)
        self.ui.btnKEY_BALANCA.clicked.connect(self.KEY_BALANCA_Click)
        self.ui.btnKEY_DAV.clicked.connect(self.KEY_DAV_Click)
        self.ui.btnKEY_PRE.clicked.connect(self.KEY_PRE_Click)
        self.ui.btnKEY_FICHA.clicked.connect(self.KEY_FICHA)
        self.ui.btnKEY_TROCA_FICHA.clicked.connect(self.KEY_TROCA_FICHA)
        ## RESTAURANTE
        self.ui.btnKEY_FecharMesaRes.clicked.connect(self.obj_controle_mesa.KEY_FecharMesaRes_Click)
        self.ui.btnKEY_VenderItemRes.clicked.connect(self.obj_controle_mesa.KEY_VenderItemRes_Click)
        self.ui.btnKEY_AbrirMesaRes.clicked.connect(self.obj_controle_mesa.KEY_AbrirMesaRes_Click)
        self.ui.btnKEY_ConferMesaRes.clicked.connect(self.obj_controle_mesa.KEY_ConferMesaRes_Click)
        self.ui.btnKEY_TransfMesaRes.clicked.connect(self.obj_controle_mesa.KEY_TransfMesaRes_Click)
        self.ui.btnKEY_TransfItensMesaRes.clicked.connect(self.obj_controle_mesa.KEY_TransfItensMesaRes_Click)

        # linka os campos a funcao de mudanca de texto
        if self.teclado:
            self.ui.spinPgtoDinheiro.valueChanged[str].connect(self.on_text_changed)
            self.ui.edtCodigo.textChanged.connect(self.on_text_changed)
            self.ui.edtUser.textChanged.connect(self.on_text_changed)
            self.ui.edtPwd.textChanged.connect(self.on_text_changed)
            self.ui.edtBuscaCliente.textChanged.connect(self.on_text_changed)
            self.ui.spinQtda.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinDecItemValor.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinCartaoValor.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinValorTicket.valueChanged[str].connect(self.on_text_changed)
            self.ui.edtOCodCliente.textChanged.connect(self.on_text_changed)
            self.ui.spinOValor.valueChanged[str].connect(self.on_text_changed)
            self.ui.edtChCliente.textChanged.connect(self.on_text_changed)
            self.ui.spinChNroBanco.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinChNroAgencia.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinChNroCC.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinChNroCheque.valueChanged[str].connect(self.on_text_changed)
            self.ui.edtChDocEmit.textChanged.connect(self.on_text_changed)
            self.ui.dateChVcto.textChanged.connect(self.on_text_changed)
            self.ui.spinChValor.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinSSDinheiro.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinSSCheque.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinSSCartao.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinSSTickets.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinSSOutros.valueChanged[str].connect(self.on_text_changed)
            self.ui.spinSigTefValor.valueChanged[str].connect(self.on_text_changed)
            self.ui.edtSigTefTexto.textChanged.connect(self.on_text_changed)

        # linka os comboboxs a mudanca de indicemensagem_do_foco
        self.ui.cbTipoDesc.currentIndexChanged.connect(self.on_index_change)
        self.ui.cbTipoTef.currentIndexChanged.connect(self.on_index_change)
        self.ui.cbChTipoCartao.currentIndexChanged.connect(self.on_index_change)
        self.ui.cbTipoTicket.currentIndexChanged.connect(self.on_index_change)
        self.ui.cbFinTicket.currentIndexChanged.connect(self.on_index_change)
        self.ui.cbOTipoPgto.currentIndexChanged.connect(self.on_index_change)
        self.ui.cbChTipoCheque.currentIndexChanged.connect(self.on_index_change)
        self.ui.cbChProp.currentIndexChanged.connect(self.on_index_change)


        #-----------------------------------------------------------------------
        # AJUSTAR O WIGDET tableitens
        #-----------------------------------------------------------------------
        self.ui.menuTef.setEnabled(True)
        self.ui.mainMenu.setEnabled(True)
        self.ui.btnKEY_ImprimeDav.setEnabled(config['imprimir_dav'])
        self.btn_imprime_dav_down = False

        self._CURRENT_ROW = None
        self._SUBTOTAL = Decimal('0.0')
        tableItens = self.ui.tableItens

        tableItens.hideColumn(7)
        tableItens.hideColumn(8)
        self.ui.tableItensFicha.hideColumn(6)
        self.ui.tableItensFicha.hideColumn(7)
        self.ui.tableItensFicha.hideColumn(8)
        self.ui.tableItensFicha.hideColumn(9)
        self.ui.tableItensFicha.hideColumn(10)
        self.ui.tableItensFicha.hideColumn(11)

        # AJUSTAR O WIGDET TABLE DAV
        tableDav = self.ui.tableBuscaDav

        # Definir Largura das Colunas
        tableDav.setColumnWidth(0, 148)  # Codigo
        tableDav.setColumnWidth(1, 140)  # Data
        tableDav.setColumnWidth(2, 320)  # Nome
        tableDav.setColumnWidth(3, 120)  # Valor

        # AJUSTAR O WIGDET TABLE PVE
        tablePve = self.ui.tableBuscaPve

        # Definir Largura das Colunas
        tablePve.setColumnWidth(0, 148)  # Codigo
        tablePve.setColumnWidth(1, 470)  # Nome
        tablePve.setColumnWidth(2, 120)  # Valor

        # AJUSTAR O WIGDET MENU SIGTEF
        tableMenuTef = self.ui.tableSigTefMenu

        # Definir Largura das Colunas
        tableMenuTef.setColumnWidth(0, 200)  # código
        tableMenuTef.setColumnWidth(1, 539)  # nome do cliente
        self.ui.frmSerie.setVisible(False)

        # ICONES
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("gui/ui/images/16x16/media_play_green.png"),
                       QtGui.QIcon.Disabled, QtGui.QIcon.Off)
        self.icon_media_play_green = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("gui/ui/images/16x16/replace2.png"), QtGui.QIcon.Disabled,
                       QtGui.QIcon.Off)
        self.icon_replace = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("gui/ui/images/16x16/media_stop_red.png"),
                       QtGui.QIcon.Disabled, QtGui.QIcon.Off)
        self.icon_media_stop_red = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("gui/ui/images/16x16/nav_plain_green.png"),
                       QtGui.QIcon.Disabled, QtGui.QIcon.Off)
        self.icon_green = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("gui/ui/images/16x16/nav_plain_blue.png"),
                       QtGui.QIcon.Disabled, QtGui.QIcon.Off)
        self.icon_blue = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("gui/ui/images/16x16/nav_plain_red.png"), QtGui.QIcon.Disabled,
                       QtGui.QIcon.Off)
        self.icon_red = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("gui/ui/images/16x16/hourglass.png"), QtGui.QIcon.Disabled,
                       QtGui.QIcon.Off)
        self.icon_wait = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("gui/ui/images/16x16/arrow_right_green.png"),
                       QtGui.QIcon.Disabled,
                       QtGui.QIcon.Off)
        self.icon_arrow = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/images/16x16/add2.png"), QtGui.QIcon.Disabled,
                       QtGui.QIcon.Off)
        self.icon_add = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/images/16x16/check2.png"), QtGui.QIcon.Disabled,
                       QtGui.QIcon.Off)
        self.icon_check = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/images/16x16/red_minus.png"), QtGui.QIcon.Disabled,
                       QtGui.QIcon.Off)
        self.icon_minus = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/images/16x16/delete2.png"), QtGui.QIcon.Disabled,
                       QtGui.QIcon.Off)
        self.icon_delete = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("gui/ui/images/16x16/nav_plain_green.png"),
                       QtGui.QIcon.Disabled, QtGui.QIcon.Off)
        self.icon_round_green = icon

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("gui/ui/images/16x16/nav_plain_red.png"), QtGui.QIcon.Disabled,
                       QtGui.QIcon.Off)
        self.icon_round_red = icon

        #-----------------------------------------------------------------------
        # VARIAVÉIS PARA DEFINIR ESTADO PARA OS WIDGETS E CUPOM
        #-----------------------------------------------------------------------

        self.check_button_status()
        self.status_conexao_supervisor = status_conexao
        self.thread_exportacao = exportacao(self, config['time_export'])
        checar_conexao(self)
        QtCore.QObject.connect(self, QtCore.SIGNAL('checkServer(bool)'),
                               self.set_status_conexao_servidor)

        # IMPORTAÇÃO AUTOMÁTICA NO ÍNICIO DA APLICAÇÃO
        # Todo: caso o cupom estiver aberto nao faz importacao
        if True:
            try:
                self._set_screen_state(False)
                self.teclado and self.teclado.envia_display('Tentando conectar no Supervisor ...')
                self.ui.stackedActions.setCurrentIndex(stk_importacao)
                QtGui.QApplication.processEvents()
                QtGui.QApplication.processEvents()
                QtCore.QTimer.singleShot(0, self.executar_importacao)
            except Exception as err:
                Log.info(str(err))
                self.parar_importacao(str(err))
        else:
            if self._state_screen == in_end_sale:
                self.KEY_ESCAPE_Click()
            else:
                self._set_screen_state(self._state_screen)
            self._lock_keyboard(False)
            self._executando_importacao = False

        # TIMER PRINCIPAL
        timer_principal = QtCore.QTimer(self)
        timer_principal.timeout.connect(self._timer_principal)
        timer_principal.start(1000)
        self.timer_fim_de_venda = QtCore.QTimer(self)
        self.timer_fim_de_venda.timeout.connect(self.thread_tela_final)

        self.cmb_disable()
        self.spin_step()

    def set_btn_shortcut(self, usar_codificado):
        '''
        Adiciona atalhos para os botoes, para teclado codificado e teclado normal
        '''
        if usar_codificado:
            self.ui.btnKEY_COMMA.setShortcut(self.get_key_name(config['bt_num_.']))
            self.ui.btnKEY_BACKSPACE.setShortcut(self.get_key_name(config['bt_limpar']))
            self.ui.btnKEY_QTDA.setShortcut(self.get_key_name(config['bt_quantidade']))
            self.ui.btnKEY_CANCEL_ITENS.setShortcut(self.get_key_name(config['bt_cancelar_item']))
            self.ui.btnKEY_CONSULTA.setShortcut(self.get_key_name(config['bt_consulta']))
            self.ui.btnKEY_BALANCA.setShortcut(self.get_key_name(config['bt_balanca']))
            self.ui.btnKEY_ESCAPE.setShortcut(self.get_key_name(config['bt_sair']))
            self.ui.btnKEY_UP.setShortcut(self.get_key_name(config['bt_cima']))
            self.ui.btnKEY_DOWN.setShortcut(self.get_key_name(config['bt_baixo']))
            self.ui.btnKEY_RIGHT.setShortcut(self.get_key_name(config['bt_direita']))
            self.ui.btnKEY_LEFT.setShortcut(self.get_key_name(config['bt_esquerda']))
            self.ui.btnKEY_VENDEDOR.setShortcut(self.get_key_name(config['bt_vendedor']))
            self.ui.btnKEY_CLIENTE.setShortcut(self.get_key_name(config['bt_cliente']))
            self.ui.btnKEY_DINHEIRO.setShortcut(self.get_key_name(config['bt_dinheiro']))
            self.ui.btnKEY_CHEQUE.setShortcut(self.get_key_name(config['bt_cheque']))
            self.ui.btnKEY_CARTAO.setShortcut(self.get_key_name(config['bt_cartao']))
            self.ui.btnKEY_OUTROS.setShortcut(self.get_key_name(config['bt_outros']))
            self.ui.btnKEY_TICKET.setShortcut(self.get_key_name(config['bt_ticket']))
            self.ui.btnKEY_ESTACIONAMENTO.setShortcut(self.get_key_name(config['bt_estacionamento']))
            self.ui.btnKEY_DOACAO.setShortcut(self.get_key_name(config['bt_doacao']))
            self.ui.btnKEY_ACRESCIMO.setShortcut(self.get_key_name(config['bt_acrescimo']))
            self.ui.btnKEY_DESCONTO.setShortcut(self.get_key_name(config['bt_desconto']))
            self.ui.btnKEY_COMANDA.setShortcut(self.get_key_name(config['bt_comanda']))
            self.ui.btnKEY_MENU.setShortcut(self.get_key_name(config['bt_menu']))
            self.ui.btnKEY_CANCELAR_NOTA.setShortcut(self.get_key_name(config['bt_cancelarnota']))
            self.ui.btnKEY_DAV.setShortcut(self.get_key_name(config['bt_dav_pve']))
            self.ui.btnKEY_PRE.setShortcut(self.get_key_name(config['bt_dav_pve']))
            self.ui.btnKEY_SUPORTE.setShortcut(self.get_key_name(config['bt_suporte']))
            self.ui.btnKEY_RECARGA_CEL.setShortcut(self.get_key_name(config['bt_recarga_cel']))
            self.ui.btnKEY_MENSAGEM.setShortcut(self.get_key_name(config['bt_mensagem']))
            self.ui.btnKEY_SOCIO_TORCEDOR.setShortcut(self.get_key_name(config['bt_socio_torcedor']))

        else: # nao usar teclado codificado
            self.ui.btnKEY_COMMA.setShortcut("," )
            self.ui.btnKEY_BACKSPACE.setShortcut("Backspace")
            self.ui.btnKEY_QTDA.setShortcut("Ctrl+Q")
            self.ui.btnKEY_CANCEL_ITENS.setShortcut("Ctrl+C")
            self.ui.btnKEY_CONSULTA.setShortcut("Ctrl+S")
            self.ui.btnKEY_BALANCA.setShortcut("Ctrl+B")
            self.ui.btnKEY_ESCAPE.setShortcut("Esc")
            self.ui.btnKEY_UP.setShortcut("Up")
            self.ui.btnKEY_DOWN.setShortcut("Down")
            self.ui.btnKEY_RIGHT.setShortcut("Right")
            self.ui.btnKEY_LEFT.setShortcut("Left")
            self.ui.btnKEY_VENDEDOR.setShortcut("Ctrl+V")
            self.ui.btnKEY_CLIENTE.setShortcut("Ctrl+L")
            self.ui.btnKEY_DINHEIRO.setShortcut("Ctrl+D")
            self.ui.btnKEY_CHEQUE.setShortcut("Ctrl+H")
            self.ui.btnKEY_CARTAO.setShortcut("Ctrl+R")
            self.ui.btnKEY_OUTROS.setShortcut("Ctrl+O")
            self.ui.btnKEY_TICKET.setShortcut("Ctrl+K")
            self.ui.btnKEY_ESTACIONAMENTO.setShortcut("Ctrl+T")
            self.ui.btnKEY_DOACAO.setShortcut("Ctrl+D")
            self.ui.btnKEY_ACRESCIMO.setShortcut("Ctrl+A")
            self.ui.btnKEY_DESCONTO.setShortcut("Ctrl+E")
            self.ui.btnKEY_COMANDA.setShortcut("Ctrl+N")
            self.ui.btnKEY_MENU.setShortcut("Ctrl+M")
            self.ui.btnKEY_DAV.setShortcut("F2")
            self.ui.btnKEY_PRE.setShortcut("F3")
            self.ui.btnKEY_SUPORTE.setShortcut("Ctrl+Y")
            self.ui.btnKEY_RECARGA_CEL.setShortcut("Ctrl+R")
            self.ui.btnKEY_MENSAGEM.setShortcut("Ctrl+G")
            self.ui.btnKEY_ImprimeDav.setShortcut("Ctrl+I")
            self.ui.btnKEY_FICHA.setShortcut("Ctrl+C")
            self.ui.btnKEY_TROCA_FICHA.setShortcut("Ctrl+T")
            # mesmo da troca de ficha, mas nao da conflito pois nao ficam ativos ao mesmo tempo
            self.ui.btnKEY_SOCIO_TORCEDOR.setShortcut("Ctrl+T")

    def get_cnpj_empresa(self):
        self.cnpj_empresa = '' # cnpj da empresa que usa o Frente
        if self.loja:
            self.cnpj_empresa = str(self.loja.supcfc_cnpj)

        return True

    @Log
    def verifica_z_emitido(self):
        '''
         Verifica se o cupom Z do dia ja foi emitido para bloquear o frente
         :return: True se o Z do dia foi emitido, False se nao foi emitido
        '''
        capa = self.obj_mov_capa.search({'supcap_tipo': 'Z',
                                         'supcap_caixa': self.caixa_id,
                                         'supcap_data_cupom': datetime.now().date()})
        if capa:
            if isinstance(capa, list):
                capa = capa[0]

            Log.info('COO: %s' % capa.supcap_nro_cupom)
            return True

        return False

    def atualiza_valor_a_pagar(self, promo_din=False):
        '''
        :param promo_din: Usado apenas para promocao valida para dinheiro, para atualizar os valores
        quando o pagamento for parcial, nao valendo mais o desconto da promocao
        :return: 
        '''
        valor = u'%.2f' % (Decimal(self._coupon_amount - self.desconto_promo -
                                   self.desconto_socio_torcedor -self.desconto_funcionario))
        if (Decimal(self._coupon_amount - self.desconto_promo - self.desconto_socio_torcedor -
                        self.desconto_funcionario)) < 0 and not self.ficha:
            valor = u'0.0'

        self.ui.lblVlrTotalAPagar.setText(valor)
        if promo_din:
            self.atualiza_desconto()
            self.ui.lblVlrTotalR.setText(u'%.2f' % float(self.ui.lblVlrTotalAPagar.text()))
            self.ui.lblVlrReceber.setText(u'%.2f' % float(self.ui.lblVlrTotalAPagar.text()))

        return False

    def atualiza_desconto(self):
        self.ui.lblInfoDescTotalItens.setText(
                        u'%1.2f' % (Decimal(self.desconto_total_itens) +
                                    self.desconto_promo + self.desconto_socio_torcedor +
                                    self.desconto_funcionario))

        desc_acres = 1 if self._indicador_desc_acre == item_add_surchage else -1
        dif = Decimal(self.ui.lblInfoDescCapa.text()).quantize(TWOPLACES) * desc_acres + \
              Decimal(self.ui.lblInfoAcresTotalItens.text()).quantize(TWOPLACES) - \
              Decimal(self.ui.lblInfoDescTotalItens.text()).quantize(TWOPLACES)


        if dif < 0: # desconto
            self.ui.lbl_Desconto_Total.setText('Desconto\nTotal')
            dif = dif * -1
        elif dif == 0:
            self.ui.lbl_Desconto_Total.setText('Desconto\nTotal')
        else: # acrescimo
            self.ui.lbl_Desconto_Total.setText(u'Acréscimo\nTotal')

        self.ui.lblVlrDescontoTotal.setText( u'%1.2f' % (dif))
        return True

    def on_index_change(self, index):
        '''
        ao mudar o indice corrente do combobox, envia o valor corrente para o display
        :param index:
        :return:
        '''
        if not self.teclado or type(self._CONTROL_FOCUS) != QtGui.QComboBox:
            return False
        try:
            texto = self._CONTROL_FOCUS.currentText()
            self.teclado.envia_linha_display(texto, 2)
            return True
        except Exception as e:
            Log.error(str(e))

        return False

    def on_text_changed(self, val):
        '''
        Esta funcao captura o evento de escrita nos campos, cada letra que eh escrita
        o valor eh capturado e enviado para o display
        :param val: texto do campo
        :return: True se enviou e false se nao enviou para o display
        '''
        # deve-se colocar o nome do campo na lista para quando o valor do campo for alterado
        # e o foco nao estiver nele, seu valor nao apareca no display
        campos = ['edtCodigo', 'edtBuscaCliente','edtUser', 'edtPwd', 'spinPgtoDinheiro',
                  'spinQtda', 'spinDecItemValor', 'spinCartaoValor', 'spinValorTicket',
                  'edtOCodCliente', 'spinOValor', 'edtChCliente', 'spinChNroBanco', 'spinChNroAgencia',
                  'spinChNroCC', 'spinChNroCheque', 'edtChDocEmit', 'dateChVcto', 'spinChValor',
                  'spinSSDinheiro','spinSSCheque','spinSSCartao','spinSSTickets','spinSSOutros',
                  'spinSigTefValor', 'edtSigTefTexto', 'spinSigTefValor']

        if not self.teclado or self._CONTROL_FOCUS.objectName() not in campos:
            return False

        try:
            texto = str(val)
            linha = 2
            if not val:
                texto = ''

            if self._CONTROL_FOCUS.objectName() in ['spinPgtoDinheiro', 'spinCartaoValor',
                                                    'spinValorTicket', 'spinOValor', 'spinChValor']:
                linha = 4
            elif self._CONTROL_FOCUS.objectName() == 'edtPwd':
                linha = 4
                texto = ''.ljust(len(val),'*')

            elif type(self._CONTROL_FOCUS) == QtGui.QDoubleSpinBox:
                try:
                    texto = self._CONTROL_FOCUS.currentText()
                except:
                    pass

            if 'smak' in config['teclado_selecionado']:
                linha = 2

            if texto:
                self.teclado.envia_linha_display(texto, linha)
            else:
                self.teclado.limpar_linha(linha)

            return True

        except Exception as e:
            Log.error('on_text_changed(%s): %s' % (self._CONTROL_FOCUS.objectName(), str(e)))

        return False


    def check_button_status(self, check_payment_buttons=None):

        states_pm = [pm_dinheiro, pm_cheque, pm_cartao, pm_ticket, pm_outros]

        # KEYS ITENS
        KEY_QTDA = (self.ui.btnKEY_QTDA, [in_coupon, in_restaurante, in_ficha, in_troca_ficha],
                    [coupon_closed, coupon_opened, mesa_opened, mesa_closed, coupon_ficha],
                    [item_none], [])
        KEY_ACRESCIMO = (
            self.ui.btnKEY_ACRESCIMO, [in_coupon], [coupon_closed, coupon_opened], [item_none], [])
        KEY_DESCONTO = (
            self.ui.btnKEY_DESCONTO, [in_coupon, in_troca_ficha],
            [coupon_closed, coupon_opened, coupon_ficha], [item_none], [])
        KEY_CANCEL_ITENS = (
            self.ui.btnKEY_CANCEL_ITENS, [in_coupon, in_restaurante, in_ficha],
            [coupon_opened, mesa_opened,coupon_ficha],
            [item_none], [])
        KEY_CONSULTA = (
            self.ui.btnKEY_CONSULTA,
            [in_login, in_menu, in_menu_fiscal, in_coupon, in_restaurante, in_ficha],
            [coupon_closed, coupon_opened, coupon_finish, mesa_opened, mesa_closed, coupon_ficha],
            [item_none, item_balanca], [])

        # KEYS CLIENTE VENDEDOR
        KEY_CLIENTE = (
            self.ui.btnKEY_CLIENTE, [in_login, in_menu, in_menu_fiscal, in_coupon, in_restaurante],
            [coupon_closed, coupon_opened, mesa_opened, mesa_closed, coupon_finish], [item_none],
            [])
        KEY_VENDEDOR = (
            self.ui.btnKEY_VENDEDOR, [in_coupon], [coupon_closed, coupon_opened], [item_none], [])
        # KeysFinalizadoras
        KEY_DINHEIRO = (self.ui.btnKEY_DINHEIRO, [in_coupon, in_add_payment, in_ficha, in_troca_ficha],
                        [coupon_opened, coupon_add_payment, coupon_finish, coupon_ficha],
                        [item_none], [pm_dinheiro])

        KEY_CHEQUE = (self.ui.btnKEY_CHEQUE, [in_coupon, in_add_payment, in_ficha, in_troca_ficha],
                      [coupon_opened, coupon_add_payment, coupon_finish, coupon_ficha], [item_none],
                      [pm_cheque])
        KEY_ESTACIONAMENTO = (self.ui.btnKEY_ESTACIONAMENTO, [in_add_payment], [coupon_opened],
                              [item_none] ,[])
        KEY_DOACAO = (self.ui.btnKEY_DOACAO, [in_coupon], [coupon_closed], [item_none] ,[])
        KEY_CARTAO = (
            self.ui.btnKEY_CARTAO, [in_coupon, in_add_payment, in_ficha, in_troca_ficha],
            [coupon_opened, coupon_add_payment, coupon_finish, coupon_ficha], [item_none],
            [pm_cartao])
        KEY_TICKET = (
            self.ui.btnKEY_TICKET, [in_coupon, in_add_payment, in_ficha, in_troca_ficha],
            [coupon_opened, coupon_add_payment, coupon_finish, coupon_ficha], [item_none],
            [pm_ticket])

        KEY_OUTROS = (
            self.ui.btnKEY_OUTROS, [in_coupon, in_add_payment, in_ficha, in_troca_ficha],
            [coupon_opened, coupon_add_payment, coupon_finish, coupon_ficha], [item_none],
            [pm_outros])
        KEY_ESC = (self.ui.btnKEY_ESCAPE,
                   [in_login, in_menu_fiscal, in_coupon, in_close_turn, in_ss, in_search_client,
                    in_add_payment, in_dav, in_ficha,in_troca_ficha,
                    in_pve, in_menu, in_item_menu_fiscal, in_end_sale, in_restaurante],
                   [coupon_add_discount, coupon_add_surchage, coupon_opened, coupon_closed,
                    coupon_add_payment, coupon_finish, mesa_opened, mesa_closed, coupon_ficha],
                   [item_none, item_add_discount, item_add_surchage, item_cancel, item_add_quantity,
                    item_balanca,
                    item_balanca],
                   [pm_outros, pm_ticket, pm_cheque, pm_outros, pm_cartao])
        KEY_CONSULTA_CH = (self.ui.btnKEY_CONSULTA_CH, [in_coupon], [coupon_closed, coupon_opened],
                           [item_none], [])

        KEY_MENSAGEM = (self.ui.btnKEY_MENSAGEM, [in_coupon], [coupon_closed, coupon_opened],
                        [item_none], [])

        # KeysOutras

        KEY_CANCELAR_NOTA = (self.ui.btnKEY_CANCELAR_NOTA, [in_coupon], [coupon_opened],
                             [item_none], [])

        # KEYS ACRESCIMO/DESCONTO
        KEY_MENU = (self.ui.btnKEY_MENU,
                    [in_login, in_menu_fiscal, in_coupon, in_close_turn, in_ss, in_search_client,
                     in_add_payment, in_dav, in_pve, in_item_menu_fiscal, in_restaurante],
                    [coupon_closed, mesa_opened, mesa_closed], [item_none], [])
        if config['apenas_ficha']:
            KEY_MENU[1].append(in_ficha)

        KEY_BALANCA = (
            self.ui.btnKEY_BALANCA, [in_coupon], [coupon_closed, coupon_opened], [item_none], [])
        usar_dav = config['usardav']
        usar_pve = config['usarprevenda']
        usar_comanda = config['usarcomanda']
        usar_recarga_cel = config['usa_recarga_cel']
        usar_socio_torcedor = config['usa_socio_torcedor']
        usar_ficha = config['usar_ficha']
        KEY_DAV = (self.ui.btnKEY_DAV, [], [], [], [])
        KEY_PRE = (self.ui.btnKEY_PRE, [], [], [], [])
        KEY_COMANDA = (self.ui.btnKEY_COMANDA, [], [], [], [])
        KEY_RECARGA_CEL = (self.ui.btnKEY_RECARGA_CEL, [], [], [], [])
        KEY_SOCIO_TORCEDOR = (self.ui.btnKEY_SOCIO_TORCEDOR, [], [], [], [])
        KEY_FICHA = (self.ui.btnKEY_FICHA, [], [], [], [])
        KEY_TROCA_FICHA = (self.ui.btnKEY_TROCA_FICHA, [], [], [], [])

        if usar_recarga_cel:
            KEY_RECARGA_CEL = (
                self.ui.btnKEY_RECARGA_CEL, [in_coupon], [coupon_closed], [item_none], [])

        if usar_socio_torcedor:
            KEY_SOCIO_TORCEDOR = (self.ui.btnKEY_SOCIO_TORCEDOR, [in_coupon], [coupon_closed],
                                  [item_none], [])

        if usar_dav:
            KEY_DAV = (
                self.ui.btnKEY_DAV, [in_menu, in_coupon, in_close_turn, in_ss, in_search_client,
                                     in_add_payment, in_dav, in_pve], [coupon_closed], [item_none],
                [])
        if usar_pve:
            KEY_PRE = (
                self.ui.btnKEY_PRE, [in_menu, in_coupon, in_close_turn, in_ss, in_search_client,
                                     in_add_payment, in_dav, in_pve], [coupon_closed], [item_none],
                [])

        if usar_comanda:
            KEY_COMANDA = (
                self.ui.btnKEY_COMANDA, [in_coupon], [coupon_closed, coupon_opened], [item_none],
                [])

        if usar_ficha:
            KEY_FICHA = (self.ui.btnKEY_FICHA, [in_coupon, in_troca_ficha],
                         [coupon_closed, coupon_ficha], [item_none], [])
            KEY_TROCA_FICHA = (self.ui.btnKEY_TROCA_FICHA, [in_ficha], [coupon_closed, coupon_ficha],
                               [item_none], [])

        keys_itens = KEY_QTDA, KEY_ACRESCIMO, KEY_DESCONTO, KEY_CANCEL_ITENS, KEY_CONSULTA, \
                     KEY_CLIENTE, KEY_VENDEDOR, KEY_DINHEIRO, KEY_CHEQUE, KEY_CARTAO, KEY_TICKET, \
                     KEY_OUTROS, KEY_COMANDA, KEY_CONSULTA_CH, KEY_MENSAGEM, KEY_MENU, KEY_BALANCA, \
                     KEY_DAV, KEY_PRE, KEY_ESC, KEY_CANCELAR_NOTA, KEY_RECARGA_CEL, KEY_SOCIO_TORCEDOR, \
                     KEY_ESTACIONAMENTO, KEY_DOACAO, KEY_FICHA, KEY_TROCA_FICHA
        str_btn = ''
        for k in keys_itens:
            btn = k[0]
            state_screen = k[1]
            state_coupon = k[2]
            state_item = k[3]
            if (self._state_screen in state_screen) & (self._state_coupon in state_coupon) & (
                        self._state_item in state_item) and \
                self._state_screen != in_importacao:
                btn.setEnabled(True)
            else:
                btn.setEnabled(False)

            if self._state_screen == in_add_payment:
                if self.vr_doacao:
                    self.ui.btnKEY_DINHEIRO.setEnabled(True)
                    self.ui.btnKEY_CHEQUE.setEnabled(True)
                    self.ui.btnKEY_CARTAO.setEnabled(True)
                    self.ui.btnKEY_TICKET.setEnabled(True)
                    self.ui.btnKEY_OUTROS.setEnabled(True)

                if self._coupon_payment_method == pm_dinheiro:
                    self.ui.btnKEY_DINHEIRO.setEnabled(False)
                elif self._coupon_payment_method == pm_cheque:
                    self.ui.btnKEY_CHEQUE.setEnabled(False)
                elif self._coupon_payment_method == pm_cartao:
                    self.ui.btnKEY_CARTAO.setEnabled(False)
                elif self._coupon_payment_method == pm_ticket:
                    self.ui.btnKEY_TICKET.setEnabled(False)
                elif self._coupon_payment_method == pm_outros:
                    self.ui.btnKEY_OUTROS.setEnabled(False)

                if k == KEY_ESTACIONAMENTO and not config['config_nepos_active']:
                    btn.setEnabled(False)

            str_btn += '%s=%s, ' % (btn.objectName(), btn.isEnabled())

        # inabilita as teclas de pagamento quando estiver no tef
        campos_tef = ['tableSigTefMenu', 'menuTef', 'spinSigTefValor', 'edtSigTefTexto']
        if self._CONTROL_FOCUS and self._CONTROL_FOCUS.objectName() in campos_tef:
            Log.info('Teclas de pagamento bloqueadas: TEF')
            self.ui.btnKEY_DINHEIRO.setEnabled(False)
            self.ui.btnKEY_CHEQUE.setEnabled(False)
            self.ui.btnKEY_CARTAO.setEnabled(False)
            self.ui.btnKEY_TICKET.setEnabled(False)
            self.ui.btnKEY_OUTROS.setEnabled(False)
            self.ui.btnKEY_ESTACIONAMENTO.setEnabled(False)
            self.ui.btnKEY_DOACAO.setEnabled(False)

        if not self.user_logged:
            self.ui.btnKEY_MENU.setEnabled(False)

        return str_btn

    def on_change_dateChVcto(self, value):
        ''' Funcao que captura a mudanca de valor do campo 'dateChVcto'(Data de vencimento do cheque),
        qualcula quantos dias faltam para vencer e seta no campo 'diasChVcto'(Qtd. dias)
        :param value: novo valor do campo
        :return: True se calculou a qtd de dias, e False se nao calculou.
        '''
        if len(value) < 10:
            self.ui.diasChVcto.setText('0')
            return True

        hoje = datetime.now().date()
        dif = datetime.strptime(value, "%d/%m/%Y").date() - hoje
        if dif.days < 0:
            self.ui.diasChVcto.setText('0')
            return False
        self.ui.diasChVcto.setText(str(dif.days))
        return True


    @Log
    def cmb_disable(self, enabled=False):
        self.ui.cbSerie.setEnabled(enabled)
        self.ui.cbTipoDesc.setEnabled(enabled)
        self.ui.cbChTipoCheque.setEnabled(enabled)
        self.ui.cbChProp.setEnabled(enabled)
        self.ui.cbTipoTef.setEnabled(enabled)
        self.ui.cbChTipoCartao.setEnabled(enabled)
        self.ui.cbTipoTicket.setEnabled(enabled)
        self.ui.cbFinTicket.setEnabled(enabled)
        self.ui.cbOTipoPgto.setEnabled(enabled)
        self.ui.cbMFIntervalo.setEnabled(enabled)
        self.ui.cbMFDestino.setEnabled(enabled)
        self.ui.cbMFVendaPeriodo.setEnabled(enabled)
        self.ui.cbMFRegistro60I.setEnabled(enabled)
        self.ui.cbMFRegistro60R.setEnabled(enabled)
        self.ui.cbMFSubcategoria.setEnabled(enabled)
        return enabled

    @Log
    def spin_step(self, step=0):
        self.ui.spinQtda.setSingleStep(step)
        self.ui.spinDecItemValor.setSingleStep(step)
        self.ui.spinPgtoDinheiro.setSingleStep(step)
        self.ui.spinChNroBanco.setSingleStep(step)
        self.ui.spinChNroAgencia.setSingleStep(step)
        self.ui.spinChNroCC.setSingleStep(step)
        self.ui.spinChNroCheque.setSingleStep(step)
        self.ui.spinChValor.setSingleStep(step)
        self.ui.spinCartaoValor.setSingleStep(step)
        self.ui.spinValorTicket.setSingleStep(step)
        self.ui.spinOValor.setSingleStep(step)
        self.ui.spinSSCheque.setSingleStep(step)
        self.ui.spinSSCartao.setSingleStep(step)
        self.ui.spinSSTickets.setSingleStep(step)
        self.ui.spinSSOutros.setSingleStep(step)
        self.ui.spinSSDinheiro.setSingleStep(step)
        self.ui.spinSigTefValor.setSingleStep(step)
        self.ui.spinMFEcf.setSingleStep(step)
        return step

    @Log
    def get_status_tela(self):
        ret = '[SS:%s|SC:%s|SI:%s]' % (STATE_SCREEN_NAMES[self._state_screen],
                                       STATE_CUPOM_NAMES[self._state_coupon],
                                       STATE_ITEM_NAMES[self._state_item])
        return ret

    @Log
    def get_tef_nsu_socio_torcedor(self, nsu):
        self._tef_nsu_socio_torcedor = nsu

    @Log
    def verificar_confirmacao_tef(self):
        """ Função que verifica a existencia de alguma transação do tef que tenha ficado pendente
        por algum motivo.
        Essa função le a coluna TblMov_TransTef e verifica a existencia de algum registro. Caso
        exista algum registro,
        é informado ao operador que existe uma tranção do tef que não foi confirmada e que esta
        pendente, pergunta se deseja cancelar a transação ou corfirma a mesma."""

        resultSQL = None
        nroCupom = None
        dataCupom = None
        horaTransacao = None
        dataTransacao = None
        try:
            resultSQL = self.obj_cor_nsu.query.all()
        except Exception as err:
            Log.error(err)
        if resultSQL:
            continua_transacao = False
            socio_torcedor = False
            self.sig_tef.usando_sitef = True
            self.sig_tef.tipo_finalizadora = False
            ultimo_coo = self.obj_cupom_fiscal.get_numero_nf() - 1
            cupom_aberto = self.obj_cupom_fiscal.get_cupom_aberto()
            for r in resultSQL:
                nroCupom = r.supnsu_nro_cupom_venda
                dataCupom = r.supnsu_data_cupom
                horaTransacao = r.supnsu_tef_hora_transacao
                dataTransacao = r.supnsu_tef_data_transacao
                try:
                    a = ler_arq_socio_torcedor()
                    # interrompeu a aplicacao no meio do TEF
                    if (ultimo_coo != nroCupom or not cupom_aberto) and not a:
                        raise Exception

                    msg = 'Transação TEF pendente.\n\nNúmero da NFC-e de venda.: %s\nData NFC-e.: %s' \
                          '\n\nDeseja continuar a transação do tef ?' % (nroCupom, dataCupom)
                    continua_transacao = True
                    socio_torcedor = True
                except:
                    msg = 'Transação TEF pendente.\n\nNúmero da NFC-e de venda.: %s\nData NFC-e.: %s' \
                          '\n\nDeseja confirmar a transação do tef ?' % (nroCupom, dataCupom)

            if message_dlg(self, MSG_QUESTION, msg) == YES_ROLE:
                if self.user_logged.supope_operador_supervisor == 'O':
                    if not password_dlg(self, msg_info=u'Transação TEF').retval:
                        return

                resultConf = False
                # finaliza transacao
                if not continua_transacao:
                    resultConf = True
                    self.sig_tef.finalizar_transacao(1, nroCupom, dataTransacao,
                                                       horaTransacao)
                elif socio_torcedor:
                    self.ui.lblCliente.setText(u'Sócio  ')
                    self.ui.lblImgCliente.setPixmap(QtGui.QPixmap(
                        ":/logos/images/logos/socio_torcedor.png"))
                    self.socio_torcedor = True

                if resultConf and not continua_transacao:
                    message_dlg(self, MSG_INFO, 'Transação confirmada com sucesso. '
                                                '\n\nPara reimpressão do CDC, favor acessar o '
                                                'Menu Tef')

            else:
                if self.user_logged.supope_operador_supervisor == 'O':
                    if not password_dlg(self, msg_info=u'Transação TEF').retval:
                        self.sig_tef.usando_sitef = False
                        return
                self.sig_tef.finalizar_transacao(0, nroCupom, dataTransacao,
                                                        horaTransacao)
                message_dlg(self, MSG_INFO, 'Transação cancelada com sucesso.\nFavor reter a última NFC-e.')
                # para cupom aberto, cancela o cupom, para cupom finalizado, emite um cupom  de cancelamento
                cancelamento = 'N' if cupom_aberto else 'C'
                if socio_torcedor and cupom_aberto:
                    self.mov_capa = self.obj_mov_capa.search({'supcap_nro_cupom': ultimo_coo})
                    self.cancelar_nota_fiscal(automatico=True)

    @Log
    def show_printer_erro(self, text):
        msg = '<center><h2><font color="red">Não foi possível enviar o comando para ECF.' \
              '</font></h2></center><b><h2><font color="white"><font color="yellow">Sim: ' \
              '</font>Para tentar novamente.</font></h2><h2><font color="white">' \
              '<font color="yellow">Não: </font>Para encerrar a operação.</font></h2>'
        ret = message_dlg(self, MSG_QUESTION, msg)
        if ret == YES_ROLE:
            return True
        else:
            self._clear_globals(coupon_closed)
            self._set_controls_default_values()
            self._set_screen_state(in_menu_fiscal)
            self._habilitar_mnu_fiscal = True
            raise Exception(text)

    @Log
    def verificar_saldo_super(self):
        self.infoText('Verificando o Saldo Inicial')
        QtGui.QApplication.processEvents()
        if not self._flag_saldo_super:
            atualizar_saldo_inicial_supervisor()
            self._flag_saldo_super = True
        return self._flag_saldo_super

    @Log
    def send_command(self, cmd, test_status_time_out=True, qtd_tentativas=30):
        if cmd.startswith(('open', 'cancel', 'summarize', 'gerencial_report_open', 'close_till',
                           'till_add_cash', 'till_remove_cash', 'till_read_memory',
                           'till_read_memory_by_reductions')):

            if self.z_pendente and not cmd.startswith(('cancel', 'close_till')):
                return message_dlg(self, MSG_INFO,
                                   'Z Pendente. Favor Efetuar a Redução Z para efetuar '
                                   'esta operação')

        loop = 0
        res = None
        while True:
            loop += 1
            try:
                res = eval("self._printer.%s" % cmd)
                return res
            except TimeoutError as err:
                if not test_status_time_out:
                    raise TimeoutError(str(err))

                Log.error("Erro: '%s', Comando: %s. Vou aguarda 1 segundos e tentar novamente %d"
                          % (str(err), cmd, loop))
                if loop == qtd_tentativas:
                    raise TimeoutError('Foram feitas %d tentativas, e não foi possível comunicar '
                                       'com o ECF.' % loop)
                continue

            except (CodigoInvalido, CouponNotOpenError, CancelItemError, AlreadyTotalized,
                    CouponOpenError, ItemAdditionError) as err:
                raise err

            except OutofPaperError:
                raise OutofPaperError('Impressora sem papel!')

            except CouponNoFiscalOpenError as e:
                Log.error("%s" % e)
                # cancela o cupom nao fiscal
                self._printer.cancel_last_coupon()
                coo = self._printer.get_coo()
                message_dlg(self, MSG_INFO, 'Erro: %s\n Cupom não fiscal %s cancelado.' %
                            ("%s" % e,coo))
                raise e

            except SummerTime as err:
                raise err

            except (DriverError, ValueError), err:
                if not test_status_time_out:
                    raise TimeoutError(str(err))

                if err.message == 'Dados da última RZ corrompidos':
                    msg = 'Não foi possível enviar o comando para a ECF\n\nTENTAR NOVAMENTE?'
                    if message_dlg(self, MSG_QUESTION, msg, str(err.message)) != YES_ROLE:
                        return False

                elif err.message == 'Totalizador sem alíquota programada':
                    raise err

                Log.error("Erro: '%s', Comando: %s. Vou aguarda 1 segundos e tentar novamente %d"
                          % (str(err), cmd, loop))

                if loop == qtd_tentativas:
                    raise TimeoutError('Foram feitas %d tentativas, e não foi possível comunicar '
                                       'com o ECF.' % loop)
                continue

            except CommError as err:
                raise err

            except Exception as err:
                if loop == qtd_tentativas:
                    Log.error("Erro ao enviar comando '%s' para ECF ! \n %s" % (cmd, str(err)))
                    self._habilitar_mnu_fiscal = True
                    traceback.print_exc()
                    self.show_printer_erro(str(err))
            finally:
                # executa apenas uma vez se estiver adicionando item
                if not res and 'add_item' in cmd:
                    return res

        return None

    def set_status_conexao_servidor(self, st=False):
        self.status_conexao_supervisor = st
        if st:
            self.ui.lblConImg.setPixmap(QtGui.QPixmap("resources/24x24/nav_plain_green.png"))
        else:
            self.ui.lblConImg.setPixmap(QtGui.QPixmap("resources/24x24/nav_plain_red.png"))
        return st

    @Log
    def th_focus(self, control):
        if control == 'menu':
            control = self.ui.tableSigTefMenu
        elif control == 'texto':
            control = self.ui.edtSigTefTexto
        self.setControlFocus(self._CONTROL_FOCUS, control)
        return control.objectName()

    def executa_animacao_info(self, tipo_curva=QtCore.QEasingCurve.InOutElastic):
        """
        Executa a animação do texto de informação
        """
        w = (self.ui.frmInfo.width() - self.ui.lblInfo.width()) / 2

        self.info_anim.setEasingCurve(tipo_curva)
        h = (self.ui.frmInfo.height() - self.ui.lblInfo.height()) / 2
        self.info_anim.setStartValue(QtCore.QPointF(w, self.ui.lblInfo.height() * -1))

        self.info_anim.setEndValue(QtCore.QPointF(w, h))
        self.info_anim.setDuration(500)
        self.info_anim.setLoopCount(1)
        self.info_anim.start()
        return None

    @Log
    def bloqueia_importacao(self):
        self.importacao_bloqueada = True
        return 'Importacao Bloqueada'

    @Log
    def desbloqueia_importacao(self):
        self.importacao_bloqueada = False
        return 'Importacao Desbloqueada'

    @Log
    def infoText(self, text, anim=True, display=False):
        """
        Ajusta o texto a ser exibido no widget 'lblInfo' e executa a animação
        :param display: apenas as chamadas de funcao com display =True enviarao
                        a mensagem para o display do teclado
        """
        n_text = self.trUtf8(text)
        if n_text:
            text = n_text

        metrics = QtGui.QFontMetrics(self.font)
        w, h = metrics.width(text), metrics.height()  # Pega a largura/altura do texto em pixels
        self.ui.lblInfo.setGeometry(0 if anim else 50, 0 if anim else 10, w, h)
        self.ui.lblInfo.setText(text)
        self.ui.lblInfo.setAlignment(QtCore.Qt.AlignCenter | QtCore.Qt.AlignHCenter)
        if anim:
            self.executa_animacao_info()  # Executa a animação

        self._limpar_info = 0
        if display and self.teclado:
            self.limpa_display = True
            self.teclado.envia_display(text)
        return text

    @Log
    def _add__payment_receipt(self, method, value, sequence, texto=False):
        if method.supfin_outros_qtd_parcelas > 0:
            qtd_parcelas = method.supfin_outros_qtd_parcelas
        else:
            qtd_parcelas = 1
        vlr_parcela = value / qtd_parcelas
        data = datetime.today().date()
        supfin_outros_prazo_primeira = method.supfin_outros_prazo_primeira

        a = []
        if not texto:
            text = 'Reconheco a exatidao desta NOTA PROMISSORIA que pagarei a %s, CNPJ: %s. ' \
                   'Ou a sua ordem nos vencimentos indicados.\n\n' \
                   % (self.loja.supcfc_cliente, self.loja.supcfc_cnpj)
            text += 'Valor por Extenso: %s \n \n' % str(
                NumeroPorExtenso(value).extenso_unidade).upper()
            text += 'Parcela Vencimento Valor \n'
            for x in range(qtd_parcelas):
                text += '%7s %s R$ %.2f\n' % (
                    (str(x + 1) + '/' + str(qtd_parcelas)), data.strftime("%d/%m/%Y"), vlr_parcela,)
                data += timedelta(days=supfin_outros_prazo_primeira)
                a.append((x + 1, data, vlr_parcela))
            text += ' \n \n \nAss. Emitente: --------------------------------- \n'
            complemento = self.cliente.supcli_complemento and ('-' + self.cliente.supcli_complemento) or ''
            text += '%s\n%s\n%s - %s%s \n%s %s\n' % (
                self.cliente.name, self.cliente.supcli_cnpj_cpf, self.cliente.supcli_endereco,
                self.cliente.supcli_endereco_numero, complemento, self.cliente.supcli_cep,
                self.cliente.supcli_cidade)
            text += '================================================ \n'
        else:
            text = texto
            a.append((1, data, vlr_parcela))
        return self._payment_receipt.append(('%.2d' % method.supfec_codigo_na_ecf, text, value,
                                             sequence, a))

    @Log
    def _add_remove_cash_till(self, op, logoff=False):
        """ Efetua sangria/suprimento
            op: SA = Sangria; SU = Suprimento
        """

        @Log
        def screen_ss(op):
            if op == 'SU':
                self.ui.lblSangriaSuprimento.setText(u'Suprimento - Adicionar dinheiro no caixa')
                self._set_controls_default_values(ss_visible=False)
            elif op == 'SA':
                self.ui.lblSangriaSuprimento.setText(u'Sangria - Retirar dinheiro do caixa')
                self._set_controls_default_values(ss_visible=True)

                if not config['sangriamanual']:
                    data_caixa = """

                    SELECT MAX(supcap_data_inicio) AS data_caixa
                    FROM   caixa_mov_capa
                    WHERE  supcap_loja = %d
                           AND supcap_caixa = %d
                           AND supcap_tipo = 'A'
                           AND supcap_data_inicio > '%s'

                    """ % (self.loja.supcfc_loja, self.caixa_id,
                           self.mov_caixa.supcax_data_hora_abertura)

                    data_caixa = exec_sql(data_caixa, 'C', ret=True)

                    if data_caixa:
                        data = data_caixa[0]['data_caixa'] or self.mov_caixa.supcax_data_hora_abertura

                        total_fech = """

                        SELECT SUM(CASE mov_capa.supcap_tipo
                                     WHEN 'C' THEN 0
                                     ELSE mov_fechamentos.supfec_valor
                                   END)                                              AS valor,
                               mov_fechamentos.supfec_tipo_finalizadora              AS finalizadora
                        FROM   caixa_mov_fechamentos mov_fechamentos,
                               caixa_mov_capa mov_capa
                        WHERE  mov_capa.supcap_loja = %d
                               AND (mov_capa.supcap_state IN ('3', '4', '2E', '6') 
                                    OR mov_capa.supcap_tipo = 'W')
                               AND mov_capa.supcap_caixa = %d
                               AND mov_fechamentos.supfec_capa_id = mov_capa.id
                               AND mov_capa.supcap_tipo IN ('V', 'U', 'C', 'D', 'W')
                               AND mov_capa.supcap_data_inicio >= '%s'
                        GROUP  BY  mov_fechamentos.supfec_tipo_finalizadora

                        """ % (self.loja.supcfc_loja, self.caixa_id, data)

                        for f in exec_sql(total_fech, 'C', ret=True):
                            if f['finalizadora'] == 'H':
                                self.ui.spinSSCheque.setValue(f['valor'])
                            if f['finalizadora'] == 'C':
                                self.ui.spinSSCartao.setValue(f['valor'])
                            if f['finalizadora'] == 'O':
                                self.ui.spinSSOutros.setValue(f['valor'])
                            if f['finalizadora'] == 'T':
                                self.ui.spinSSTickets.setValue(f['valor'])

            self.ui.stackedActions.setCurrentIndex(stk_page_ss)
            self._tab_order = [self.ui.spinSSDinheiro]
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinSSDinheiro)
            return op

        if not self.user_logged:
            message_dlg(self, MSG_INFO, 'É necessário estar logado para efetuar')
        else:
            if not self.mov_caixa:
                self.obj_abertura_fechamento_turno.abrir_turno(self.loja.supcfc_loja,
                                                          self.caixa_id,
                                                          self.user_logged.supope_codigo)
            if self.mov_caixa:
                if self.par_sangria_x_z_suprimento and self.user_logged.supope_operador_supervisor == 'O':
                    if password_dlg(self, msg_info='Sangria').retval:
                        if config['usagaveta']:
                            self.obj_impressoras.supcci_imprimir("{gaveta}")
                            # self._timer_end_sale.start(500)
                        screen_ss(op)
                else:
                    if self.mov_caixa.supcax_operador_id != self.user_logged.id \
                            and self.user_logged.supope_operador_supervisor == 'O':
                        if self._state_screen == in_close_turn:
                            message_dlg(self, MSG_INFO,
                                        'Você não pode encerrar o turno de outro operador')
                        elif self._state_screen == in_ss:
                            message_dlg(self, MSG_INFO, 'Você não pode efetuar Sangria/Suprimento '
                                                        'no Turno de outro operador')

                    else:
                        if config['usagaveta']:
                            self.obj_impressoras.supcci_imprimir("{gaveta}")
                            # self._timer_end_sale.start(500)
                        screen_ss(op)
                        if logoff:
                            self.ui.stackedActions.setCurrentIndex(stk_page_login)
                            self.ui.edtUser.clear()
                            self.ui.edtPwd.clear()
                            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtUser)
                            self.user_logged = False
                            Log.info('Operador: %s' % self.user_logged)
                            self.mov_caixa = False
                            self.ui.lblNomeUser.setText(u'Off-line')
                            self._state_screen = in_login

        self.emitindo_dav = False
        return

    @Log
    def run_tef(self, value, nro_cupom, tipo=None, modalidade=0):
        try:
            import time
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.menuTef)
            self.check_button_status()
            self.show_info_proccess(True, 'Iniciando T.E.F')
            if not self.sig_tef._status_iniciou_sitef:
                self.ui.edtSigTefTexto.setVisible(False)
                self.ui.tableSigTefMenu.setVisible(False)

            self.ui.spinSigTefValor.setVisible(False)

            self.sig_tef.usando_sitef = True
            self.sig_tef.tipo_finalizadora = tipo

            self.ui.stackedActions.setCurrentIndex(stk_tef)

            now = datetime.now()
            data_fiscal = now.strftime('%Y%m%d')
            horario = now.strftime('%H%M%S')

            self.sig_tef.inicia_sitef(value, nro_cupom, data_fiscal, horario,
                                      self.user_logged.name, modalidade=modalidade)
            if self._tef_nsu_socio_torcedor and self.socio_tef:
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtSigTefTexto)

            if (self.cartao_tef or self.ticket_tef) and self.sig_tef.usando_sitef:
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.tableSigTefMenu)

            self.socio_tef = False
            self.cartao_tef = False
            self.ticket_tef = False
            self.show_info_proccess(False)
        except Exception as e:
            self.tratar_except()
        return True

    @Log
    def _set_screen_state(self, state):
        if self._IdDav:
            self.emitindo_dav = True # bloqueia o botao enter quando for dav

        if self.verifica_z_emitido() and state in [in_coupon, in_dav]:
            state = in_menu_fiscal
            message_dlg(self, MSG_INFO, 'Redução Z de hoje já foi emitida!')

        if QtGui.QApplication.desktop().screenGeometry().width() > 1024 and state == in_coupon:
            self.ui.logo_foot.setVisible(False)
            self.ui.lblImageProdFoot.setVisible(False)
        else:
            path_logo = os.path.join(ROOT_PATH, config['local'], 'logo.png')
            self.ui.lblImageProdFoot.setPixmap(QtGui.QPixmap(path_logo))
            self.ui.logo_foot.setVisible(True)
            self.ui.lblImageProdFoot.setVisible(True)

        if state in (in_menu, in_menu_fiscal):
            menu = self.ui.mainMenu
            self.ui.stackedActions.setCurrentIndex(stk_page_menu)
            if state == in_menu:
                if not self.user_logged:
                    menu.item(imenu_login).setText(u'Login')
                else:
                    menu.item(imenu_login).setText(u'LogOff')
                if not self.mov_caixa:
                    menu.item(imenu_turno).setText(u'Abrir Turno')
                else:
                    menu.item(imenu_turno).setText(u'Encerrar Turno')

                if self.user_logged:
                    self.ui.mainMenu.setCurrentItem(menu.item(imenu_turno))
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.mainMenu)
                    self._tab_order = [self.ui.mainMenu]
                    self.ui.stackedMenus.setCurrentIndex(0)
                else:
                    self._set_screen_state(in_menu)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.mainMenu)
            else:
                self.ui.stackedMenus.setCurrentIndex(1)
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.mainMenu)
                self.ui.stackedMenus.setCurrentIndex(1)

        elif state == in_login:  # TELA DE LOGIN
            self.ui.stackedActions.setCurrentIndex(stk_page_login)
            self.ui.label_biometria.setVisible(False)
            self.ui.frame_biometria.setVisible(False)
            if config['usar_biometria']:
                self.ui.label_biometria.setVisible(True)
                self.ui.frame_biometria.setVisible(True)
                self.ui.label_3.setVisible(False)
                self.ui.edtUser.setVisible(False)
                self.ui.label_4.setVisible(False)
                self.ui.edtPwd.setVisible(False)
                self.ui.frmInfoLoginIcon.setVisible(False)
                self.ui.lblInfoLogin.setVisible(False)

            self.ui.edtUser.clear()
            self.ui.edtPwd.clear()
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtUser)
            # nao deixa perder usuario logado quando a importacao entra logo apos emitir um z pendente e abrir o turno
            if not self._state_screen == in_login:
                self.user_logged = False

            Log.info('Operador: %s' % self.user_logged)
            self.mov_caixa = False
            self._wait_logoff = False
            self.ui.lblNomeUser.setText(u'Off-line')

        elif state == in_coupon:
            #### TELA DE VENDAS/CUPOM ###
            if not config['impressoras']:
                message_dlg(self, MSG_INFO, 'Sem set de comandos para impressora cadastrada!')
                return False

            self.ficha = False
            self.check_button_status()
            self._clear_globals(coupon_closed)
            self.desconto_socio_calculado = False
            if self._state_screen == in_end_sale: # entrou na tela de cupom vindo da tela de final de venda
                self.desbloqueia_importacao()

            if self._state_coupon == coupon_closed:
                if not self.obj_cupom_fiscal.check_coupon_is_opened():
                    return False

            if not self.mov_caixa:
                if not self.obj_abertura_fechamento_turno.abrir_turno(self.loja.supcfc_loja,
                                                                 self.caixa_id,
                                                                 self.user_logged.supope_codigo):
                    return False
            if self.mov_caixa and parse(str(self.mov_caixa.supcax_data_hora_abertura),
                                        yearfirst=True).date() != date.today():
                message_dlg(self, MSG_INFO, 'Encerre o turno')
                self._set_screen_state(in_menu)
            if not self._PveCancelamento and config[
                'usarprevenda'] and self._state_coupon == \
                    coupon_closed and self.status_conexao_supervisor:
                if not self._IdPve and not self.consulta_produto:
                    self.show_info_proccess(True, "Checando PVE's")
                    self.check_pves(days=2)
                    self.show_info_proccess(False)

            self.ui.stackedActions.setCurrentIndex(stk_page_coupom)
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
            self._tab_order = [self.ui.edtCodigo]
            if config['apenas_ficha']:
                self.KEY_FICHA()
                state = in_ficha

        elif state == in_ficha:
            self.ui.edtItemFicha.clear()
            self.ui.spinQtdItemFicha.setValue(1)
            self.clear_widget_table(self.ui.tableItensFicha)
            self.ui.stackedActions.setCurrentIndex(stk_page_ficha)
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtItemFicha)

        elif state == in_search_client:
            #### TELA LOCALIZAR CLIENTE ###
            tableBuscaCliente = self.ui.tableBuscaCliente
            # Definir Largura das Colunas
            tableBuscaCliente.setColumnWidth(0, 141)  # cpf/cnpj
            tableBuscaCliente.setColumnWidth(1, 141)  # código
            tableBuscaCliente.setColumnWidth(2, 570)  # nome do cliente
            # Verifica se hà dados
            self.clear_widget_table(tableBuscaCliente)
            self.infoText('Digite o Código/Nome do Cliente')
            self.ui.lbl_consulta.setText('.: Consulta Cliente :.')
            self.ui.stackedActions.setCurrentIndex(stk_search_client)
            self.ui.stackedBuscaCliPro.setCurrentIndex(0)
            self._tab_order = [self.ui.edtBuscaCliente]
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtBuscaCliente)
            self.ui.edtBuscaCliente.clear()

        elif state == in_dav:
            tableDav = self.ui.tableBuscaDav
            # LIMPA O WIDGET TABLE
            self.clear_widget_table(tableDav)

            self.ui.lblPreDav.setText(
                u'DAV - Documento Auxiliar de Venda [%s]' % config.get('aplicativo_dav', ''))
            self.ui.stackedActions.setCurrentIndex(stk_dav_pre)
            self.ui.stackedPreDav.setCurrentIndex(0)
            davs = SupervisorDavCapa.query.filter(SupervisorDavCapa.supdva_restricao == 1).filter(
                SupervisorDavCapa.supdva_nro_cupom == None).order_by('supdva_nro_pedido desc')

            search_result_count = davs.count()

            if search_result_count:
                # POPULA O WIDGET TABLE COM O RESULTADO DA BUSCA
                rows = 0
                text = self.ui.edtBuscaClienteDav.text()
                if text:
                    cliente_davs = SupervisorDavCapa.query.filter(
                        SupervisorDavCapa.supdva_restricao == 1).filter(
                        SupervisorDavCapa.supdva_nro_cupom == None).filter(
                        SupervisorDavCapa.supdva_nome_cliente.ilike('%s%s%s' % ('%', text, '%'))).order_by(
                                                                   'supdva_nro_pedido desc')
                    davs = cliente_davs
                    try: # testa se text eh inteiro se for pesquisa pelo numero do pedido
                        if not cliente_davs.count():
                            cod_davs = SupervisorDavCapa.query.filter(
                                SupervisorDavCapa.supdva_restricao == 1).filter(
                                SupervisorDavCapa.supdva_nro_cupom == None).filter(
                                SupervisorDavCapa.supdva_nro_pedido == int(text)).order_by('supdva_nro_pedido desc')
                            davs = cod_davs
                    except:
                        pass
                emp = config['db_ges_emp'].replace(' ', '').replace('-', ',').split(',')
                emp_list = []
                for x in emp:
                    emp_list.append(int(x))
                for dav in davs:
                    loja = dav.supdva_loja
                    if loja in emp_list:
                        brush = QtGui.QBrush(QtCore.Qt.green)
                        if dav.supdva_restricao == 3:
                            brush = QtGui.QBrush(QtCore.Qt.red)
                        brush.setStyle(QtCore.Qt.SolidPattern)
                        tableDav.insertRow(rows)
                        # 0 - EMPRESA
                        iLoja = QtGui.QTableWidgetItem('%.4d' % loja)
                        iLoja.setIcon(self.icon_check)
                        iLoja.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                        iLoja.setBackground(brush)
                        # 1 - CODIGO DAV
                        iCOD = QtGui.QTableWidgetItem('%.14d' % dav.supdva_nro_pedido)
                        iCOD.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                        # 2 - DATA DO DAV
                        iData = QtGui.QTableWidgetItem('%s' % dav.supdva_data_hora)
                        iData.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                        # 3 - NOME CLIENTE
                        iNome = QtGui.QTableWidgetItem('%s' % dav.supdva_nome_cliente)
                        iNome.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                        # 4 - VALOR
                        iValor = QtGui.QTableWidgetItem('%.2f' % dav.supdva_total_dav)
                        iValor.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                        iValor.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
                        # Adiciona as linhas no Widget
                        tableDav.setItem(rows, 0, iLoja)
                        tableDav.setItem(rows, 1, iCOD)
                        tableDav.setItem(rows, 2, iData)
                        tableDav.setItem(rows, 3, iNome)
                        tableDav.setItem(rows, 4, iValor)
                        rows += 1
                tableDav.selectRow(0)
                tableDav.resizeColumnsToContents()
                tableDav.horizontalHeader().setStretchLastSection(True)
                self._tab_order = [self.ui.tableBuscaDav]
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.tableBuscaDav)
                self.ui.edtBuscaClienteDav.setText('')
            else:
                message_dlg(self, MSG_INFO, "Não existem DAV's cadastrados.")
                return self._set_screen_state(in_coupon)

        elif state == in_pve:
            tablePve = self.ui.tableBuscaPve
            # LIMPA O WIDGET TABLE
            self.clear_widget_table(tablePve)
            self.ui.lblPreDav.setText(u'PVE :: Pré-Venda')
            self.ui.stackedActions.setCurrentIndex(stk_dav_pre)
            self.ui.stackedPreDav.setCurrentIndex(1)
            rows = 0

            # POPULA O WIDGET TABLE COM O RESULTADO DA BUSCA
            for pve in self._pves_list:
                tablePve.insertRow(rows)
                # 0 - CODIGO PVE
                iCOD = QtGui.QTableWidgetItem('%.14d' % pve.id)
                if pve.suppva_restricao in (-1, -2):
                    iCOD.setIcon(self.icon_delete)
                else:
                    iCOD.setIcon(self.icon_check)
                iCOD.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                # 2 - NOME CLIENTE
                iNome = QtGui.QTableWidgetItem('%s' % pve.suppva_nome_cliente)
                iNome.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                # 2 - VALOR
                if pve.suppva_descontos:
                    iValor = QtGui.QTableWidgetItem('%.2f' % (pve.suppva_total_pve -
                                                              pve.suppva_descontos))
                else:
                    iValor = QtGui.QTableWidgetItem('%.2f' % (pve.suppva_total_pve or 0))
                iValor.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                iValor.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
                # Adiciona as linhas no Widget
                tablePve.setItem(rows, 0, iCOD)
                tablePve.setItem(rows, 1, iNome)
                tablePve.setItem(rows, 2, iValor)
                rows += 1
            tablePve.selectRow(0)
            tablePve.resizeColumnsToContents()
            tablePve.horizontalHeader().setStretchLastSection(True)
            self._tab_order = [self.ui.tableBuscaPve]
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.tableBuscaPve)

        elif state == in_add_payment:
            # Efetuando pagamento
            if not self._coupon_subtotal and not self.ficha:
                ret_msg = message_dlg(self, MSG_QUESTION, 'Deseja cancelar a Nota',
                                      'Não há itens adicionado à Nota.')
                if ret_msg == YES_ROLE:
                    self.cancelar_nota_fiscal(automatico=True)
                    self.ui.lblNumCaixa.setText(u'%03d <b>Aberto' % self.caixa_id)
                    return
                else:
                    self._state_screen = in_coupon
                    self.check_button_status()
                    return

            pm_method = self._coupon_payment_method
            if not self._coupon_totalize:
                self.ui.lblVlrTotalR.setText(u'%.2f' % float(self.ui.lblVlrTotalAPagar.text()))
                self.ui.lblVlrReceber.setText(u'%.2f' % float(self.ui.lblVlrTotalAPagar.text()))

            vlr_receber = Decimal(self.ui.lblVlrReceber.text().split(' ')[-1])
            if pm_method == pm_dinheiro:
                self._tab_order = [self.ui.edtBuscaCliente]
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinPgtoDinheiro)
                self.ui.spinPgtoDinheiro.setValue(vlr_receber)
                self.ui.spinPgtoDinheiro.selectAll()

            elif pm_method == pm_cheque:
                self.emitindo_dav = False # desbloqueia a tecla 'enter' para preencher o cheque
                self.ui.dateChVcto.setInputMask('00/00/0000')
                self.ui.dateChVcto.setText(datetime.now().date().strftime('%d/%m/%Y'))
                self.ui.edtChCliente.setText(self.cliente.name)
                self.ui.edtChDocEmit.setText(self.cliente.supcli_cnpj_cpf)
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbChTipoCheque)
                self.ui.spinChValor.setValue(vlr_receber)
                if self._DavFechamentosIter:
                    try:
                        vals = self._DavFechamentosIter.next()
                        self.ui.cbChTipoCheque.setCurrentIndex(vals['Index'])
                        self.ui.spinChValor.setValue(vals['Valor'])
                        self.ui.dateChVcto.setText(vals['Vencimento'])
                        self.ui.spinChNroBanco.setValue(vals['banco'])
                        self.ui.spinChNroAgencia.setValue(vals['agencia'])
                        self.ui.spinChNroCC.setValue(vals['conta'])
                        self.ui.spinChNroCheque.setValue(vals['numero'])
                        self.ui.spinChValor.setReadOnly(True)
                        self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbChProp)
                    except:
                        self.ui.spinChValor.setReadOnly(False)
                        self._DavFechamentosIter = False

            elif pm_method == pm_cartao:
                self.emitindo_dav = False # desbloqueia a tecla 'enter' para preencher o cheque
                self.ui.spinCartaoValor.setValue(vlr_receber)
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbTipoTef)

            elif pm_method == pm_ticket:
                self.emitindo_dav = False # desbloqueia a tecla 'enter' para preencher o cheque
                self.ui.spinValorTicket.setValue(vlr_receber)
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbTipoTicket)

            elif pm_method == pm_outros:
                self.emitindo_dav = False # desbloqueia a tecla 'enter' para preencher o cheque
                if self.cliente.name:
                    self.ui.edtOCodCliente.setText(self.cliente.supcli_cnpj_cpf)
                    self.ui.lblOCliente.setText(
                        """<span style=" font-family:'Ubuntu'; font-size:13pt; font-weight:600;
                        font-style:italic; color:#dcdcdc;">%s</span>""" % self.cliente.name)
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbOTipoPgto)
                self.ui.spinOValor.setValue(vlr_receber)

            self.ui.stackedActions.setCurrentIndex(stk_page_payments)
            self.ui.stackedPgto.setCurrentIndex(pm_method)
            self._tab_order = [self._CONTROL_FOCUS]

        elif state == in_importacao:
            try:
                self.teclado and self.teclado.envia_display('Tentando conectar no Supervisor ...')
                self.ui.stackedActions.setCurrentIndex(stk_importacao)
                QtCore.QTimer.singleShot(0, self.executar_importacao)
            except Exception as err:
                Log.info(str(err))
                return self.parar_importacao(str(err))
        elif state == in_end_sale and (not self.vr_doacao or not self.vr_ficha):
            self.cliente = NotClient
            self.cliente_total_aberto = 0.0
            self._message_in_coupon = None
            self.vendedor_codigo = 0
            self._nome_vendedor = ''
            self.vendedor_comanda_ids = []
            self._set_controls_default_values()
            self.show_info_proccess(False)
            self._payment_receipt = []
            self._payment_receipt_g = []
            self._coupon_totalize = None
            self._state_coupon = coupon_closed
            self._timer_end_sale.stop()
            self.sig_tef.usando_sitef = False
            self.socio_torcedor = False
            self.emitindo_dav = False
            self._IdDav = False
            self.desbloqueia_importacao()

        self._state_screen = state
        self.check_button_status()

        return STATE_SCREEN_NAMES[state]

    @Log
    def biometria(self, autorizacao=False):
        '''
        Controle de acesso usando biometria
        :param autorizacao: se for true que dizer que esta solicitando autorizacao do supervisor e
                            nao esta na tela de login
        :return:
        '''
        self.show_info_proccess(True, 'Lendo Biometria')
        sql = """SELECT id,
                        name,
                        supope_biometria,
                        supope_codigo
                 FROM caixa_cad_operadores
                 WHERE supope_biometria IS NOT NULL
                   AND active=True
                   """
        msg = '<font size=16>Atenção!</font>\n\n\nOperador não encontrado!'
        if autorizacao:
            sql += "AND supope_operador_supervisor = 'S' "
            msg = u'<font size=16>Atenção!</font>\n\n\nEste operador não é Supervisor! Tente Novamente!'

        dados_operadores = exec_sql(sql, 'C')
        if not dados_operadores:
            self.show_info_proccess(False)
            return message_dlg(self, MSG_INFO,
                               '<font size=16>Atenção!</font>\n\n\nOperador não encontrado!')

        dict_fir = {}
        map(lambda x: dict_fir.update({x.get('supope_codigo') :x.get('supope_biometria')}),
              dados_operadores)

        self.obj_fingertech.dict_firs = dict_fir
        self.obj_fingertech.qtd_fir = len(dict_fir)

        try:
            self.show_info_proccess(True, 'Comparando Biometria')
            operador_id = self.obj_fingertech.verifica_biometria_bd()
        except Exception as e:
            Log.error('Erro ao ler biometria: %s' % str(e))
            return message_dlg(self, MSG_ERROR, 'Houve um erro interno!'
                                       '\nInforme este fato ao Suporte Técnico!\n'
                                       '\n%s' % str(e))
        finally:
            self.show_info_proccess(False)

        if operador_id:
            # se pedir autorizacao retorna apenas o operador
            if autorizacao:
                return {'autorizado': True, 'operador': CaixaCadOperadores.get_by(id=operador_id)}

            return self.operador_encontrado(CaixaCadOperadores.get_by(id=operador_id))

        return message_dlg(self, MSG_INFO, msg)

    @Log
    def _returnPressed_edtUser(self):
        if config['usar_biometria']:
            return self.biometria()

        text = self.ui.edtUser.text()
        if self.ui.edtUser.text():
            operador = CaixaCadOperadores.get_by(supope_codigo=self.ui.edtUser.text(),
                                                 active=True)
            if operador:
                msg = 'Seja bem vindo, <span style=" color:#ff0000;"><b>%s</b></span>!<br>' % operador.name+ \
                      'Digite sua senha.'
                print msg
                self.ui.lblInfoLogin.setText(self.trUtf8(str(msg)))
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtPwd)
                return
            else:
                self.ui.lblInfoLogin.setText(self.trUtf8('Código do Usuário não cadastrado!'))
                self.ui.edtUser.clear()
                return
        else:
            self.ui.lblInfoLogin.setText(self.trUtf8('Digite o Código do Usuário!'))
        return text

    @Log
    def _returnPressed_edtMFBuscaProduto(self):
        text = self.ui.edtMFBuscaProduto.text()
        self.clear_widget_table(self.ui.tableMFBuscaProduto)
        if text:
            if text.isdigit():
                print 'Buscando Código'
                search_result = CaixaCadItens.query.filter(
                    CaixaCadItens.supite_loja == self.loja.supcfc_loja).filter(
                    CaixaCadItens.supite_codigo_item == text.rjust(14, '0')).order_by(
                    CaixaCadItens.supite_descricao_completa).limit(100)
            else:
                search_by = CaixaCadItens.supite_descricao_completa.like('%s%s' % (text, '%'))
                search_result = CaixaCadItens.query.filter(
                    CaixaCadItens.supite_loja == self.loja.supcfc_loja).filter(search_by).order_by(
                    CaixaCadItens.supite_descricao_completa).limit(100)
                print 'Buscando Inicia', search_by
                if not search_result.count():
                    search_by = CaixaCadItens.supite_descricao_completa.like(
                        '%s%s%s' % ('%', text, '%'))
                    search_result = CaixaCadItens.query.filter(
                        CaixaCadItens.supite_loja == self.loja.supcfc_loja).filter(
                        search_by).order_by(CaixaCadItens.supite_descricao_completa).limit(100)
                    print 'Buscando Contêm', search_by

            if search_result.count():
                tableBuscaProduto = self.ui.tableMFBuscaProduto
                rows = 0
                tableBuscaProduto.setEnabled(False)
                for produto in search_result:
                    tributacao_sql = CaixaCadTributacao.query.filter(
                        CaixaCadTributacao.id == produto.supite_tributacao_id).order_by(
                        CaixaCadTributacao.id)
                    tributacao_produto = tributacao_sql.one()

                    tableBuscaProduto.insertRow(rows)
                    # POPULA TABELA
                    # No. Item
                    newItem = QtGui.QTableWidgetItem(produto.supite_codigo_item)
                    newItem.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                    newItem.setFont(self._font)
                    # Descrição
                    newDesc = QtGui.QTableWidgetItem(produto.name)
                    newDesc.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                    newDesc.setFont(self._font)
                    # Pesavel
                    iat_str = 'A'
                    if produto.supite_iat == 'T':
                        iat_str = 'T'
                    newPesavel = QtGui.QTableWidgetItem(iat_str)
                    newPesavel.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                    newPesavel.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
                    newPesavel.setFont(self._font)
                    prod = 'P'
                    if produto.supite_producao == 'T':
                        prod = 'T'
                    newFracionado = QtGui.QTableWidgetItem(prod)
                    newFracionado.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                    newFracionado.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
                    newFracionado.setFont(self._font)
                    # Unidade
                    newUnid = QtGui.QTableWidgetItem(produto.supite_unidade)
                    newUnid.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                    newUnid.setFont(self._font)
                    # Situação Tributária
                    newSt = QtGui.QTableWidgetItem(tributacao_produto.suptri_codigo_trib_ecf)
                    newSt.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                    newSt.setFont(self._font)
                    # Valor Unitário
                    newVUni = QtGui.QTableWidgetItem('%1.2f' % produto.supite_preco)
                    newVUni.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                    newVUni.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
                    newVUni.setFont(self._font)
                    # WIDGET OS NEW ITENS NA TABLE ITENS
                    tableBuscaProduto.setItem(rows, 0, newItem)
                    tableBuscaProduto.setItem(rows, 1, newDesc)
                    tableBuscaProduto.setItem(rows, 2, newPesavel)
                    tableBuscaProduto.setItem(rows, 3, newFracionado)
                    tableBuscaProduto.setItem(rows, 4, newUnid)
                    tableBuscaProduto.setItem(rows, 5, newSt)
                    tableBuscaProduto.setItem(rows, 6, newVUni)
                    rows += 1
                tableBuscaProduto.selectRow(0)
                self.setControlFocus(self._CONTROL_FOCUS, tableBuscaProduto)
                tableBuscaProduto.resizeColumnsToContents()

                tableBuscaProduto.horizontalHeader().setStretchLastSection(True)
            else:
                self.infoText('Nenhum registro encontrado!!!', display=True)
                self.ui.edtBuscaCliente.clear()
        else:
            self.infoText('Digite o código ou a descrição do produto para efetuar a busca!', display=True)
        return text

    @Log
    def _consulta_cliente(self, text):
        try:
            int(text)
            if len(text) >= 11:
                _cpf_cnpj = text.rjust(15, '0')  # 14 zeros a esquerda
                search_by = CaixaCadClientes.supcli_cnpj_cpf == (_cpf_cnpj)
                search_result = CaixaCadClientes.query.filter(search_by) \
                    .order_by(CaixaCadClientes.name).limit(100)
                if not search_result.count():
                    search_by = CaixaCadClientes.supcli_codigo_interno == (_cpf_cnpj)
                    search_result = CaixaCadClientes.query.filter(search_by) \
                        .order_by(CaixaCadClientes.name).limit(100)
            else:
                search_by = CaixaCadClientes.supcli_codigo_interno == (text)
                search_result = CaixaCadClientes.query.filter(search_by).order_by(
                    CaixaCadClientes.name).limit(100)
        except Exception as e:
            if len(text) < 5:
                self.infoText('Digite ao menos 5 caracteres p/ efetuar a busca', display=True)
                return False
            else:
                search_by = "name ~* '%s'" % text
                search_result = CaixaCadClientes.query.filter(search_by).order_by(
                    CaixaCadClientes.name).limit(100)
        search_result_count = search_result.count()
        tableBuscaCliente = self.ui.tableBuscaCliente
        if search_result_count:
            # LIMPA O WIDGET TABLE
            self.clear_widget_table(tableBuscaCliente)
            # POPULA O WIDGET TABLE COM O RESULTADO DA BUSCA
            rows = 0
            tableBuscaCliente.setEnabled(False)
            for cliente in search_result:
                tableBuscaCliente.insertRow(rows)
                # CNPJ/CPF
                isupcli_cnpj_cpf = QtGui.QTableWidgetItem(cliente.supcli_cnpj_cpf)
                isupcli_cnpj_cpf.setFlags(
                    QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                # CODIGO INTERNO
                isupcli_codigo = QtGui.QTableWidgetItem(str(cliente.supcli_codigo_interno))
                isupcli_codigo.setFlags(
                    QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                # Quantidade
                inome = QtGui.QTableWidgetItem(cliente.name)
                inome.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                # Adiciona as linhas no Widget
                tableBuscaCliente.setItem(rows, 0, isupcli_cnpj_cpf)
                tableBuscaCliente.setItem(rows, 1, isupcli_codigo)
                tableBuscaCliente.setItem(rows, 2, inome)
                rows += 1
            tableBuscaCliente.selectRow(0)
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.tableBuscaCliente)
            tableBuscaCliente.resizeColumnsToContents()
            tableBuscaCliente.horizontalHeader().setStretchLastSection(True)
        else:
            self.infoText('Nenhum registro encontrado!!!', display=True)
            self.clear_widget_table(tableBuscaCliente)
            self.ui.edtBuscaCliente.clear()

        return True

    @Log
    def get_produtos(self, text):
        sql_main = """
            SELECT  cci.supite_codigo_item,
                    cci.name,
                    cci.active,
                    cci.supite_descricao_completa,
                    cci.supite_iat,
                    cci.supite_unidade,
                    cct.suptri_codigo_trib_ecf,
                    cci.supite_preco,
                    cci.supite_producao
            FROM    caixa_cad_itens         AS cci
            INNER JOIN caixa_cad_tributacao AS cct ON cct.id = cci.supite_tributacao_id
            WHERE supite_loja = {loja}
              {cond_active}
              {cond_item}
            {order_limit}
        """
        result = []
        loja = self.loja.supcfc_loja
        order_limit = """ORDER BY name
            LIMIT 100"""
        try:
            if text.isdigit():
                if text.startswith('2') and len(text) == 13:
                    sql_main = sql_main.format(loja=loja, cond_active='', cond_item='', order_limit='')
                    result = self._busca_produto_balanca(text, consulta=sql_main)
                    if result == 99:
                        return False

                else:
                    cond_item = """AND supite_codigo_item = '%s' """ % text.rjust(14, '0')
                    sql_main = sql_main.format(loja=loja, cond_active='', cond_item=cond_item,
                                               order_limit=order_limit)
                    result = exec_sql(sql_main, 'C', ret=True)
                    if result:
                        if not result[0].get('active'):
                            return message_dlg(self, MSG_INFO,
                                               '<font size=6>Atenção!</font>\n\n\nItem inativo!')
            else:
                campo = 'name'
                if not config['descricao_resumida']:
                    campo = 'supite_descricao_completa'

                cond_item = """AND %s ~*'%s'""" % (campo, text)
                cond_active = """AND cci.active = True """
                result = exec_sql(
                    sql_main.format(loja=loja, cond_active=cond_active, cond_item=cond_item,
                                    order_limit=order_limit),
                    'C', ret=True)
                if not result:
                    cond_item = """AND %s like '%%%s%%' """ % (campo, text)
                    sql_main = sql_main.format(loja=loja,
                                               cond_active=cond_active,
                                               cond_item=cond_item,
                                               order_limit=order_limit)
                    result = exec_sql(sql_main, 'C', ret=True)
        except Exception:
            self.tratar_except()

        return result

    @Log
    def _consulta_produto(self, text):
        if self._previous_state != in_ficha and config['usar_ficha'] and \
                (not text.startswith('99') or len(text) < 15 or text[-1] != modulo10(text[:-1])):
            return message_dlg(self, MSG_INFO, 'Atenção!\n\nCódigo de ficha inválido!')

        result = self.get_produtos(text)
        tableBuscaProduto = self.ui.tableBuscaProduto
        if result == 99:
            self.clear_widget_table(tableBuscaProduto)
            self.ui.edtBuscaCliente.clear()
            return True

        if result:
            codigo_itens = tuple(filter(None,
                                        map(lambda l: l.get('supite_codigo_item'), result)))
            if len(codigo_itens) == 1:
                codigo_itens = "('%s')" % str(codigo_itens[0])

            saldo_itens = []
            if self.status_conexao_supervisor:
                try:
                    sql_saldo = """
                        SELECT  supsal_item AS item,
                                supsal_saldo AS saldo
                        FROM   supervisor_mov_saldo
                        WHERE  supsal_item IN %s
                        ORDER  BY supsal_item
                    """ % str(codigo_itens)

                    self.consulta_saldo = True
                    saldo_itens = exec_sql(sql_saldo, 'S', ret=True)
                    self.consulta_saldo = False
                except:
                    saldo_itens = []

            # LIMPA O WIDGET TABLE
            self.clear_widget_table(tableBuscaProduto)
            rows = 0

            if not isinstance(result, list):
                result = [result]
            dict_aux = {}
            for item in saldo_itens:
                dict_aux[item.get('item')] = item.get('saldo') if item.get('saldo') or item.get('saldo') > 0\
                                                               else Decimal(0)
            saldo_itens = dict_aux.copy()
            for i in result:
                item_name = i.get('name') if config['descricao_resumida'] else i.get('supite_descricao_completa')
                item_codigo = i.get('supite_codigo_item') or ''
                item_descricao = item_name or ''
                item_iat = i.get('supite_iat') or ''
                item_unidade = i.get('supite_unidade') or ''
                item_tributacao = i.get('trib.suptri_codigo_trib_ecf') or ''
                item_preco = i.get('supite_preco') or 0
                item_producao = i.get('supite_producao') or ''
                item_saldo = saldo_itens.get(item_codigo) or Decimal(0)

                if isinstance(item_saldo, list):
                    item_saldo = item_saldo[0]

                tableBuscaProduto.insertRow(rows)
                # POPULA TABELA
                # No. Item
                new_item = QtGui.QTableWidgetItem(item_codigo)
                new_item.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                new_item.setFont(self._font)
                # Descrição
                new_desc = QtGui.QTableWidgetItem(item_descricao[:50])
                new_desc.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                new_desc.setFont(self._font)
                # Pesavel
                iat_str = 'A'
                if item_iat == 'T':
                    iat_str = 'T'
                new_pesavel = QtGui.QTableWidgetItem(iat_str)
                new_pesavel.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                new_pesavel.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
                new_pesavel.setFont(self._font)
                prod = 'P'
                if item_producao == 'T':
                    prod = 'T'
                new_fracionado = QtGui.QTableWidgetItem(prod)
                new_fracionado.setFlags(
                    QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                new_fracionado.setTextAlignment(QtCore.Qt.AlignRight |
                                                QtCore.Qt.AlignVCenter)
                new_fracionado.setFont(self._font)
                # Unidade
                new_unid = QtGui.QTableWidgetItem(item_unidade)
                new_unid.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                new_unid.setFont(self._font)
                # Situação Tributária
                new_st = QtGui.QTableWidgetItem(item_tributacao)
                new_st.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                new_st.setFont(self._font)
                # Valor Unitário
                new_vuni = QtGui.QTableWidgetItem('%1.2f' % item_preco)
                new_vuni.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                new_vuni.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
                new_vuni.setFont(self._font)
                # Saldo
                saldo = item_saldo
                new_saldo = QtGui.QTableWidgetItem('%1.2f' % saldo)
                new_saldo.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                new_saldo.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
                new_saldo.setFont(self._font)

                # WIDGET OS NEW ITENS NA TABLE ITENS
                tableBuscaProduto.setItem(rows, 0, new_item)
                tableBuscaProduto.setItem(rows, 1, new_desc)
                tableBuscaProduto.setItem(rows, 2, new_pesavel)
                tableBuscaProduto.setItem(rows, 3, new_fracionado)
                tableBuscaProduto.setItem(rows, 4, new_unid)
                tableBuscaProduto.setItem(rows, 5, new_st)
                tableBuscaProduto.setItem(rows, 6, new_vuni)
                tableBuscaProduto.setItem(rows, 7, new_saldo)
                rows += 1
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtBuscaCliente)
            tableBuscaProduto.selectRow(0)
            if QtGui.QApplication.desktop().screenGeometry().width() > 832:
                tableBuscaProduto.resizeColumnsToContents()
            tableBuscaProduto.horizontalHeader().setStretchLastSection(True)

        else:
            self.infoText('Nenhum Item encontrado!!!', display=True)
            self.clear_widget_table(tableBuscaProduto)
            self.ui.edtBuscaCliente.clear()
        return True

    @Log
    def _returnPressed_edtBuscaCliente(self):
        text = self.ui.edtBuscaCliente.text()
        if not text:
            self.infoText('Digite algum texto', display=True)
            return text
        if self.ui.stackedBuscaCliPro.currentIndex() == 0:  # consulta cliente
            if not self._consulta_cliente(text):
                return False
        else: # consulta produto
            self._consulta_produto(text)

        return text

    @Log
    def _returnPressed_edtItemFicha(self):
        text = self.ui.edtItemFicha.text()
        if not text:
            self.infoText('Digite algum texto', display=True)
            return text

        cod_ficha = False
        valor = 0
        if self._state_screen == in_troca_ficha:
            text, cod_ficha, valor, desconto = self.obj_ficha.get_dados_item_ficha(text)
            self.ui.spinQtdItemFicha.setValue(1)
            self.ui.spinDescontoFicha.setValue(desconto)
            if not text:
                self.ui.edtItemFicha.clear()
                return False

        res_item = self._search_item(text)
        if not res_item:
            self.ui.edtItemFicha.clear()
            return False

        if self._state_screen == in_ficha and res_item.supite_producao == 'P':
            self.item_ficha = res_item
            self.cod_ficha = cod_ficha
            BuscaObsProduto(self).exec_()
            self.ui.lbl_data_entrega_ficha.setVisible(True)
            self.ui.lbl_cliente_ficha.setVisible(True)
            self.ui.edtDataEntregaFicha.setVisible(True)
            self.ui.edtClienteFicha.setVisible(True)
            self.ui.edtDataEntregaFicha.setText(u'%s' % datetime.now().strftime('%H:%M %d/%m/%Y '))
            return self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtDataEntregaFicha)

        self.add_item_ficha(res_item, cod_ficha=cod_ficha, valor_ficha=valor)
        return text

    @Log
    def _returnPressed_spinQtdItemFicha(self):
        qtd = self.ui.spinQtdItemFicha.value()
        if qtd <= 0:
            message_dlg(self, MSG_ERROR, 'Quantidade inválida, tente outro valor!')
            return True

        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtItemFicha)
        return True

    @Log
    def _returnPressed_edtDataEntregaFicha(self):
        try:
            data = datetime.strptime(self.ui.edtDataEntregaFicha.text(), '%H:%M %d/%m/%Y')
            if data < (datetime.now() - timedelta(minutes=5)):
                raise Exception
        except:
            message_dlg(self, MSG_ERROR, 'Atenção!\n\nData inválida!')
            self.ui.edtDataEntregaFicha.clear()
            return

        return self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtClienteFicha)

    @Log
    def  _returnPressed_edtClienteFicha(self):
        self.add_item_ficha(self.item_ficha, cod_ficha=self.cod_ficha)
        del self.item_ficha
        del self.cod_ficha
        self.obs_produto = ''
        self.ui.edtDataEntregaFicha.clear()
        self.ui.lbl_data_entrega_ficha.setVisible(False)
        self.ui.lbl_cliente_ficha.setVisible(False)
        self.ui.edtDataEntregaFicha.setVisible(False)
        self.ui.edtClienteFicha.setVisible(False)
        return self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtItemFicha)

    @Log
    def _returnPressed_spinDescontoFicha(self):
        desconto = self.ui.spinDescontoFicha.value()
        if desconto < 0:
            message_dlg(self, MSG_ERROR, 'Quantidade inválida, tente outro valor!')
            return True

        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtItemFicha)
        return True

    @Log
    def _returnPressed_tableItensFicha(self, row):
        item = self.ui.tableItensFicha.item(row, 0)
        desc = self.ui.tableItensFicha.item(row, 1)
        itemAtivo = self.ui.tableItensFicha.item(row, 6)
        itemSubtotal = Decimal(self.ui.tableItensFicha.item(row, 5).text())
        if itemAtivo.checkState() == QtCore.Qt.CheckState.Checked:
            item.setIcon(self.icon_delete)
            itemAtivo.setCheckState(QtCore.Qt.CheckState.Unchecked)
            self.infoText('Cancelado: %s - %s' % (item.text(), desc.text()))
            total = Decimal(Decimal(self.ui.lblTotalFichas.text().replace(',', '.')) -
                            itemSubtotal).quantize(TWOPLACES)
            self.ui.lblTotalFichas.setText(('%s' % total).replace('.', ','))

        else:
            self.infoText('O Item selecionado já foi cancelado!')

        sem_ficha = True
        for i in range(0, self.ui.tableItensFicha.rowCount()):
            if self.ui.tableItensFicha.item(i, 6).checkState() == QtCore.Qt.CheckState.Checked:
                sem_ficha = False
                break

        if sem_ficha:
            self._state_coupon = coupon_closed

        self.ui.tableItensFicha.clearSelection()
        self._tab_order = []
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtItemFicha)
        self._state_item = item_none
        self.ui.tableItensFicha.scrollToBottom()
        self.check_button_status()
        return

    @Log
    def operador_encontrado(self, operador):
        self.ui.lblInfoLogin.setText(self.trUtf8('Login efetuado com sucesso!'))
        self.ui.lblNomeUser.setText(operador.name)
        self.user_logged = operador
        Log.info('Operador: %s' % self.user_logged)
        self._operador_codigo = self.user_logged.supope_codigo
        self._operador_id = self.user_logged.id
        state = in_menu
        if self.z_pendente:
            if message_dlg(self, MSG_QUESTION,
                           'Deseja Emitir Redução-Z Pendente') == YES_ROLE:
                if self.par_sangria_x_z_suprimento and \
                                self.user_logged == 'O':
                    dlg = password_dlg(self, msg_info='')
                    if not dlg.retval:
                        return False

                ReducaoXZ(self).gerar_reducao_xz(self.loja.supcfc_loja, self.caixa_id, 'Z',
                                                 self.user_logged.supope_codigo)
                if not self._IdPve and not self.consulta_produto:
                    self.show_info_proccess(True, "Checando PVE's")
                    self.check_pves(days=2)
                    self.show_info_proccess(False)
                self.show_info_proccess(False)
                state = in_coupon
            else:
                state = in_menu
            self.show_info_proccess(False)

        mov_caixa = MovCaixa().search({'supcax_loja': config['loja'],
                                       'supcax_fechado': 'A'}, order_by='id DESC', limit=1)
        if mov_caixa:
            cx_pendente = mov_caixa.supcax_data_hora_abertura.date() < datetime.today().date()
            self.mov_caixa = mov_caixa
            if mov_caixa.supcax_operador_id == self._operador_id and \
                            mov_caixa.supcax_caixa == self.caixa_id:
                self.ui.edtUser.clear()
                self.ui.edtPwd.clear()
            else:
                other_operator = CaixaCadOperadores.get_by(id=mov_caixa.supcax_operador_id)
                if other_operator:
                    if operador.supope_operador_supervisor == 'S':
                        message_dlg(self, MSG_INFO,
                                    'Turno Aberto:\n\nOperador: %s\nData: %s\nLoja: %.6d '
                                    'Caixa: %.6d' % (other_operator.name,
                                                     mov_caixa.supcax_data_hora_abertura.
                                                     strftime("%d/%m/%Y %H:%M:%S"),
                                                     mov_caixa.supcax_loja,
                                                     mov_caixa.supcax_caixa),
                                    detail=False)
                    else:
                        if message_dlg(self, MSG_QUESTION,
                                       'Turno aberto para o operador {operador} no caixa: {caixa}\n'
                                       '{nome}\nDeseja encerrar este turno?'.format(
                                               operador=other_operator.supope_codigo,
                                               caixa=mov_caixa.supcax_caixa,
                                               nome=other_operator.name),
                                       detail=False) == YES_ROLE:
                            dlg_info = ''
                            if self.user_logged.supope_operador_supervisor == 'O':
                                if not self.user_logged.supope_libera_restricao:
                                    dlgpwd = password_dlg(self, msg_info=dlg_info)
                                    if dlgpwd.retval:
                                        self._libera_restricao = dlgpwd.user.id
                                        cx_pendente = True
                                    else:
                                        self.desbloqueia_importacao()
                                        return message_dlg(self, MSG_INFO,
                                                           'Operador: %s.\n'
                                                           '\nNão tem permissão para '
                                                           'executar esta operação' %
                                                           str(self.user_logged.name))
                                else:
                                    self._libera_restricao = self.user_logged.id
                                    cx_pendente = True
                        else:
                            state = in_menu_fiscal
                            self.user_logged = False
                            Log.info('Operador: %s' % self.user_logged)
                else:
                    cx_pendente = True

            if cx_pendente and self.user_logged:
                # o caixa deve ser do turno que esta aberto e nao o caixa atual
                caixa = mov_caixa.supcax_caixa
                retval = self.obj_abertura_fechamento_turno.fechar_turno(self.mov_caixa,
                                                                    config['loja'],
                                                                    caixa,
                                                                    (self.user_logged.id,
                                                                     self.user_logged.name))
            else:
                self.ui.lblNumCaixa.setText(u'%03d <b>Aberto' % self.caixa_id)
                message_dlg(self, MSG_INFO,
                            'Turno Aberto\n\n Data: %s'
                            % mov_caixa.supcax_data_hora_abertura.strftime("%d/%m/%Y %H:%M:%S"),
                            detail=False)
                state = in_coupon if self.user_logged else in_menu_fiscal

        else:
            if self.obj_abertura_fechamento_turno.abrir_turno(self.loja.supcfc_loja,
                                                         self.caixa_id,
                                                         self.user_logged.supope_codigo):
                state = in_coupon
            else:
                state = in_menu

        if state == in_coupon and config['ativar_restaurante']:
            state = in_restaurante

        self.verificar_confirmacao_tef()
        self.show_info_proccess(False)
        self._set_screen_state(state)
        return True

    @Log
    def _returnPressed_edtPwd(self):
        if self.bloqueia_campo_senha:
            return False

        self.bloqueia_importacao()
        self.bloqueia_campo_senha = True
        # try:
        #     arquivo = os.path.join(ROOT_PATH, "pdvsig.sig")
        #     if not os.path.exists(arquivo):
        #         self.desbloqueia_importacao()
        #         self.bloqueia_campo_senha = False
        #         raise Exception(
        #             '[Erro 2] Arquivo de Identificação da ECF não encontrado: %s' % arquivo)
        # except:
        #     self.tratar_except()
        #     self._set_screen_state(in_menu_fiscal)
        #     self.desbloqueia_importacao()
        #     self.bloqueia_campo_senha = False
        #     return

        text = self.ui.edtPwd.text()
        self.bloqueia_importacao()
        if self.ui.edtPwd.text():
            operador = CaixaCadOperadores.get_by(supope_codigo=self.ui.edtUser.text(),
                                                 active=True)
            if not operador:
                self.ui.lblInfoLogin.setText(self.trUtf8('Operador não cadastrado!'))

            if self.ui.edtPwd.text() == operador.supope_senha:
                self.operador_encontrado(operador)

            else:
                message_dlg(self, MSG_INFO,
                            '<br><br><center><h1><font color="red" size="10">Senha incorreta!'
                            '</font></h1>')
                self.ui.lblInfoLogin.setText(self.trUtf8( 'Senha não confere! Tente novamente.'))
                if self.teclado:
                    texto = 'Digite o usuario:\n' + str(self.ui.edtUser.text()) + '\n'
                    texto += 'Digite a senha:'
                    self.teclado.envia_display(texto)

                self.ui.edtPwd.clear()

        self.bloqueia_campo_senha = False
        self.desbloqueia_importacao()
        return len(text) * '*'

    @Log
    def _obter_peso_balanca(self, p_timeout=10):
        inc = 0
        self.show_info_proccess(True, 'Conectando com a balança')
        QtGui.QApplication.processEvents()
        while inc != p_timeout:
            try:
                if self._state_item == item_none:
                    Log.warning('Erro: %s' % STATE_ITEM_NAMES[self._state_item])
                    raise AssertionError('Item None')
                self._balanca = get_balanca(show_dialog=False)
                ret = self._balanca.read_data()
                if ret <= 0:
                    raise Exception('Não foi possivel obter o peso da Balança')
                self.show_info_proccess(False, '')
                return ret
            except DriverError as e:
                raise DriverError('Erro ao conectar com a Balança')

            except AssertionError:
                self.show_info_proccess(False)
                raise AssertionError('Erro balança')
            except Exception as e:
                self.show_info_proccess(True,
                                        '%s\nChecando a balança: %d/%d' % (e, inc + 1, p_timeout))
                QtGui.QApplication.processEvents()
                inc += 1
                continue

        if inc == p_timeout:
            self.show_info_proccess(False)
            raise InvalidState('Não foi possível obter o peso da Balança')


    @Log
    def _returnPressed_spinQtda(self):
        value = Decimal(str(self.ui.spinQtda.value())) if self.ui.spinQtda.text() else 0
        if value > 9999:
            self.ui.spinQtda.setValue(1)
            message_dlg(self, MSG_INFO, 'Atenção!\nValor não permitido, máximo 9999.')
            return self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinQtda)

        min_max = CaixaCfgParametros.get_by()
        if min_max and value:
            autorizacaosup_mult_itens = config['autorizacaomultiplicaitens']
            msg = ''
            if value < min_max.supcfp_qtd_minima and not self._state_item == \
                    item_balanca and not self._IdDav:
                msg = 'Qtda. digitada menor que a permitida!'
                if autorizacaosup_mult_itens:
                    dlg_info = ''
                    if message_dlg(self, MSG_QUESTION, msg +
                                    '\nDeseja liberar esta venda? \n\n%s' % dlg_info) == YES_ROLE:
                        if self.user_logged.supope_operador_supervisor == 'O':
                            if not self.user_logged.supope_libera_restricao:
                                dlgpwd = password_dlg(self, msg_info=dlg_info)
                                if dlgpwd.retval:
                                    self._libera_restricao = dlgpwd.user.id
                                else:
                                    return message_dlg(self, MSG_INFO,
                                                       'Operador: %s.\n\nNão tem permissão para '
                                                       'executar esta operação' %
                                                       str(self.user_logged.name))
                            else:
                                self._libera_restricao = self.user_logged.id
                    else:
                        self.ui.edtCodigo.clear()
                        self.ui.spinQtda.setValue(1)
                        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                        self._state_item = item_none
                        self.KEY_ESCAPE_Click()

            elif value > min_max.supcfp_qtd_maxima and not self._state_item == \
                    item_balanca and not self._IdDav:
                msg = 'Qtda. digitada maior que a permitida!'
                if autorizacaosup_mult_itens:
                    dlg_info = ''
                    if message_dlg(self, MSG_QUESTION, msg +
                                    '\nDeseja liberar esta venda? \n\n%s' % dlg_info) == YES_ROLE:
                        if self.user_logged.supope_operador_supervisor == 'O':
                            if not self.user_logged.supope_libera_restricao:
                                dlgpwd = password_dlg(self, msg_info=dlg_info)
                                if dlgpwd.retval:
                                    self._libera_restricao = dlgpwd.user.id
                                else:
                                    return message_dlg(self, MSG_INFO,
                                                       'Operador: %s.\n\nNão tem permissão para '
                                                       'executar esta operação' %
                                                       str(self.user_logged.name))
                            else:
                                self._libera_restricao = self.user_logged.id
                    else:
                        self.ui.edtCodigo.clear()
                        self.ui.spinQtda.setValue(1)
                        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                        self._state_item = item_none
                        self.KEY_ESCAPE_Click()
                else:
                    if msg and value:
                        if message_dlg(self, MSG_QUESTION,
                                       msg + '\nDeseja tentar novamente?') == YES_ROLE:
                            self.ui.spinQtda.setValue(1)
                            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinQtda)
                            return
                        else:
                            self.ui.edtCodigo.clear()
                            self.ui.spinQtda.setValue(1)
                            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                            self._state_item = item_none
                            self.KEY_ESCAPE_Click()
        if value <= 0:
            if message_dlg(self, MSG_QUESTION,
                           'Qtda. do Item não pode ser menor ou igual a zero!'
                           '\nDeseja tentar novamente?') == YES_ROLE:
                self.ui.spinQtda.clear()
            else:
                self.ui.spinQtda.setValue(1)
                self.ui.edtCodigo.clear()
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                self._state_item = item_none
                self.KEY_ESCAPE_Click()
        else:
            if self._state_item == item_add_quantity:
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                self._state_item = item_none
            elif self._state_item == item_balanca:
                item_quantity = Decimal(
                    str(self.ui.spinQtda.value())) if self.ui.spinQtda.value() > 0 else Decimal(1)
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                cod_item = self.ui.edtCodigo.text()
                res_item = self._search_item(cod_item)
                self.obj_cupom_fiscal.add_item(res_item, item_quantity,
                                           self.vendedor_codigo, self._nome_vendedor)
            elif self._state_item in [item_add_discount, item_add_surchage]:
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbTipoDesc)
        self.check_button_status()
        return self.ui.spinQtda.text()

    @Log
    def _returnPressed_cbTipoDesc(self):
        if self.ui.cbTipoDesc.currentIndex() == 0:
            self.ui.spinDecItemValor.setValue(0)
            self.ui.spinDecItemValor.setDecimals(2)
            self.ui.spinDecItemValor.setRange(0.0, 99999.99)
        elif self.ui.cbTipoDesc.currentIndex() == 1:
            self.ui.spinDecItemValor.setValue(0)
            self.ui.spinDecItemValor.setDecimals(2)
            self.ui.spinDecItemValor.setRange(0.0, 99.99)

        self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinDecItemValor)
        return self.ui.cbTipoDesc.currentText()

    @Log
    def get_desconto_acrescimo_restringido(self, percent_desconto, value):
        '''
        Se for dado desconto/acrescimo percentual na capa e tiver marcado o parametro do supervisor
        'supcfp_restringe_desconto', o desconto percentual será em cima do valor total dos itens
        que nao participaram de nenhuma promocao ou nao tiveram desconto.
        :param percent_desconto:
        :return:
        '''
        if not percent_desconto:
            return value

        parametros = CaixaCfgParametros.get_by()
        if not parametros or not parametros.supcfp_restringe_desconto:
            total_itens = sum(map(lambda y: y.supcor_preco_total - y.supcor_desconto_item,
                            filter(lambda x: x.supcor_venda_cancelamento == 'V' ,
                                   self.mov_corpos)))
            value = Decimal((total_itens * percent_desconto) / 100).quantize(TWOPLACES)
            return value

        # soma os itens filtrando os que nao possuem desconto, sao do tipo venda(exclui cancelados)
        # e nao possuem  tipo de promocao, pois podem ter itens sem desconto mas participando de
        # promocao
        total_itens = sum(map(lambda y: y.supcor_preco_total,
                              filter(lambda x: not x.supcor_desconto_item and
                                                   x.supcor_venda_cancelamento == 'V' and
                                               not x.supcor_tipo_promo,
                                     self.mov_corpos)))
        value = Decimal((total_itens * percent_desconto) / 100).quantize(TWOPLACES)
        return value

    def verifica_valor_desconto(self, value):
        '''Verifica se o valor do desconto dado na capa, ao reaterar para os itens, ficara algum
            item com desconto igual ou maior que o valor do item, se isto ocorrer nao permite 
            aplicar este desconto.'''
        self.atualiza_valor_a_pagar()
        qtd_corpos = len(filter(lambda x: x.supcor_venda_cancelamento != 'C', self.mov_corpos))
        total_cupom = Decimal(self.ui.lblVlrTotalAPagar.text())
        if value >= (total_cupom - (qtd_corpos / Decimal(100))):
            message_dlg(self, MSG_ERROR, 'Atenção!\nValor de desconto maior/igual que total da '
                                         'nota menos quantidade de itens:\nDesconto: R$ %s'
                                         '\nTotal Cupom: R$ %s \nQtd. Itens: %s' %
                                         (value, total_cupom, qtd_corpos))
            return False

        # se o desconto representrar menos de 80 por cento do valor do cupom nao verifica o desconto
        percent = value / total_cupom * 100
        if percent < 80:
            return True


        seq_desconto = {}
        map(lambda x: seq_desconto.update({x.get('sequencia_item'): x.get('desconto')}),
            self.list_itens_desconto_promo)

        for i in self.dict_mov_corpos_socio:
            if i in seq_desconto:
                continue

            seq_desconto[i] = self.dict_mov_corpos_socio.get(i).get('desconto')

        desc_calculado = 0
        desc_maior_item = 0
        maior_item = False
        for corpo in self.mov_corpos:
            if corpo.supcor_venda_cancelamento == 'C':
                continue

            outros_desc = (seq_desconto.get(corpo.supcor_sequencia) or 0) + \
                          corpo.supcor_desconto_item
            desconto = Decimal(((corpo.supcor_preco_total - outros_desc) * percent) / 100 +
                                 outros_desc).quantize(TWOPLACES, rounding=ROUND_UP)
            if not maior_item or maior_item.supcor_preco_total < corpo.supcor_preco_total:
                maior_item = corpo
                desc_maior_item = desconto

            desc_calculado += desconto
            if desconto >= corpo.supcor_preco_total - corpo.supcor_desconto_item + \
                    corpo.supcor_acrescimo_item:
                message_dlg(self, MSG_ERROR, 'Atenção!\nValor de desconto inválido!\n'
                              'O valor do desconto rateado execede o próprio valor em alguns itens')
                return False

        dif = value - desc_calculado
        if dif <= Decimal('-0.01') or Decimal('0.01') <= dif:
            desc_maior_item += dif + maior_item.supcor_desconto_item - maior_item.supcor_acrescimo_item
            if desc_maior_item >= maior_item.supcor_preco_total:
                message_dlg(self, MSG_ERROR, 'Atenção!\nValor de desconto inválido!\n'
                              'O valor do desconto rateado execede o próprio valor em alguns itens')
                return False

        return True


    @Log
    def _returnPressed_spinDecItemValor(self):
        if self._state_item == item_add_discount:
            if self.user_logged.supope_operador_supervisor == 'O' \
                    and not self.user_logged.supope_desconto_item \
                    and not self._IdDav and not self._IdPve:
                dlg = password_dlg(self, msg_info='Desconto')
                if not dlg.retval:
                    message_dlg(self, MSG_INFO, 'Operador: %s.'
                                                '\n\nNão tem permissão para executar esta operação'
                                % (dlg.user.name))
                    return
                else:
                    self._libera_desconto = dlg.user.id

            else:
                self._libera_desconto = self.user_logged.id

        value = Decimal(str(self.ui.spinDecItemValor.value())).quantize(TWOPLACES) or 0
        percent_desconto = 0
        if self.ui.cbTipoDesc.currentIndex() == 1: # precentrual
            if not self.ui.edtCodigo.text(): # calcula desconto/acrescrimo para capa
                percent_desconto = value
                value = Decimal((self._coupon_amount * value) / 100).quantize(TWOPLACES)
            else: # calcula desconto/acrescrimo para item
                qtd = Decimal(self.ui.spinQtda.value())
                value = Decimal(((self._search_item(self.ui.edtCodigo.text()).supite_preco *
                                  value * qtd)) / 100).quantize(TWOPLACES)

        st_cupom = self._state_coupon
        item_quantity = Decimal(str(self.ui.spinQtda.value()))
        if value <= 0:
            if message_dlg(self, MSG_QUESTION,
                           'Valor do Desconto/Acréscimo não pode ser menor ou igual a zero!\n'
                           'Deseja tentar novamente?') == YES_ROLE:
                self.ui.spinDecItemValor.clear()

            else:
                self._clear_add_item_st()

            self.check_button_status()
            return

        elif value > (Decimal((self._coupon_amount) - Decimal(
                '0.01')) * item_quantity) and self._coupon_amount and not self.ui.edtCodigo.text():
            if message_dlg(self, MSG_QUESTION,
                           'Valor do Desconto/Acréscimo não pode ser maior que  99.99 % do cupom!'
                           '\nDeseja tentar novamente?') == YES_ROLE:
                self.ui.spinDecItemValor.clear()

            else:
                self._state_coupon = self._state_coupon_old
                self._clear_add_item_st()

            self.check_button_status()
            return

        elif self.ui.edtCodigo.text() and value > (self._search_item(self.ui.edtCodigo.text()).supite_preco
                                                   * item_quantity):
            if message_dlg(self, MSG_QUESTION,
                           'Valor do Desconto/Acréscimo não pode ser maior que  99.99 % do item!'
                           '\nDeseja tentar novamente?') == YES_ROLE:
                self.ui.spinDecItemValor.clear()
            else:
                self._clear_add_item_st()

            self.check_button_status()
            return

        else:
            # desconto ou acrescimo na capa do cupom
            if self._state_coupon in [coupon_add_discount, coupon_add_surchage] and value > 0:
                total_anterior = self._coupon_amount
                if self._state_coupon == coupon_add_discount: # desconto
                    self.ui.lblResumo_9.setText('Descon. Cupom')
                    if value >= self._coupon_amount:
                        if message_dlg(self, MSG_QUESTION,
                                       'Valor do desconto não pode ser maior ou igual o total do '
                                       'cupom\nDeseja tentar novamente ?') == YES_ROLE:
                            self.ui.spinDecItemValor.clear()
                        else:
                            self._state_coupon = coupon_opened
                            self._state_item = item_none
                            self._clear_add_item_st()
                        return
                    try:
                        self.obj_cad_promo_capa.calcula_descontos_promo() # nao alterar esta ordem
                        self.obj_cupom_fiscal.get_desconto_socio_torcedor()# nao alterar esta ordem
                        self.obj_cad_funcionarios.get_desconto_funcionario()# nao alterar esta ordem
                        premite_deconto = self.verifica_valor_desconto(value)
                        if not premite_deconto:
                            self._state_coupon = coupon_opened
                            self._state_item = item_none
                            return self._clear_add_item_st()

                        self.obj_cupom_fiscal.aplica_desc_itens()           # nao alterar esta ordem
                        value = self.get_desconto_acrescimo_restringido(percent_desconto, value) # nao alterar esta ordem
                        self._coupon_totalize = self.obj_cupom_fiscal.totalizar_cupom()
                        self.infoText('Cupom totalizado! Efetue o pagamento.', display=True)
                    except:
                        self.tratar_except()
                        self._state_coupon = coupon_opened
                        return self._clear_add_item_st()

                    self.check_button_status()
                    self._coupon_amount -= Decimal(str(value))
                    self._coupon.supcap_alteracao = 'D'
                elif self._state_coupon == coupon_add_surchage: #acrescimo
                    self.ui.lblResumo_9.setText(u'Acrésc. Cupom')
                    try:
                        self.obj_cad_promo_capa.calcula_descontos_promo()# nao alterar esta ordem
                        self.obj_cupom_fiscal.get_desconto_socio_torcedor()# nao alterar esta ordem
                        self.obj_cad_funcionarios.get_desconto_funcionario()# nao alterar esta ordem
                        self.obj_cupom_fiscal.aplica_desc_itens()# nao alterar esta ordem
                        value = self.get_desconto_acrescimo_restringido(percent_desconto, value) # nao alterar esta ordem
                        self._coupon_totalize = self.obj_cupom_fiscal.totalizar_cupom()
                        self.infoText('Cupom totalizado! Efetue o pagamento.', display=True)
                    except:
                        self.tratar_except()
                        self.ui.edtCodigo.clear()
                        return self._clear_add_item_st()

                    self.check_button_status()
                    self._coupon_amount += Decimal(str(value))
                    self._coupon.supcap_alteracao = 'A'

                self._add_desc_acres_in_table(self._state_coupon, value)
                self.ui.lblInfoDescCapa.setText(u'%1.2f' % value)
                sess.begin()
                self._coupon.supcap_total_cupom = self._coupon_amount
                self._coupon.supcap_valor_desc_acres = value + self._coupon.supcap_valor_desc_acres
                sess.commit()

                self._state_coupon = coupon_finish
                self.atualiza_valor_a_pagar()
                self.ui.lblVlrReceber.setText(u'%.2f' % float(self.ui.lblVlrTotalAPagar.text()))
                self.ui.lblVlrTotal.setText(u'%.2f' % self._coupon_amount)
                self.ui.lblVlrTotal_2.setText(u'%1.2f' % self._coupon_amount)
                self.ui.lblVlrTotalR.setText(u'%.2f' % float(self.ui.lblVlrTotalAPagar.text()))
                self.check_button_status()
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.stackedActions)

            # desconto ou acrescimo no item
            elif self._state_item in [item_add_discount, item_add_surchage]:
                try:
                    item_quantity = Decimal(
                        str(self.ui.spinQtda.value())) if self.ui.spinQtda.value() > 0 else Decimal(
                        1)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                    serie = self.ui.cbSerie.currentText() if self.ui.frmSerie.isVisible() else ''
                    cod_item = self.ui.edtCodigo.text()
                    res_item = self._search_item(cod_item)
                    discount = [self._state_item, value]

                    self.obj_cupom_fiscal.add_item(res_item, item_quantity, self.vendedor_codigo,
                                              self._nome_vendedor, discount, serie=serie)

                except Exception as err:
                    message_dlg(self, MSG_INFO, str(err))
                    return self._clear_add_item_st()
        return self._clear_add_item_st()

    @Log
    def _returnPressed_cbSerie(self):
        if self._state_item in [item_add_discount, item_add_surchage]:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbTipoDesc)
        else:
            cod_item = self.ui.edtCodigo.text()
            res_item = self._search_item(cod_item)
            self.obj_cupom_fiscal.add_item(res_item,
                                       Decimal(str(self.ui.spinQtda.value())),
                                       self.vendedor_codigo, self._nome_vendedor,
                                       serie=self.ui.cbSerie.currentText())
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
            return self._clear_add_item_st()

    @Log
    def _clear_add_item_st(self):
        self.ui.frmSerie.setVisible(False)
        self.ui.frmDescAcre.setVisible(False)
        self.ui.spinDecItemValor.setValue(0)
        self.ui.cbTipoDesc.setCurrentIndex(0)
        self.ui.cbSerie.clear()
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
        self.ui.edtCodigo.clear()
        self.ui.spinQtda.setDecimals(0)
        self.ui.spinQtda.setValue(1)
        self.ui.lblItems.clear()
        self.ui.lblVlrItem.setText(u'%1.2f' % 0.0)
        self._state_item = item_none
        self.check_button_status()
        return True

    def check_promo_dinheiro(self):
        if not self.list_itens_desconto_promo:
            return True

        somente_dinheiro = all(map(lambda x: x.get('somente_dinheiro'),
                                   self.list_itens_desconto_promo))
        if not somente_dinheiro:
            return True

        if Decimal(self.ui.spinPgtoDinheiro.value()).quantize(TWOPLACES) < \
                Decimal(self.ui.lblVlrReceber.text()).quantize(TWOPLACES):
            self.obj_cad_promo_capa.calcula_descontos_promo()
            self.atualiza_valor_a_pagar(True)
            message_dlg(self, MSG_INFO, 'Atenção!\nPagamento parcial perde desconto '
                                        'da promoção em Dinheiro!')
            return False

        return True

    @Log
    def _returnPressed_spinPgtoDinheiro(self):
        if not self.verifica_certificado():
            return True

        if not self.check_promo_dinheiro():
            return True

        try:
            Decimal(self.ui.spinPgtoDinheiro.text()[3:].replace(',', '.'))
        except:
            return message_dlg(self, MSG_ERROR, u'Atenção!\n Valor inválido: %s' %
                               self.ui.spinPgtoDinheiro.text())

        if self._keyboard_lock:
            return 'Teclado bloqueado'

        self._lock_keyboard(True)
        value = self.ui.spinPgtoDinheiro.value()
        if self._state_coupon == coupon_add_discount:
            if not self._coupon_totalize:
                try:
                    self._coupon_totalize = self.obj_cupom_fiscal.totalizar_cupom()
                    self.infoText('Cupom totalizado! Efetue o pagamento.', display=True)
                except Exception:
                    self.tratar_except()
                    self._coupon_totalize = False
                    self._lock_keyboard(False)
                    return

                self.infoText('Desconto R$ %.2f Adicionado' % value)
                self._coupon_amount -= Decimal(str(value))
                self.atualiza_valor_a_pagar()
                self.ui.lblVlrReceber.setText(u'%.2f' % float(self.ui.lblVlrTotalAPagar.text()))
                self.ui.lblVlrTotal.setText(u'%.2f' % self._coupon_amount)
                self.ui.lblVlrTotalR.setText(u'%.2f' % float(self.ui.lblVlrTotalAPagar.text()))
                sess.begin()
                self._coupon.Alteracao = 'D'
                self._coupon.supcap_valor_desc_acres = value
                sess.commit()

        self.obj_cupom_fiscal.add_payment(Decimal(str(value)))
        if self.teclado and Decimal(self.ui.lblVlrTroco.text()):
            texto = 'Troco:\nR$%s' % str(self.ui.lblVlrTroco.text())
            if 'smak' in config['teclado_selecionado']:
                self.teclado.envia_linha_display(texto,2)
            else:
                self.teclado.envia_linha_display(texto,3)

        self._lock_keyboard(False)
        return value

    @Log
    def _returnPressed_tableBuscaPve(self, row=None, IdPve=None, cancela=False):
        if not row:
            row = self.ui.tableBuscaPve.currentRow()
        if not self.user_logged:
            message_dlg(self, MSG_INFO, 'Efetue login')
            self._set_screen_state(in_login)
            return
        Log.warning('%s, %s' % (row, IdPve))
        if IdPve:
            IdPreVenda = IdPve.lstrip('0')
        else:
            IdPreVenda = self.ui.tableBuscaPve.item(row, 0).text().lstrip('0')
        self.show_info_proccess(True, msg='Processando PVE No. %s' % IdPreVenda)
        try:
            capa = SupervisorPveCapa.get_by(id=IdPreVenda)
            if capa.suppva_restricao != 1:
                msg = {-1: '"Cancelado"',
                        0: "'PVE aberto! O mesmo deve ser concluído para ser faturado!'",
                        1: '"Aguardando Emissão"',
                        2: '"Faturado / Fechado"',
                        4: '"Anulado"',
                        5: '"Faturando (Está sendo impresso no SigFrente)"',
                }.get(capa.suppva_restricao) or ''
                self.show_info_proccess(False, '')
                return message_dlg(self, MSG_INFO, 'Este PVE está %s! \n\nAtualize a tela!' % msg)

            corpo = SupervisorPveCorpo.query.filter(SupervisorPveCorpo.suppvo_capa_id == IdPreVenda).all()
            listItens = []
            for itemPve in corpo:
                dict = {}
                dict['Item'] = itemPve.suppvo_item
                dict['DescontoItem'] = itemPve.suppvo_desconto_item
                dict['AcrescimoItem'] = itemPve.suppvo_acrescimo_item
                dict['Quantidade'] = itemPve.suppvo_quantidade
                dict['supcor_venda_cancelamento'] = itemPve.suppvo_venda_cancelamento
                listItens.append(dict)

                b_sess.begin()
                if not capa.suppva_restricao in (-1, -2):
                    capa.suppva_restricao = 5  # Imprimindo PVE
                b_sess.commit()
                self._IdPve = capa
        except Exception:
            self.tratar_except()
            return

        if capa.suppva_cliente:
            self._search_cliente(capa.suppva_cliente, False, troca_cli=True)
        self._set_screen_state(in_coupon)

        def add_item(item):
            self._lock_keyboard(True)
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
            self.ui.edtCodigo.setText(item)
            QtGui.QApplication.processEvents()
            self._lock_keyboard(False)

            return self.KEY_RETURN_Click()

        for x in listItens:
            self.ui.spinQtda.setDecimals(3)
            if not self._IdPve:
                break

            if x['DescontoItem'] > 0:
                self.KEY_DESCONTO_Click()
                add_item(x['Item'])
                if x['Quantidade'] > 0:
                    self.ui.spinQtda.setValue(x['Quantidade'])

                self.KEY_RETURN_Click()
                self.KEY_RETURN_Click()
                self.ui.spinDecItemValor.setValue(Decimal(x['DescontoItem']).quantize(TWOPLACES))
                self.KEY_RETURN_Click()
            elif x['AcrescimoItem'] > 0:
                self.KEY_ACRESCIMO_Click()
                add_item(x['Item'])
                if x['Quantidade'] > 0:
                    self.ui.spinQtda.setValue(x['Quantidade'])

                self.KEY_RETURN_Click()
                self.KEY_RETURN_Click()
                self.ui.spinDecItemValor.setValue(Decimal(x['AcrescimoItem']).quantize(TWOPLACES))
                self.KEY_RETURN_Click()

            elif x['Quantidade'] > 0:
                self.ui.spinQtda.setValue(x['Quantidade'])
                add_item(x['Item'])
                self.KEY_RETURN_Click()
            else:
                add_item(x['Item'])
            if x['supcor_venda_cancelamento'] == 'C':
                self.KEY_CANCEL_ITENS_Click()
                self.KEY_RETURN_Click()

        if capa.suppva_descontos > 0:
            self.KEY_DESCONTO_Click()
            self.KEY_RETURN_Click()
            self.KEY_RETURN_Click()
            self.ui.spinDecItemValor.setValue(capa.suppva_descontos)
            self.KEY_RETURN_Click()

        if capa.suppva_restricao == -1 or cancela and self._IdPve:
            self._PveCancelamento = True
            self.KEY_DINHEIRO_Click()
            self.KEY_RETURN_Click()
            self.cancelar_nota_fiscal(automatico=True)
        elif capa.suppva_restricao == -2:  # Anular pve
            self._PveCancelamento = True
            self.cancelar_nota_fiscal(automatico=True)
            try:
                Log.info('Gravando PVE #: %s' % self._IdPve.id)
                b_sess.begin()
                self._IdPve.suppva_data_cupom = self._date_coupon
                self._IdPve.suppva_nro_cupom = self._coupon_id
                self._IdPve.suppva_loja = self.loja.supcfc_loja
                self._IdPve.suppva_caixa = self.caixa_id
                b_sess.commit()
            except Exception:
                self.tratar_except()
            self._clear_globals(coupon_closed)
            self._set_screen_state(in_coupon)
            self._set_controls_default_values()
            self.check_button_status()
        else:
            if not self._coupon_totalize and self._IdPve:
                try:
                    self._coupon_totalize = self.obj_cupom_fiscal.totalizar_cupom()
                    # self._state_coupon = coupon_opened
                    self.infoText('Cupom totalizado! Efetue o pagamento.', display=True)
                    self.atualiza_valor_a_pagar()
                    self.ui.lblVlrReceber.setText(u'%.2f' % float(self.ui.lblVlrTotalAPagar.text()))
                    self.ui.lblVlrTotal.setText(u'%.2f' % self._coupon_amount)
                    self.ui.lblVlrTotalR.setText(u'%.2f' % float(self.ui.lblVlrTotalAPagar.text()))
                    self.check_button_status()
                except Exception:
                    self.tratar_except()
                    return
        self.show_info_proccess(False)

    @Log
    def _returnPressed_tableMFBuscaProduto(self, row, key):
        @Log
        def retorna_menu():
            self._set_screen_state(in_menu)
            self.ui.stackedMenus.setCurrentIndex(1)
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.mainMenu)
            self.ui.edtMFBuscaProduto.clear()
            self.ui.cbMFSubcategoria.setCurrentIndex(0)
            self.ui.tableMFBuscaProduto.clearContents()
            self.ui.tableMFProdutos.clearContents()
            self.show_info_proccess(False, '')
            self.clear_widget_table(self.ui.tableMFBuscaProduto)

        cod_item = self.ui.tableMFBuscaProduto.item(row, 0).text()

        tableProdutos = self.ui.tableMFProdutos
        if cod_item and key == QtCore.Qt.Key_Right:
            if tableProdutos.findItems(cod_item, QtCore.Qt.MatchExactly):
                # Verifica se o item já consta na tabela de selecionados
                self.infoText('Item %s já adicionado.' % cod_item)
            else:
                self.infoText('Adicionado item %s.' % cod_item)
                item = CaixaCadItens.get_by(supite_codigo_item=cod_item,
                                            supite_loja=self.loja.supcfc_loja)
                tableProdutos.setEnabled(False)
                rows = tableProdutos.rowCount()
                tableProdutos.insertRow(rows)
                # POPULA TABELA
                # Cod. Item
                newItem = QtGui.QTableWidgetItem(str(item.supite_codigo_item))
                newItem.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                # Descrição
                newDesc = QtGui.QTableWidgetItem(item.name)
                newDesc.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                # Unidade
                newUnid = QtGui.QTableWidgetItem(item.supite_unidade)
                newUnid.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                # VERIFICAR SALDO
                saldo = CaixaMovSaldo.get_by(supsal_loja=self.loja.supcfc_loja,
                                             supsal_item=cod_item)
                if saldo:
                    mensuracao = '+' if saldo.Saldo >= 0 else '-'
                    qtda = saldo.Saldo
                    data = saldo.Data.strftime('%d/%m/%Y')
                else:
                    mensuracao = '-'
                    qtda = 0.0
                    data = datetime.now().date().strftime('%d/%m/%Y')

                # Mensuracao
                newMensuracao = QtGui.QTableWidgetItem(mensuracao)
                newMensuracao.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                newMensuracao.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
                # Quantidade
                newQtda = QtGui.QTableWidgetItem('%.3f' % qtda)
                newQtda.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                newQtda.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
                # Data
                newData = QtGui.QTableWidgetItem(data)
                newData.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                newData.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

                # WIDGET OS NEW ITENS NA TABLE ITENS
                tableProdutos.setItem(rows, 0, newItem)
                tableProdutos.setItem(rows, 1, newDesc)
                tableProdutos.setItem(rows, 2, newUnid)
                tableProdutos.setItem(rows, 3, newMensuracao)
                tableProdutos.setItem(rows, 4, newQtda)
                tableProdutos.setItem(rows, 5, newData)
                tableProdutos.scrollToBottom()
                tableProdutos.resizeColumnsToContents()
                tableProdutos.horizontalHeader().setStretchLastSection(True)
        elif cod_item and key == QtCore.Qt.Key_Left:
            if tableProdutos.rowCount() > 0:
                del_item = tableProdutos.findItems(cod_item, QtCore.Qt.MatchExactly)
                if del_item:
                    self.infoText('Item %s removido.' % del_item[0].text(), display=True)
                    tableProdutos.removeRow(del_item[0].row())
                else:
                    self.infoText('Não há itens para remover')
        elif key == QtCore.Qt.Key_Return:
            if tableProdutos.rowCount() > 0:
                if message_dlg(self, MSG_QUESTION,
                               'Deseja gerar o arquivo Eletrônico do Estoque') == YES_ROLE:
                    QtGui.QApplication.processEvents()
                    self.show_info_proccess(True, 'Gerando arquivo Eletrônico do Estoque')
                    QtGui.QApplication.processEvents()
                    itens = []
                    for r in range(tableProdutos.rowCount()):
                        b = tableProdutos.item(r, 0).text()
                        itens.append(str(b))

                    try:
                        loja = get_loja()
                        p1 = gerar_registro_ident_estab('E1')
                        p2 = gerar_registro_relacao_merc(loja.supcfc_cnpj, 'E2',
                                                         loja.supcfc_loja, itens=itens)
                        p9 = gerar_registro_totalizacao(loja.supcfc_cnpj,
                                                        loja.supcfc_inscricao_estadual,
                                                        p2[1], 0, 'E9')
                        filename = os.path.join(ROOT_PATH, 'EstoqueParcial.txt')
                        f = open(filename, 'w')
                        f.write(p1 + p2[0] + p9)
                        f.close()
                        assinatura_ead(filename)
                        self.show_info_proccess(False, '')
                        message_dlg(self, MSG_INFO, 'Arquivo gerado:', file=filename)

                    except Exception as e:
                        self.tratar_except()
                        retorna_menu()
                else:
                    retorna_menu()
            else:
                message_dlg(self, MSG_INFO, 'Não há dados para gerar o arquivo')
            retorna_menu()
        return

    @Log
    def _returnPressed_tableBuscaDav(self, row):
        try:
            return self.faturar_dav(row)
        except Exception as e:
            self.emitindo_dav = False
            return message_dlg(self, MSG_ERROR, 'Atenção!\n Erro ao faturar DAV: %s' % str(e))

    @Log
    def faturar_dav(self, row):
        if self.emitindo_dav:
            return

        self.emitindo_dav = True
        if not self.user_logged:
            message_dlg(self, MSG_INFO, 'Efetue login')
            self._set_screen_state(in_login)
            self.emitindo_dav = False
            return

        id_dav = None
        loja = self.ui.tableBuscaDav.item(row, 0).text().lstrip('0')
        cod_dav = self.ui.tableBuscaDav.item(row, 1).text().lstrip('0')

        if config['aplicativo_dav'] == 'Local':
            pass
        elif config['aplicativo_dav'] == 'SiG-ERP':
            self.infoText('Conectando no SiG-ERP...', anim=False)
            QtGui.QApplication.processEvents()
            if not self.conectado:
                return message_dlg(self, MSG_INFO,
                               'Sem conexão com o SiG-ERP!\nNão foi possível faturar o dav.')

            capa = SupervisorDavCapa.get_by(supdva_nro_pedido=cod_dav, supdva_loja=loja)
            id_dav = capa.id
            try:
                if cancelar_pedidos([id_dav]):
                    message_dlg(self, MSG_ERROR, 'PVE-%s Emp.: %s foi cancelado no SigERP!' % (
                        self.ui.tableBuscaDav.item(row, 1).text(),
                        self.ui.tableBuscaDav.item(row, 0).text()))
                    self.emitindo_dav = False
                    return self.KEY_DAV_Click()

            except ErrConnRPC as e:
                message_dlg(self, MSG_ERROR, 'Atencao!\n %s' % e)
                self.emitindo_dav = False
                return False

            except Exception as e:
                message_dlg(self, MSG_ERROR, 'Atencao!\n Erro: %s' % e)
                self.emitindo_dav = False
                return False

            state_pedido, exportado = get_state_pedido_dav(id_dav)
            if state_pedido == 'F': # pedido faturado
                b_sess.begin()
                capa.supdva_restricao = 2
                b_sess.commit()
                message_dlg(self, MSG_INFO, 'O pedido PVE-%010d ja esta faturado no SigERP!' % int(cod_dav))
                self.emitindo_dav = False
                return False
            elif state_pedido == 'R': # pedido em esboco
                message_dlg(self, MSG_INFO, 'O pedido PVE-%010d não é mais válido no SigERP!' % int(cod_dav))
                self.emitindo_dav = False
                return False
            elif state_pedido == 'C': # pedido cancelado
                b_sess.begin()
                capa.supdva_restricao = -1
                b_sess.commit()
                message_dlg(self, MSG_INFO, 'O pedido PVE-%010d foi cancelado no SigERP!' % int(cod_dav))
                self.emitindo_dav = False
                return False
            elif state_pedido == 'I': # pedido nao existe mais no erp
                b_sess.begin()
                capa.supdva_restricao = -1
                b_sess.commit()
                message_dlg(self, MSG_INFO, 'O pedido PVE-%010d não existe mais no SigERP!' % int(cod_dav))
                self.emitindo_dav = False
                return False
            elif not exportado:
                msg = 'O pedido PVE-%010d foi alterado no SigERP!\n\nRecarreque a tela de DAV' \
                          % int(cod_dav)
                self.emitindo_dav = False
                message_dlg(self, MSG_INFO, msg)
                return False

        elif config['aplicativo_dav'] == 'SiG-Gestao':
            importar_davs_gestao(int(cod_dav))
            capa = SupervisorDavCapa.get_by(supdva_nro_pedido=cod_dav)
            id_dav = capa.id

        if id_dav:
            if capa.supdva_restricao == 3:
                msg_detail = '\n\n\n' + capa.Avisos.replace('|', '\n') if capa.Avisos else ''
                message_dlg(self, MSG_INFO, u'O Dav #%010d possui restrições. %s' %
                            (capa.IdDav, msg_detail))
                self.emitindo_dav = False
                return False

            corpos = SupervisorDavCorpo.query. \
                filter(SupervisorDavCorpo.supdvo_capa_id == id_dav).all()
            msg = []
            cod_itens = "'%s'" % "','".join(map(lambda x: x.supdvo_item, corpos))
            itens_invalidos = exec_sql(
            """SELECT sci.supite_codigo_item AS cb,
                      sci.name
               FROM supervisor_cad_itens AS sci
               INNER JOIN supervisor_dav_corpo AS sdc ON sdc.supdvo_item = sci.supite_codigo_item
               WHERE sci.supite_codigo_item IN (%s)
                 AND (sdc.supdvo_quantidade - trunc(sdc.supdvo_quantidade)) > 0
                 AND (sci.supite_fracionado IS null OR sci.supite_fracionado = False)
                 AND sdc.supdvo_capa_id = %s
                 """ % (cod_itens, id_dav), 'S', ret=True)
            map(lambda x: msg.append('%s - %s' % (x.get('cb'), x.get('name'))), itens_invalidos)
            if msg:
                message_dlg(self, MSG_ERROR,
                            'Quantidade fracionada não permitida pela embalagem nos itens:\n%s'
                            % '\n'.join(msg) )
                self.emitindo_dav = False
                return False

            # verifica se os itens estao no banco de dados
            msg_trib = []
            for dav_corpo in corpos:
                item = self._search_item(dav_corpo.supdvo_item)
                if not item:
                    msg.append('%s - %s' % (dav_corpo.supdvo_item, dav_corpo.supdvo_descricao))
                    continue

                trib_icms = CaixaCadTributacaoNfce.get_by(suptnf_codigo=item.supite_tributacao_icms_id)
                trib_pis = CaixaCadTributacaoNfce.get_by(suptnf_codigo=item.supite_tributacao_pis_id)
                trib_cofins = CaixaCadTributacaoNfce.get_by(suptnf_codigo=item.supite_tributacao_cofins_id)
                msg_tribs = []
                if not item.supite_tributacao_icms_id or not trib_icms:
                    msg_tribs.append('ICMS')
                if not item.supite_tributacao_pis_id or not trib_pis:
                    msg_tribs.append('PIS')
                if not item.supite_tributacao_cofins_id or not trib_cofins:
                    msg_tribs.append('COFINS')

                if msg_tribs:
                    msg_trib.append('%s - %s: %s' % (dav_corpo.supdvo_item,
                                                     dav_corpo.supdvo_descricao,
                                                     ', '.join(msg_tribs)))

            if msg: # se tiver itens inconcistentes nao deixa faturar o dav
                message_dlg(self, MSG_ERROR, 'Itens não encontrados ou \n'
                                             'quantidade não permitida pela embalagem:\n%s'
                                            % '\n'.join(msg) )
                self.emitindo_dav = False
                return False

            if msg_trib:
                message_dlg(self, MSG_ERROR, 'Itens sem tributação: %' % '\n'.join(msg_trib) )
                self.emitindo_dav = False
                return False

            fechamentos = SupervisorDavFechamento.query. \
                filter(SupervisorDavFechamento.supdvf_capa_id == id_dav).order_by('supdvf_sequencia').all()

            self._DavPgtoManual = True if not fechamentos else False
            self._IdDav = capa

            if capa.supdva_cliente:
                self.KEY_CLIENTE_Click()
                self.ui.edtOCodCliente.setText(str(capa.supdva_cliente))
                self.KEY_RETURN_Click()

            self._set_screen_state(in_coupon)
            QtGui.QApplication.processEvents()
            if capa.supdva_vendedor_id:
                vend_dav = SupervisorCadVendedor.query.filter(
                    SupervisorCadVendedor.id == capa.supdva_vendedor_id).all()
                for v in vend_dav:
                    self.KEY_VENDEDOR_Click()
                    self.ui.edtCodigo.setText(str(v.supven_codigo))
                    self.KEY_RETURN_Click()
            QtGui.QApplication.processEvents()

            def add_item(item):
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                self.ui.edtCodigo.setText(item)
                QtGui.QApplication.processEvents()
                return self.KEY_RETURN_Click()

            for corpo in corpos:
                if corpo.supdvo_vendedor_id:
                    vend_dav = SupervisorCadVendedor.query.filter(
                        SupervisorCadVendedor.id == corpo.supdvo_vendedor_id).all()
                    for v in vend_dav:
                        self.KEY_VENDEDOR_Click()
                        self.ui.edtCodigo.setText(str(v.supven_codigo))
                        self.KEY_RETURN_Click()

                iten = CaixaCadItens.get_by(supite_codigo_item=corpo.supdvo_item,
                                            supite_loja=self.loja.supcfc_loja)
                self.vlr_item_dav = corpo.supdvo_vr_unit
                QtGui.QApplication.processEvents()

                item_name = iten.name if config['descricao_resumida'] else iten.supite_descricao_completa
                # ao faturar o dav, o item fracionado estava perdendo as casas decimais ao ser iserido na tela
                if iten.supite_fracionado or corpo.supdvo_item.startswith('2'):
                    self.ui.spinQtda.setDecimals(3)

                if corpo.supdvo_desconto_item > 0:
                    self.KEY_DESCONTO_Click()
                    add_item(corpo.supdvo_item)
                    if corpo.supdvo_quantidade > 0:
                        self.ui.spinQtda.setValue(corpo.supdvo_quantidade)
                    self.KEY_RETURN_Click()
                    self.KEY_RETURN_Click()
                    self.ui.spinDecItemValor.setValue(corpo.supdvo_desconto_item)
                    self.KEY_RETURN_Click()
                elif corpo.supdvo_acrescimo_item > 0:
                    self.KEY_ACRESCIMO_Click()
                    add_item(corpo.supdvo_item)
                    if corpo.supdvo_quantidade > 0:
                        self.ui.spinQtda.setValue(corpo.supdvo_quantidade)
                    self.KEY_RETURN_Click()
                    self.KEY_RETURN_Click()
                    self.ui.spinDecItemValor.setValue(corpo.supdvo_acrescimo_item)
                    self.KEY_RETURN_Click()
                elif corpo.supdvo_quantidade > 0:
                    self.ui.spinQtda.setValue(corpo.supdvo_quantidade)
                    add_item(corpo.supdvo_item)
                    self.KEY_RETURN_Click()
                else:
                    add_item(corpo.supdvo_item)
                if corpo.supdvo_venda_cancelamento == 'C':
                    self.KEY_CANCEL_ITENS_Click()
                    self.KEY_RETURN_Click()

                cliente = self._search_cliente(capa.supdva_cliente, True)

                if iten and cliente:
                    trib = CaixaCadTributacao.get_by(id=iten.supite_tributacao_id)
                    try:
                        sess.begin()
                        corpo.supdvo_totalizador_parcial = CadTributacao().format_st(
                            trib.suptri_codigo_trib_ecf)
                        sess.commit()
                    except Exception as e:
                        Log.error("Erro ao atualizar 'supdvo_totalizador_parcial': %s" % str(e))

            QtGui.QApplication.processEvents()
            self.vlr_item_dav = False
            dif = self.mov_capa.supcap_total_cupom - capa.supdva_total_dav
            # se o valor do cupom for maior que o dav a diferenca vira desconto
            if dif > 0.001:
                self.KEY_DESCONTO_Click()
                self.KEY_RETURN_Click()
                self.KEY_RETURN_Click()
                self.ui.spinDecItemValor.setValue(dif)
                self.KEY_RETURN_Click()
                capa.supdva_descontos.quantize(TWOPLACES) + dif

            # valor do dav maior que valor do cupom, aplica a diferenca como acrescimo
            # este caso contempla frete, despesas acessorias, seguro, etc...
            elif dif < -0.001:
                dif = dif * -1
                self.KEY_ACRESCIMO_Click()
                self.KEY_RETURN_Click()
                self.KEY_RETURN_Click()
                self.ui.spinDecItemValor.setValue(dif)
                self.KEY_RETURN_Click()

            # ao faturar um pedido, se houver diferenca no total do pedido com o total do cupom, a diferenca entra
            # no total de uma das finalizadoras, primeiro dinheiro se houver, depois em qualquer uma das outras
            dict_finalizadoras= {}
            total_fechamento = 0
            dinheiro = False
            for fechamento in fechamentos:
                total_fechamento += fechamento.supdvf_valor
                dict_finalizadoras[fechamento.supdvf_finalizadora] = CaixaCadFinalizadora.get_by(
                                                                    supfin_codigo=fechamento.supdvf_finalizadora)
                if dict_finalizadoras[fechamento.supdvf_finalizadora].supfin_tipo_finalizadora == 'D':
                    dinheiro = True

            diferenca = self._coupon.supcap_total_cupom - total_fechamento
            if -0.001 < diferenca and diferenca < 0.001:
                diferenca = Decimal(0)
            DavFechamentosP = ()
            for fechamento in fechamentos:
                finalizadora = dict_finalizadoras.get(fechamento.supdvf_finalizadora)
                if not dinheiro: # quando tiver dinheiro, a diferenca entra na finalizadora dinheiro, do contrario entra em qualquer uma
                    total_finalizadora = fechamento.supdvf_valor + diferenca
                    diferenca = 0  # depois que aplica a dif. em uma finalizadora, nao aplica mais
                else:
                    total_finalizadora = fechamento.supdvf_valor

                if finalizadora.supfin_tipo_finalizadora == 'D':
                    total_finalizadora = fechamento.supdvf_valor + diferenca
                    diferenca = 0  # depois que aplica a dif. em uma finalizadora, nao aplica mais
                    self.KEY_DINHEIRO_Click()
                    self.ui.spinPgtoDinheiro.setValue(total_finalizadora)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinPgtoDinheiro)
                    self.KEY_RETURN_Click()
                elif finalizadora.supfin_tipo_finalizadora == 'O':
                    self.KEY_OUTROS_Click()
                    self.ui.cbOTipoPgto.setCurrentIndex(self.ui.cbOTipoPgto.findText(finalizadora.name))
                    self.KEY_RETURN_Click()
                    self.ui.spinOValor.clear()
                    self.ui.spinOValor.setValue(total_finalizadora)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinOValor)
                    self.KEY_RETURN_Click()
                elif finalizadora.supfin_tipo_finalizadora == 'T':
                    self.KEY_TICKET_Click()
                    self.ui.spinValorTicket.setValue(total_finalizadora)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinValorTicket)
                    self.KEY_RETURN_Click()
                elif finalizadora.supfin_tipo_finalizadora == 'C':
                    if not finalizadora.supfin_usa_tef:
                        self.ui.cbTipoTef.setCurrentIndex(1)
                    self.KEY_CARTAO_Click()
                    self.ui.cbChTipoCartao.setCurrentIndex(self.ui.cbChTipoCartao.findText(finalizadora.name))
                    self.KEY_RETURN_Click()
                    self.ui.spinCartaoValor.clear()
                    self.ui.spinCartaoValor.setValue(total_finalizadora)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinCartaoValor)
                    self.KEY_RETURN_Click()
                elif finalizadora.supfin_tipo_finalizadora == 'H':
                    vencimento = SupervisorDavVencimentos.query.filter(
                                        SupervisorDavVencimentos.supdvv_fechamento_id == fechamento.id).first()
                    data_vencimento = ''
                    if vencimento:
                        data_vencimento = vencimento.supdvv_vencimento.strftime('%d/%m/%Y') or ''
                    self.KEY_CHEQUE_Click()
                    DavFechamentosP += ({'Tipo': 'H',
                                         'Index': self.ui.cbChTipoCheque.findText(finalizadora.name),
                                         'Valor': total_finalizadora,
                                         'Vencimento': data_vencimento,
                                         'agencia': fechamento.supdvf_cheque_agencia or 0,
                                         'banco': fechamento.supdvf_cheque_banco  or 0,
                                         'conta': fechamento.supdvf_cheque_conta  or 0,
                                         'numero': int(fechamento.supdvf_cheque_numero or 0)},)

            self._DavFechamentosIter = iter(DavFechamentosP) if DavFechamentosP else False

        if DavFechamentosP:
            self.KEY_CHEQUE_Click()

        self.show_info_proccess(False)

    @Log
    def faturar_comanda(self, cod):
        try:
            if not cod:
                message_dlg(self, MSG_INFO, 'Informe o número da comanda!')
                return False
            capa = SupervisorCdaCapa.query.filter(
                SupervisorCdaCapa.supcda_codigo_comanda == cod).filter(
                SupervisorCdaCapa.supcda_restricao == 0).all()
            if not capa:
                message_dlg(self, MSG_INFO, 'Não existe itens para comanda %s.' % str(cod))
                self.ui.edtCodigo.clear()
                return False

            capa = capa[0]

            corpo = SupervisorCdaCorpo.query.filter(SupervisorCdaCorpo.supcdo_capa_id == capa.id).all()

            listItens = []
            item_vendedor_ids = []
            cod_item_externo = 0

            for itemComanda in corpo:
                item_cad = CaixaCadItens.query.filter(
                    CaixaCadItens.supite_codigo_item == itemComanda.supcdo_item).filter(
                    CaixaCadItens.supite_loja == config['loja']).all()

                # Contador para servir como id do item externo
                cod_item_externo += 1

                item_vendedor_ids.append((cod_item_externo, item_cad,
                                          itemComanda.supcdo_cod_vendedor,
                                          itemComanda.supcdo_nome_vendedor))

                if not item_cad:
                    message_dlg(self, MSG_INFO,
                                'Não posso faturar essa comanda. O item %s não está cadastrado no '
                                'banco de dados do caixa.' % itemComanda.supcdo_item)
                    return False

                dict = {}
                dict['Item'] = itemComanda.supcdo_item
                dict['Quantidade'] = float(itemComanda.supcdo_quantidade)
                dict['VendaCancelamento'] = itemComanda.supcdo_venda_cancelamento
                dict['Vendedor'] = itemComanda.supcdo_cod_vendedor
                dict['NomeVendedor'] = itemComanda.supcdo_nome_vendedor
                listItens.append(dict)

            self.vendedor_comanda_ids = item_vendedor_ids

            if not capa.supcda_restricao == '0':
                b_sess.begin()
                capa.supcda_restricao = 5  # Imprimindo comanda
                b_sess.commit()
        except Exception:
            self.tratar_except()
            return False

        if capa.supcda_cliente:
            self._search_cliente(capa.supcda_cliente, False)

        self.comanda_id = capa
        self._IdsComandas.append(self.comanda_id)

        if not capa.supcda_operador == self._operador_id and capa.supcda_operador:
            self._operador_id = capa.supcda_operador

        @Log
        def add_item(item):
            self._lock_keyboard(True)
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
            self.ui.edtCodigo.setText(item)
            self.KEY_RETURN_Click()
            self._lock_keyboard(False)
            return True

        for x in listItens:
            self.ui.spinQtda.setDecimals(3)
            if not self.comanda_id:
                break
            if x['Quantidade'] > 0:
                self.ui.spinQtda.setValue(x['Quantidade'])
                add_item(x['Item'])
            else:
                add_item(x['Item'])
            if x['VendaCancelamento'] == 'C':
                self.KEY_CANCEL_ITENS_Click(comanda=True)
                self.KEY_RETURN_Click()

        return True

    @Log
    def add_item_ficha(self, item, tipo_kit=False, qtd_item_no_kit=1, desconto=0, cod_ficha=False,
                       valor_ficha=0):
        '''
        Adicona itens na tabela de itens no lancamento de ficha
        :param item: 
        :return: 
        '''
        # Verifica se é kit
        cod_item = item.supite_codigo_item
        itens_kit, quantidades, vr_unitario_no_kit, descontos_kit_item = CadKits(self).\
                                                                             get_itens_kit(cod_item)

        if not tipo_kit: # para item que nao eh de kit
            quantity = int(self.ui.spinQtdItemFicha.value())
            dif = quantity - self.ui.spinQtdItemFicha.value()
            if dif <= -0.001 and 0.001 <= dif:
                return message_dlg(self, MSG_ERROR, 'Atenção!\nNão é permitido a venda de item'
                                                    ' com quantidade Fracionada na emissão de fichas!')
        else: # para item de kit
            quantity = qtd_item_no_kit
            dif = quantity - int(quantity)
            if dif <= -0.001 and 0.001 <= dif:
                return message_dlg(self, MSG_ERROR, 'Atenção!\nItem do kit possui quantidade Fracionada!\n'
                                                    '%s - QTD: %s' % (item.name, quantity))

        if itens_kit:
            Log.info('Adicionando os itens do Kit: %s' % cod_item)
            for i in range(0, len(itens_kit)):
                item_do_kit = itens_kit[i]
                desc_item = (descontos_kit_item[i] or Decimal(0)) * quantity
                self.add_item_ficha(item_do_kit, True, quantity * quantidades[i], desc_item)

            return 'Kit adicionado: %s' % cod_item

        # quando 'valor_ficha' tem valor, significa que esta trocando, e o valor deve ser o da ficha
        price = valor_ficha or item.supite_preco
        if self._state_screen == in_ficha:
            icon = self.icon_check
            subtotal = quantity * price - desconto
            checkTroca = QtCore.Qt.CheckState.Unchecked
        else: # troca de ficha
            desconto = Decimal(self.ui.spinDescontoFicha.value())
            if desconto >= price:
                msg = ['O valor do desconto deve ser menor que o valor total do item!']
                msg.append(('Total item: R$%.2f' % (price)).replace('.',','))
                msg.append(('Desconto: R$%.2f' % (desconto)).replace('.',','))
                return message_dlg(self, MSG_ERROR, '\n'.join(msg))

            icon = self.icon_minus
            subtotal = quantity * -price + desconto
            checkTroca = QtCore.Qt.CheckState.Checked

        item_name = remove_accents(item.name)
        tableItensFicha = self.ui.tableItensFicha
        rows_t = tableItensFicha.rowCount()
        sequence = rows_t
        tableItensFicha.insertRow(sequence)
        # Cod Item
        newItem = QtGui.QTableWidgetItem('%s' % cod_item)
        newItem.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newItem.setIcon(icon)
        newItem.setFont(self._font)

        # Descrição
        newDesc = QtGui.QTableWidgetItem(item_name)
        newDesc.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newDesc.setFont(self._font)

        # Quantidade
        newQtda = QtGui.QTableWidgetItem('%1.3f' % quantity)
        newQtda.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newQtda.setFont(self._font)

        # Valor Unitário
        newVUni = QtGui.QTableWidgetItem('%1.2f' % price)
        newVUni.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newVUni.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        newVUni.setFont(self._font)

        # Valor Desconto
        newVDesc = QtGui.QTableWidgetItem('%1.2f' % desconto) # setado apenas nos itens do kit
        newVDesc.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newVDesc.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        newVDesc.setFont(self._font)

        # Valor Total
        total = Decimal(Decimal(self.ui.lblTotalFichas.text().replace(',', '.')) +
                        subtotal).quantize(TWOPLACES)
        self.ui.lblTotalFichas.setText(('%s' % total).replace('.',','))
        newVTot = QtGui.QTableWidgetItem('%1.2f' % subtotal)
        newVTot.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newVTot.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        newVTot.setFont(self._font)

        # Ativo
        checkstate = QtCore.Qt.CheckState.Checked
        newAtiv = QtGui.QTableWidgetItem('V')
        newAtiv.setFlags(QtCore.Qt.ItemIsUserCheckable)
        newAtiv.setCheckState(checkstate)
        newAtiv.setFont(self._font)

        # Troca
        newTroc = QtGui.QTableWidgetItem('V')
        newTroc.setFlags(QtCore.Qt.ItemIsUserCheckable)
        newTroc.setCheckState(checkTroca)
        newTroc.setFont(self._font)

        # WIDGET OS NEW ITENS NA TABLE ITENS
        tableItensFicha.setItem(sequence, 0, newItem)
        tableItensFicha.setItem(sequence, 1, newDesc)
        tableItensFicha.setItem(sequence, 2, newQtda)
        tableItensFicha.setItem(sequence, 3, newVUni)
        tableItensFicha.setItem(sequence, 4, newVDesc)
        tableItensFicha.setItem(sequence, 5, newVTot)
        tableItensFicha.setItem(sequence, 6, newAtiv)
        tableItensFicha.setItem(sequence, 7, newTroc)
        if cod_ficha: # na troca, popula a coluna com o numero da ficha para marcar como trocada
            newFicha = QtGui.QTableWidgetItem('%s' % cod_ficha)
            newFicha.setFlags(QtCore.Qt.ItemIsUserCheckable)
            newFicha.setFont(self._font)
            newTroc.setCheckState(checkstate)
            tableItensFicha.setItem(sequence, 8, newFicha)

        # cliente / data / obs
        col = 8
        for i in [self.ui.edtClienteFicha, self.ui.edtDataEntregaFicha]:
            col += 1
            # colunas com nome do cliente e data de entrega
            text = len(i.text())>4 and i.text() or ''
            if i == self.ui.edtClienteFicha:
                text = re.sub('[a-z][A-Z][0-9]', '', text)

            newFichaInfo = QtGui.QTableWidgetItem('%s' % (text))
            newFichaInfo.setFlags(QtCore.Qt.ItemIsUserCheckable)
            newFichaInfo.setFont(self._font)
            tableItensFicha.setItem(sequence, col, newFichaInfo)

        # coluna com obs do produto de producao propria
        newFichaInfo = QtGui.QTableWidgetItem('%s' % remove_accents(self.obs_produto or ''))
        newFichaInfo.setFlags(QtCore.Qt.ItemIsUserCheckable)
        newFichaInfo.setFont(self._font)
        tableItensFicha.setItem(sequence, 11, newFichaInfo)

        tableItensFicha.scrollToBottom()
        tableItensFicha.resizeColumnsToContents()
        tableItensFicha.horizontalHeader().setStretchLastSection(True)
        self.ui.edtItemFicha.clear()
        self.ui.spinQtdItemFicha.setValue(1)
        self.ui.spinDescontoFicha.setValue(0)
        self._state_coupon = coupon_ficha
        self.check_button_status()
        return True

    @Log
    def _returnPressed_tableBuscaProduto(self, row):
        if self.ficha:
            if self.consulta_produto:
                self.consulta_produto = False
                cod_item = self.ui.tableBuscaProduto.item(row, 0).text()
                res_item = self._search_item(cod_item)
                self._state_screen = self._previous_state
                self.ui.stackedActions.setCurrentIndex(stk_page_ficha)
                if self._state_screen == in_ficha and res_item.supite_producao == 'P':
                    self.item_ficha = res_item
                    self.cod_ficha = False
                    self.ui.edtItemFicha.setText('%s' % cod_item)
                    BuscaObsProduto(self).exec_()
                    self.ui.lbl_data_entrega_ficha.setVisible(True)
                    self.ui.lbl_cliente_ficha.setVisible(True)
                    self.ui.edtDataEntregaFicha.setVisible(True)
                    self.ui.edtClienteFicha.setVisible(True)
                    self.ui.edtDataEntregaFicha.setText(u'%s' % datetime.now().strftime('%H:%M %d/%m/%Y '))
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtDataEntregaFicha)
                else:
                    self.add_item_ficha(res_item)
                    # self.ui.stackedActions.setCurrentIndex(stk_page_ficha)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtItemFicha)

        else:
            if self._state_coupon == coupon_opened:
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                self.ui.stackedActions.setCurrentIndex(stk_page_coupom)
                self._state_screen = in_coupon
            else:
                if self.user_logged:
                    self._set_screen_state(in_coupon)
                else:
                    return

            if self.consulta_produto:
                self.consulta_produto = False
                cod_item = self.ui.tableBuscaProduto.item(row, 0).text()
                res_item = self._search_item(cod_item)
                self.obj_cupom_fiscal.add_item(res_item,
                                           Decimal(str(self.ui.spinQtda.value())),
                                           vendedor_codigo=self.vendedor_codigo,
                                           nome_vendedor=self._nome_vendedor)

        self.check_button_status()

        return True

    @Log
    def _returnPressed_tableBuscaCliente(self, row):
        cnpj_cpf = self.ui.tableBuscaCliente.item(row, 0).text()
        nome = self.ui.tableBuscaCliente.item(row, 2).text()
        msg = 'Deseja Selecionar o Cliente:' \
              '<br><center><h1><font color="red">' + nome + '</font></h2>'
        if message_dlg(self, MSG_QUESTION, msg.encode('utf-8')) == YES_ROLE:
            if self.user_logged:
                self.ui.lblNomeCliente.setText(nome)
                self._search_cliente(cnpj_cpf, process_messages=True)
                if self._coupon_payment_method == pm_outros:
                    self.ui.edtOCodCliente.setText(cnpj_cpf)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinOValor)
                    self.ui.stackedActions.setCurrentIndex(stk_page_payments)
                    self._state_screen = in_add_payment
                elif self._coupon_payment_method == pm_cheque:
                    self.ui.edtChCliente.setText(cnpj_cpf)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbChProp)
                    self.ui.stackedActions.setCurrentIndex(stk_page_payments)
                    self._state_screen = in_add_payment
                else:
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                    self.ui.stackedActions.setCurrentIndex(stk_page_coupom)
                    self._state_screen = in_coupon
            else:
                self.infoText('Você precisa estar logado para selecionar um cliente')
        else:
            if self._previous_state == in_coupon:
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                self.ui.stackedActions.setCurrentIndex(stk_page_coupom)
                self._state_screen = in_coupon
            else:
                self._set_screen_state(self._previous_state)
        self.check_button_status()

    @Log
    def _returnPressed_tableItens(self, row):
        if not self._state_coupon == coupon_opened:
            return
        if self._CONTROL_FOCUS == self.ui.edtCodigo:
            return
        item = self.ui.tableItens.item(row, 0)
        desc = self.ui.tableItens.item(row, 1)
        itemAtivo = self.ui.tableItens.item(row, 7)
        itemSubtotal = self.ui.tableItens.item(row, 6)
        item_acres_desc = self.ui.tableItens.item(row, 5)

        if itemAtivo.checkState() == QtCore.Qt.CheckState.Checked:
            valor_acres_desc = Decimal(item_acres_desc.text())
            if valor_acres_desc:
                if self._indicador_desc_acre == item_add_surchage:
                    self.ui.lblInfoAcresTotalItens.setText(
                        u'%1.2f' % (Decimal(self.ui.lblInfoAcresTotalItens.text()) - valor_acres_desc))
                elif self._indicador_desc_acre == item_add_discount:
                    self.ui.lblInfoDescTotalItens.setText(
                        u'%1.2f' % (Decimal(self.ui.lblInfoDescTotalItens.text()) + valor_acres_desc))
                    self.desconto_total_itens += valor_acres_desc

            item.setIcon(self.icon_delete)
            itemAtivo.setCheckState(QtCore.Qt.CheckState.Unchecked)
            self.infoText('Cancelado: %s - %s' % (item.text(), desc.text()))
            self._coupon_amount -= Decimal(itemSubtotal.text())
            self._coupon_subtotal -= Decimal(itemSubtotal.text())
            self.ui.lblVlrTotal.setText(u'%1.2f' % self._coupon_amount)
            self.ui.lblVlrTotal_2.setText(u'%1.2f' % self._coupon_amount)
            qry_corpos = CaixaMovCorpos.get_by(
                supcor_capa_id=self._coupon.id,
                supcor_sequencia=row + 1)
            if qry_corpos:
                sess.begin()
                self._coupon.supcap_total_cupom = self._coupon_amount
                self._coupon.supcap_subtotal = self._coupon_subtotal
                qry_corpos.supcor_venda_cancelamento = 'C'
                qry_corpos.supcor_cancelado = True
                sess.commit()
            self._coupon.supcap_total_cupom = self._coupon_amount
            self.atualiza_valor_a_pagar()
            self.ui.lblVlrTotalR.setText(u'%.2f' % float(self.ui.lblVlrTotalAPagar.text()))
        else:
            self.infoText('O Item selecionado já foi cancelado!')
        self.ui.tableItens.clearSelection()
        self._tab_order = []
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
        self._state_item = item_none
        self.ui.tableItens.scrollToBottom()
        self.check_button_status()

    def _returnPressed_cbMFIntervalo(self):
        self.ui.edtMFIni.clear()
        self.ui.edtMFFim.clear()
        if self.ui.cbMFIntervalo.currentIndex() == 0:
            self.ui.edtMFIni.setMaxLength(10)
            self.ui.edtMFFim.setMaxLength(10)
            self.ui.edtMFIni.setInputMask('00/00/0000')
            self.ui.edtMFFim.setInputMask('00/00/0000')
            msg_ini, msg_fim = 'Data inicial', 'Data final'
        else:
            self.ui.edtMFIni.setValidator(QtGui.QIntValidator(self.ui.edtMFIni))
            self.ui.edtMFFim.setValidator(QtGui.QIntValidator(self.ui.edtMFFim))
            self.ui.edtMFIni.setInputMask('')
            self.ui.edtMFFim.setInputMask('')
            if self.ui.lblFiscalInfo.text() in ['Espelho MFD', 'Arquivo MFD']:
                l = 6
                s = 'COO'
            else:
                l = 4
                s = 'CRZ'
            self.ui.edtMFIni.setMaxLength(l)
            self.ui.edtMFFim.setMaxLength(l)
            msg_ini, msg_fim = '%s inicial' % s, '%s final' % s
        self.ui.edtMFIni.clear()
        self.ui.edtMFFim.clear()
        self.ui.label_edit_26.setText(msg_ini)
        self.ui.label_edit_27.setText(msg_fim)
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtMFIni)

    def _returnPressed_edtMFIni(self):
        str_ini = self.ui.edtMFIni.text()
        err = False
        if self.ui.cbMFIntervalo.currentIndex() == 0:  # Por Data
            try:
                d, m, y = map(int, str_ini.split('/'))
                start = datetime(y, m, d).date()
                if start > datetime.now().date():
                    err = 'Data inicial não pode ser maior que data atual'
            except:
                err = 'Data inválida'
        else:
            try:
                start = int(str_ini)
                if start <= 0:
                    err = 'Valor inicial não pode ser menor ou igual a zero'
            except:
                err = 'Valor inicial inválido'
        if err:
            message_dlg(self, MSG_INFO, err)
            self.ui.edtMFIni.clear()
            return
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtMFFim)

    @Log
    def _returnPressed_edtMFFim(self):
        str_ini = self.ui.edtMFIni.text()
        str_fim = self.ui.edtMFFim.text()
        err = False
        print '*' * 200
        print repr(self.ui.lblFiscalInfo.text())
        if self.ui.cbMFIntervalo.currentIndex() == 0:  # Por Data
            d, m, y = map(int, str_ini.split('/'))
            start = datetime(y, m, d).date()
            try:
                d, m, y = map(int, str_fim.split('/'))
                end = datetime(y, m, d).date()
                if end > datetime.now().date():
                    err = 'Data final não pode ser maior que data atual'
                elif start > end:
                    err = 'Data final não pode ser menor que Data inicial'
            except:
                err = 'Data inválida'
        else:
            start = int(str_ini)
            try:
                end = int(str_fim)
                if end <= 0:
                    err = 'Valor final não pode ser menor ou igual a zero'
                elif start > end:
                    err = 'Valor final não pode ser menor que Valor inicial'
            except:
                err = 'Valor inicial inválido'
        if err:
            message_dlg(self, MSG_INFO, err)
            self.ui.edtMFFim.clear()
            return
        else:
            if self.ui.lblFiscalInfo.text() == 'Meios de Pagamento':
                if not self.get_printer_status():
                    self._set_screen_state(in_menu_fiscal)
                    return

                if message_dlg(self, MSG_QUESTION, 'Deseja imprimir os Meios de Pagamento?') \
                        == YES_ROLE:
                    retval = relatorio_gerencial_meios_de_pgto(start, end)
                    if not retval:
                        message_dlg(self, MSG_INFO, 'Não há dados para Imprimir o Relatório')
                    else:
                        try:
                            # Todo: Imprimir e testar Meios de Pagamento
                            texto = self.obj_cupom_fiscal.get_texto_cupom(retval)
                            print '\n\nMeios de Pagamento\n\n', texto

                            self.obj_impressoras.supcci_imprimir(texto)
                            self.obj_cupom_fiscal.cria_capa_cupom_gerencial()
                            # GerencialReport(report_id=config['rg_meios_de_pgto'],
                            #                 text=retval,
                            #                 loja_id=self.loja.supcfc_loja,
                            #                 caixa_id=self.caixa_id,
                            #                 operador_id=self.user_logged.id,
                            #                 nro_cupom_vinculado=self._coupon_id,
                            #                 parent=self)
                        except CodigoInvalido as e:
                            message_dlg(self, MSG_ERROR, str(e),
                                        'O código(%d) do RG cadastrado no PDV-Config não existe '
                                        'na ECF.' % config['rg_meios_de_pgto'])
                self._set_screen_state(in_menu_fiscal)
                return
            elif self.ui.lblFiscalInfo.text() == 'Movimento por ECF':
                mov_ecf = MovimentoEcf(self).gera_movimento_ecf(start, end)
                self._set_screen_state(in_menu_fiscal)
                return mov_ecf
            elif self.ui.lblFiscalInfo.text() == u'Vendas do Período':
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbMFVendaPeriodo)

            else:
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbMFDestino)
        return str_fim

    @Log
    def _returnPressed_cbTipoTicket(self):
        if self.ui.cbTipoTicket.currentIndex() == 0:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbFinTicket)
        elif self.ui.cbTipoTicket.currentIndex() == 1:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinValorTicket)
        return self.ui.cbFinTicket.currentText()

    @Log
    def _returnPressed_cbFinTicket(self):
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinValorTicket)
        return self.ui.cbFinTicket.currentText()

    @Log
    def _returnPressed_cbTipoTef(self):
        if self.ui.cbTipoTef.currentIndex() == 0:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinCartaoValor)
        elif self.ui.cbTipoTef.currentIndex() == 1:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbChTipoCartao)
        return self.ui.cbTipoTef.currentText()

    @Log
    def _returnPressed_cbChTipoCheque(self):
        if not self.cliente.supcli_cnpj_cpf:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtChCliente)
        else:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbChProp)

        finalizadora = CaixaCadFinalizadora.get_by(name=self.ui.cbChTipoCheque.currentText())
        if finalizadora:
            prazo = finalizadora.supfin_outros_prazo_primeira or 0
            self.ui.dateChVcto.setText((datetime.now().date() + timedelta(prazo)
                                       ).strftime('%d/%m/%Y'))
        return self.ui.cbChTipoCheque.currentText()

    @Log
    def _returnPressed_cbChTipoCartao(self):
        if self.ui.cbChTipoCartao.currentIndex() == -1:
            return message_dlg(self, MSG_INFO, 'Escolha uma finalizadora!')

        self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinCartaoValor)
        return self.ui.cbChTipoCartao.currentText()

    @Log
    def _returnPressed_cbChProp(self):
        if self.ui.cbChProp.currentIndex() == 1:
            self.ui.edtChDocEmit.clear()
            self.cnpj_cpf_cheque = ''
        elif self.ui.cbChProp.currentIndex() == 0 and self.cliente.supcli_cnpj_cpf:
            self.ui.edtChDocEmit.setText(self.cliente.supcli_cnpj_cpf)
            self.cnpj_cpf_cheque = self.cliente.supcli_cnpj_cpf
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinChNroBanco)
        return self.ui.cbChProp.currentText()

    @Log
    def _returnPressed_spinChNroBanco(self):
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinChNroAgencia)
        return self.ui.spinChNroBanco.value()

    @Log
    def _returnPressed_spinChNroAgencia(self):
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinChNroCC)
        return self.ui.spinChNroAgencia.value()

    @Log
    def _returnPressed_spinChNroCC(self):
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinChNroCheque)
        return self.ui.spinChNroCC.value()

    @Log
    def _returnPressed_edtChCliente(self):
        text = self.ui.edtChCliente.text()

        if not text:
            return self.KEY_CLIENTE_Click()

        elif self.cliente.name and (str(text) not in [str(self.cliente.supcli_cnpj_cpf),
                                                            str(self.cliente.supcli_codigo_interno)]
                                          or self._state_screen == in_add_payment):

                if self._coupon_payment_method == pm_cheque:
                    self.ui.edtChCliente.setText(self.cliente.supcli_cnpj_cpf)

                return message_dlg(self, MSG_INFO,
                                   'Não foi possível selecionar o cliente %s \n\n\
                                   O Cliente %s já foi identificado no cupom fiscal.'
                                   % (str(text), str(self.cliente.name)))

        cod_cliente = self.ui.edtChCliente.text()
        cliente = False
        finalizadora = CaixaCadFinalizadora.get_by(name=self.ui.cbChTipoCheque.currentText())

        if cod_cliente:
            cliente = self._search_cliente(cod_cliente, True, False, False)
            if cliente:
                self.ui.edtChCliente.setText(cliente.name)
                self.ui.edtChDocEmit.setText(cliente.supcli_cnpj_cpf)
                self.cnpj_cpf_cheque = self.cliente.supcli_cnpj_cpf

        if not cliente:
            self.ui.edtChDocEmit.clear()
            self.cnpj_cpf_cheque = ''
            self.cliente = NotClient
            self.cliente_total_aberto = 0.0

        if finalizadora.supfin_avista_aprazo == 'P' and not cliente:
            self.ui.edtChCliente.clear()
            return self.infoText('É necessário informar um cliente cadastrado', display=True)

        self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbChProp)

        return self.ui.edtChCliente.text()

    @Log
    def _returnPressed_spinChNroCheque(self):
        if not self.ui.spinChNroCheque.value() or not self.ui.spinChNroCheque.text():
            return message_dlg(self, MSG_ERROR, 'É necessário informar o número do cheque!')

        if self.ui.cbChProp.currentIndex() == 0 and not self.cliente.name:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtChDocEmit)
        elif self.ui.cbChProp.currentIndex() == 1:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtChDocEmit)
        else:
            f = CaixaCadFinalizadora.get_by(name=self.ui.cbChTipoCheque.currentText())
            if f.supfin_avista_aprazo == 'V':
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinChValor)
            else:
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.dateChVcto)

        return self.ui.spinChNroCheque.value()

    @Log
    def _returnPressed_edtChDocEmit(self):
        cod_cliente = self.ui.edtChDocEmit.text()
        f = CaixaCadFinalizadora.get_by(name=self.ui.cbChTipoCheque.currentText())
        if not cod_cliente:
            return message_dlg(self, MSG_INFO, 'Informe o CPF/CNPJ do emitente do cheque')
        if not validar_cpf_cnpj(cod_cliente):
            self.ui.edtChDocEmit.clear()
            return message_dlg(self, MSG_INFO, 'CPF/CNPJ Inválido')
        if f.supfin_avista_aprazo == 'P':
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.dateChVcto)
        else:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinChValor)

        self.cnpj_cpf_cheque = self.ui.edtChDocEmit.text()

        return cod_cliente

    @Log
    def _returnPressed_dateChVcto(self):
        try:
            str_date = str(self.ui.dateChVcto.text())
            d, m, y = map(int, str_date.split('/'))
            data_vcto_ch = datetime(y, m, d).date()
        except ValueError:
            message_dlg(self, MSG_ERROR, 'Data inválida!')
            self.ui.dateChVcto.clear()
            return 'Data inválida!'

        if data_vcto_ch < datetime.today().date():
            message_dlg(self, MSG_INFO, 'Data de vencimento menor que data Atual! [%s]' % str_date)
            self.ui.dateChVcto.clear()
            return str_date
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinChValor)
        return self.ui.dateChVcto.text()

    @Log
    def _returnPressed_spinChValor(self):
        if not self.verifica_certificado():
            return True

        if self._keyboard_lock:
            return 'Teclado bloqueado'

        self._lock_keyboard(True)
        value = self.ui.spinPgtoDinheiro.valueFromText(self.ui.spinChValor.text())
        value = str(value)
        self.obj_cupom_fiscal.add_payment(Decimal(value))
        self._lock_keyboard(False)
        return value

    # FINALIZADORA OUTROS
    @Log
    def _returnPressed_edtOCodCliente(self):
        text = self.ui.edtOCodCliente.text()

        if not text:
            self.KEY_CLIENTE_Click()

        elif self.cliente.name and (str(text) not in [str(self.cliente.supcli_cnpj_cpf),
                                                            str(self.cliente.supcli_codigo_interno)]
                                          or self._state_screen == in_add_payment):

                if self._coupon_payment_method == pm_outros:
                    self.ui.edtOCodCliente.setText(self.cliente.supcli_cnpj_cpf)

                return message_dlg(self, MSG_INFO,
                                   'Não foi possível selecionar o cliente %s \n\n\
                                   O Cliente %s já foi identificado no cupom fiscal.' %
                                   (str(text), str(self.cliente.name)))

        cliente = self._search_cliente(self.ui.edtOCodCliente.text(), True)

        if cliente:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinOValor)
            self.ui.lblOCliente.setText("""<span style=" font-family:'Ubuntu'; font-size:13pt;
            font-weight:600; font-style:italic; color:#dcdcdc;">%s</span>""" % cliente.name)
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinOValor)
        else:
            self.ui.lblOCliente.setText(u"""<span style=" font-family:'Ubuntu'; font-size:13pt; f
            ont-weight:600; font-style:italic; color:#dcdcdc;">Código não Cadastrado!
            Tente Novamente.</span>""")
            self.ui.edtOCodCliente.clear()

        return self.ui.edtOCodCliente.text()

    @Log
    def _returnPressed_cbOTipoPgto(self):
        qry_finalizadora = CaixaCadFinalizadora.get_by(name=self.ui.cbOTipoPgto.itemText(
            self.ui.cbOTipoPgto.currentIndex())
        )
        if qry_finalizadora.supfin_avista_aprazo == 'P' and not self.ui.edtOCodCliente.text():
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtOCodCliente)
        else:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinOValor)
        return self.ui.cbOTipoPgto.currentText()

    @Log
    def _returnPressed_spinMFEcf(self):
        if not self.ui.spinMFEcf.text():
            message_dlg(self, MSG_INFO, 'Digite o Número da ECF')
            return
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtMFIni)
        return self.ui.spinMFEcf.text()

    @Log
    def _returnPressed_spinCartaoValor(self):
        if not self.verifica_certificado():
            return True

        if self._keyboard_lock:
            return 'Teclado bloqueado'

        self._lock_keyboard(True)
        value = self.ui.spinOValor.valueFromText(self.ui.spinCartaoValor.text())
        value = Decimal(str(value))
        valor_liquido = self._coupon_amount -self.desconto_promo - self.desconto_socio_torcedor - \
                        self.desconto_funcionario
        if self._payment_amount + value > valor_liquido:
            self.bloqueia_tecla = False
            self._lock_keyboard(False)
            Log.info('bloqueia_tecla: False')
            return message_dlg(self, MSG_INFO,
                               'O valor a ser pago no cartão não pode ser maior '
                               'que o valor a receber')

        if self.ui.cbTipoTef.currentIndex() == 1:  # pos
            self.obj_cupom_fiscal.add_payment(value, checa_tef=False)
            self._lock_keyboard(False)
        else:  # tef
            self.cartao_tef = True
            self._lock_keyboard(False)
            self.run_tef(value, self._coupon_id, tipo='C')
        return value

    @Log
    def _returnPressed_spinValorTicket(self):
        if not self.verifica_certificado():
            return True

        if self._keyboard_lock:
            return 'Teclado bloqueado'

        self._lock_keyboard(True)
        value = self.ui.spinValorTicket.valueFromText(self.ui.spinValorTicket.text())
        value = Decimal(str(value))
        if self.ui.cbTipoTicket.currentIndex() == 0:
            self.obj_cupom_fiscal.add_payment(value)
            self._lock_keyboard(False)
        else:
            self.ticket_tef = True
            self._lock_keyboard(False)
            self.run_tef(value, self._coupon_id, tipo='T')
        return value

    @Log
    def _returnPressed_spinOValor(self):
        if not self.verifica_certificado():
            return True

        value = self.ui.spinOValor.valueFromText(self.ui.spinOValor.text())
        value = Decimal(str(value))
        if self._keyboard_lock:
            return 'Teclado bloqueado'

        qry_finalizadora = CaixaCadFinalizadora.get_by(
                              name=self.ui.cbOTipoPgto.itemText(self.ui.cbOTipoPgto.currentIndex()))

        self._lock_keyboard(True)
        if not qry_finalizadora.supfin_usa_tef:
            self.obj_cupom_fiscal.add_payment(value)
            self._lock_keyboard(False)
        else:
            self._lock_keyboard(False)
            self.run_tef(value, self._coupon_id, tipo='T', modalidade=122)

        return value

    @Log
    def _returnPressed_edtCompensacao(self):
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtBanco)

    @Log
    def _returnPressed_edtBanco(self):
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtAgencia)

    @Log
    def _returnPressed_edtAgencia(self):
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtC1)

    @Log
    def _returnPressed_edtC1(self):
        compensacao = str(self.ui.edtCompensacao.text()).rjust(3, '0')
        banco = str(self.ui.edtBanco.text()).rjust(3, '0')
        agencia = str(self.ui.edtAgencia.text()).rjust(4, '0')
        codigo = '%s%s%s' % (compensacao, banco, agencia)
        c1 = self.ui.edtC1.text()
        digito_calc = int(baseModulo(codigo, '98765432'))
        if digito_calc > 9:
            digito_calc = 0
        if digito_calc != int(c1):
            return message_dlg(self, MSG_INFO, 'C1 Invalido')
        else:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtContaCorrente)

    @Log
    def _returnPressed_edtContaCorrente(self):
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtC2)

    @Log
    def _returnPressed_edtC2(self):
        conta = str(self.ui.edtContaCorrente.text()).rjust(10, '0')
        c2 = self.ui.edtC2.text()
        digito_calc = int(baseModulo(conta, '98765432'))
        if digito_calc > 9:
            digito_calc = 0
        if digito_calc != int(c2):
            return message_dlg(self, MSG_INFO, 'C2 Invalido')
        else:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtNroCheque)

    @Log
    def _returnPressed_edtNroCheque(self):
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtC3)

    @Log
    def _returnPressed_edtC3(self):
        nro_cheque = str(self.ui.edtNroCheque.text()).rjust(6, '0')
        c3 = self.ui.edtC3.text()
        digito_calc = int(baseModulo(nro_cheque, '98765432'))
        if digito_calc > 9:
            digito_calc = 0
        if digito_calc != int(c3):
            return message_dlg(self, MSG_INFO, 'C3 Invalido')
        else:
            compensacao = self.ui.edtCompensacao.text()
            banco = self.ui.edtBanco.text()
            agencia = self.ui.edtAgencia.text()
            c1 = self.ui.edtC1.text()
            conta = self.ui.edtContaCorrente.text()
            c2 = self.ui.edtC2.text()
            nro_cheque = self.ui.edtNroCheque.text()
            c3 = self.ui.edtC3.text()

            buffer = '0:%s%s%s%s%s%s%s%s' % (
                str(compensacao).rjust(3, '0'), str(banco).rjust(3, '0'),
                str(agencia).rjust(4, '0'),
                str(c1).rjust(1, '0'), str(conta).rjust(10, '0'), str(c2).rjust(1, '0'),
                str(nro_cheque).rjust(6, '0'), str(c3).rjust(1, '0'))
            self.ui.stackedActions.setCurrentIndex(stk_tef)
            self.sig_tef.continue_(buffer)

    @Log
    def _on_key_click(self, key):
        self._tempo_ocioso = 0
        text = chr(key) if key < 256 else ''
        text = text if text != '-' else ''
        ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, key, QtCore.Qt.KeyboardModifiers(), text=text)
        obj_name = self._CONTROL_FOCUS.objectName()

        if type(self._CONTROL_FOCUS) in [QtGui.QLineEdit, QtGui.QDoubleSpinBox, QtGui.QDateEdit,
                                         QtGui.QComboBox] and key == QtCore.Qt.Key_Return:
            return eval('self._returnPressed_%s()' % obj_name)

        # BACKSPACE
        if type(self._CONTROL_FOCUS) in [QtGui.QLineEdit, QtGui.QDoubleSpinBox,
                                         QtGui.QDateEdit] and key == \
                QtCore.Qt.Key_Backspace.__int__():
            self._CONTROL_FOCUS.selectAll()
            self._CONTROL_FOCUS.clear()
            if self.teclado and  type(self._CONTROL_FOCUS) ==  QtGui.QDoubleSpinBox:
                if self._CONTROL_FOCUS.objectName() in ['spinPgtoDinheiro', 'spinCartaoValor',
                                                        'spinValorTicket', 'spinOValor', 'spinChValor']:
                    if 'smak' in config['teclado_selecionado']:
                        self.teclado.limpar_linha(2) # limpa a linha
                    else:
                        self.teclado.limpar_linha(4) # limpa a linha

                elif self._CONTROL_FOCUS.objectName() in ['spinQtda','spinDecItemValor','spinChNroBanco',
                                                          'spinChNroAgencia', 'spinChNroCC', 'spinChNroCheque']:
                    self.teclado.limpar_linha(2) # limpa a linha


            self._CONTROL_FOCUS.selectAll()

        if type(self._CONTROL_FOCUS) in [QtGui.QLineEdit, QtGui.QDoubleSpinBox,
                                         QtGui.QDateEdit] and key == QtCore.Qt.Key_Down.__int__():
            return

        # Pressionou ENTER no widget tableMFBuscaProduto
        if self._CONTROL_FOCUS == self.ui.tableMFBuscaProduto and key in [QtCore.Qt.Key_Return,
                                                                          QtCore.Qt.Key_Left,
                                                                          QtCore.Qt.Key_Right]:
            return self._returnPressed_tableMFBuscaProduto(
                row=self.ui.tableMFBuscaProduto.currentRow(), key=key)

        # Pressionou ENTER no widget tableSigTefMenu
        if key == QtCore.Qt.Key_Return:
            if self._CONTROL_FOCUS == self.ui.tableSigTefMenu:
                return self._returnPressed_tableSigTefMenu(row=self.ui.tableSigTefMenu.currentRow())
            # Pressionou ENTER no widget tableBuscaPve
            elif self._CONTROL_FOCUS == self.ui.tableBuscaPve:
                return self._returnPressed_tableBuscaPve(row=self.ui.tableBuscaPve.currentRow())
            # Pressionou ENTER no widget tableBuscaDav
            elif self._CONTROL_FOCUS == self.ui.tableBuscaDav:
                return self._returnPressed_tableBuscaDav(self.ui.tableBuscaDav.currentRow())
            # Pressionou ENTER no widget tableItens
            elif self._CONTROL_FOCUS == self.ui.tableItens:
                return self._returnPressed_tableItens(self.ui.tableItens.currentRow())
            # Pressionou ENTER no widget tableItens
            elif self._CONTROL_FOCUS == self.ui.tableItensFicha:
                return self._returnPressed_tableItensFicha(self.ui.tableItensFicha.currentRow())
            # Pressionou ENTER no widget tableBuscaCliente
            elif self._CONTROL_FOCUS == self.ui.tableBuscaProduto:
                return self._returnPressed_tableBuscaProduto(self.ui.tableBuscaProduto.currentRow())
                # Pressionou ENTER no widget tableBuscaCliente
            elif self._CONTROL_FOCUS == self.ui.tableBuscaCliente:
                return self._returnPressed_tableBuscaCliente(self.ui.tableBuscaCliente.currentRow())
            # Pressionou ENTER no widget mainMenu
            elif self._CONTROL_FOCUS == self.ui.mainMenu:
                return self._returnPressed_mainmenu(self.ui.mainMenu.currentItem())
            # Pressionou ENTER no widget menuTef
            elif self._CONTROL_FOCUS == self.ui.menuTef:
                return self._returnPressed_menutef(self.ui.menuTef.currentItem())
            # Pressionou ENTER no widget tableMesa
            elif self._CONTROL_FOCUS == self.ui.tableMesa:
                return self._returnPressed_tableMesa(self.ui.tableMesa.currentItem())
            # Pressionou ENTER no widget tableMesaItens
            elif self._CONTROL_FOCUS == self.ui.tableItensMesa:
                return self._returnPressed_tableItensMesa(self.ui.tableItensMesa.currentRow())

        self._CONTROL_FOCUS.keyPressEvent(ev)

        if type(self._CONTROL_FOCUS) == QtGui.QListWidget:
            self._CONTROL_FOCUS.clearFocus()
            if self.teclado:
                item_selecionado = self._CONTROL_FOCUS.currentItem()
                try:
                    if self._CONTROL_FOCUS.objectName() == 'mainMenu':
                        texto_teclado = 'Menu:\n'
                    else:
                        texto_teclado = 'Menu Fiscal:\n'
                    texto_teclado += str(remove_accents(item_selecionado.text()))
                    self.teclado.envia_display(texto_teclado)

                except Exception as e:
                    Log.error(str(e))

        if self.teclado and self._CONTROL_FOCUS in  [self.ui.tableItens, self.ui.tableSigTefMenu]:
            row = self._CONTROL_FOCUS.currentRow()
            if self._CONTROL_FOCUS == self.ui.tableItens:
                texto = self._CONTROL_FOCUS.item(row, 1).text() + ' ' + \
                        self._CONTROL_FOCUS.item(row, 2).text() + ' R$' + \
                        self._CONTROL_FOCUS.item(row, 6).text()
            elif self._CONTROL_FOCUS == self.ui.tableSigTefMenu:
                texto = self._CONTROL_FOCUS.item(row, 1).text()

            if not 'smak' in config['teclado_selecionado']:
                self.teclado.limpar_linha(3) # limpa as linha 3 e 4 para nao ficar lixo no display
                self.teclado.limpar_linha(4) # de produtos com nome muito grande

            self.teclado.envia_linha_display(texto[:40], 2)

        return 'KEY=%s, TEXTO=%s, OBJ_NAME=%s' % (key, text, obj_name)

    @Log
    def KEY_DAV_Click(self):
        if self._executando_importacao:
            return False

        if not self.conectado:
            return message_dlg(self, MSG_INFO, 'Atenção!\n\nSem conexão com o Supervisor!')

        self.bloqueia_importacao()
        self.infoText("*** Acessando DAV's [{0}] *** Aguarde...".format(config['aplicativo_dav']),
                      False)
        self.ui.btnKEY_DAV.setEnabled(False)
        QtGui.QApplication.processEvents()
        screen_state = in_dav
        if config['aplicativo_dav'] == 'Local':
            pass
        elif config['aplicativo_dav'] == 'SiG-ERP':
            try:
                cancelar_pedidos()
                retval_dav = import_dav(self)
                msg_str = ''
                for id, msg in retval_dav.iteritems():
                    if msg:
                        msg_str += 'Pedido de Venda No. %010d possui inconsistencias:\n* ' % id
                        msg_str += '\n* '.join(map(lambda x: remove_accents(x), msg))
                        msg_str += '\n'
                if msg_str:
                    message_dlg(self, MSG_ERROR, 'Pedidos inconsistentes', str(msg_str))

            except ErrConnRPC as e:
                self.ui.btnKEY_DAV.setEnabled(True)
                message_dlg(self, MSG_ERROR, str(e))
                screen_state = in_coupon
            except:
                self.ui.btnKEY_DAV.setEnabled(True)
                self.tratar_except()
        elif config['aplicativo_dav'] == 'SiG-Gestao':
            try:
                importar_davs_gestao()
            except Exception as e:
                self.ui.btnKEY_DAV.setEnabled(True)
                Log.error(traceback.format_exc())
                message_dlg(self, MSG_ERROR, str(e))

        # caso a conexao caia com o frente ja aberto, para nao travar ao clicar em DAV, testar a conexao
        if not self.conectado:
            self._set_screen_state(screen_state)
            self.ui.btnKEY_DAV.setEnabled(True)
            return False

        self._set_screen_state(screen_state)
        self.check_button_status()
        self.show_info_proccess(False, '')
        self.ui.btnKEY_DAV.setEnabled(True)
        return True

    @Log
    def KEY_PRE_Click(self):
        ret = True
        try:
            self._pves_list = SupervisorPveCapa.query.filter(
                'suppva_restricao = 1 AND suppva_nro_cupom is Null').all()
            if self._pves_list:
                self._set_screen_state(in_pve)
                self.check_button_status()
            else:
                ret = False
                message_dlg(self, MSG_INFO, 'Não existem Pré-Vendas cadastradas.')
        except Exception:
            self.tratar_except()
        return ret

    @Log
    def KEY_FICHA(self):
        self.limpa_variaveis_abertura_cupom()
        self.ui.edtItemFicha.setMaxLength(14)
        self.ui.lbl_desconto_ficha.setVisible(False)
        self.ui.spinDescontoFicha.setVisible(False)
        self.ui.spinDescontoFicha.setEnabled(False)
        self.ui.lbl_ficha.setText(u'.: Emissão de fichas :.')
        if self._state_screen == in_troca_ficha:
            self._state_screen = in_ficha
            return self.check_button_status()

        self.ficha = True
        self.vr_ficha = Decimal(0)
        self._set_screen_state(in_ficha)
        self.check_button_status()
        return True

    @Log
    def KEY_TROCA_FICHA(self):
        self.ui.lbl_ficha.setText('.: Troca de Fichas :.')
        self.ui.edtItemFicha.setMaxLength(15)
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtItemFicha)
        self._state_screen = in_troca_ficha
        self.ui.lbl_desconto_ficha.setVisible(True)
        self.ui.spinDescontoFicha.setVisible(True)
        self.ui.spinDescontoFicha.setEnabled(True)
        self.ui.spinDescontoFicha.setValue(0)
        self.check_button_status()
        return True

    @Log
    def KEY_0_Click(self):
        return self._on_key_click(QtCore.Qt.Key_0)

    @Log
    def KEY_1_Click(self):
        return self._on_key_click(QtCore.Qt.Key_1)

    @Log
    def KEY_2_Click(self):
        return self._on_key_click(QtCore.Qt.Key_2)

    @Log
    def KEY_3_Click(self):
        return self._on_key_click(QtCore.Qt.Key_3)

    @Log
    def KEY_4_Click(self):
        return self._on_key_click(QtCore.Qt.Key_4)

    @Log
    def KEY_5_Click(self):
        return self._on_key_click(QtCore.Qt.Key_5)

    @Log
    def KEY_6_Click(self):
        return self._on_key_click(QtCore.Qt.Key_6)

    @Log
    def KEY_7_Click(self):
        return self._on_key_click(QtCore.Qt.Key_7)

    @Log
    def KEY_8_Click(self):
        return self._on_key_click(QtCore.Qt.Key_8)

    @Log
    def KEY_9_Click(self):
        return self._on_key_click(QtCore.Qt.Key_9)

    @Log
    def KEY_COMMA_Click(self):
        return self._on_key_click(QtCore.Qt.Key_Comma)

    @Log
    def KEY_BACKSPACE_Click(self):
        self.KTEXT = ''
        return self._on_key_click(QtCore.Qt.Key_Backspace)

    @Log
    def KEY_QTDA_Click(self):
        """ Tecla alterar quantidade do item """
        self.ui.spinQtda.setDecimals(3)
        if self._state_screen == in_coupon:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinQtda)
            self.infoText('Informe a quantidade desejada!')
            self._state_item = item_add_quantity
        elif self._state_screen == in_restaurante:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinQtdaRes)
        elif self._state_screen in [in_ficha, in_troca_ficha]:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinQtdItemFicha)
            self.infoText('Informe a quantidade desejada!')

        return self.check_button_status()

    @Log
    def KEY_SUPORTE_Click(self):
        if config['cmd_teamviewer']:
            ret_cmd = call('%s &' % config['cmd_teamviewer'], shell=True)
            if ret_cmd:
                message_dlg(self, MSG_INFO, 'Não Foi possível executar o team viewer')

        else:
            message_dlg(self, MSG_INFO, 'Comando para executar o Team Viewer não está '
                                        'configurado no pdvconfig')
        return True

    @Log
    def KEY_CANCEL_ITENS_Click(self, comanda=False):
        """ Tecla cancelar item na ecf """
        cancela = False
        if self._state_screen == in_coupon:
            row_count = self.ui.tableItens.rowCount()
            if row_count > 0:
                if self._IdDav or self._IdPve or self._IdMesa or comanda:
                    cancela = True
                elif self.user_logged.supope_operador_supervisor == 'O':
                    if self.user_logged.supope_cancela_item:
                        cancela = True
                    else:
                        dlg = password_dlg(self, msg_info='Cancelamento de Item')
                        if dlg.retval:
                            cancela = True
                        else:
                            return message_dlg(self, MSG_INFO,
                                               u'Operador: %s.\n\nNão tem permissão para executar '
                                               u'esta operação' % (dlg.user and dlg.user.nameor
                                                                   or self.user_logged.name))
                else:
                    cancela = True

                if cancela:
                    self._tab_order = [self.ui.tableItens]
                    self.ui.tableItens.selectRow(row_count - 1)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.tableItens)
                    self._state_item = item_cancel
            else:
                self.infoText('Não há itens para cancelar')
        elif self._state_screen == in_restaurante:
            row_count = self.ui.tableItensMesa.rowCount()
            if row_count > 0:
                if self._IdDav or self._IdPve or self._IdMesa:
                    cancela = True
                elif self.user_logged.supope_operador_supervisor == 'O':
                    if self.user_logged.supope_cancela_item:
                        cancela = True
                    else:
                        dlg = password_dlg(self, msg_info='Cancelamento de Item')
                        if dlg.retval:
                            cancela = True
                        else:
                            return message_dlg(self, MSG_INFO,
                                               'Operador: %s.\n\nNão tem permissão para executar '
                                               'esta operação' % dlg.user.name)
                else:
                    cancela = True

                if cancela:
                    self._tab_order = [self.ui.tableItensMesa]
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.tableItensMesa)
                    self.ui.tableItensMesa.selectRow(row_count - 1)
                    self._state_item = item_cancel
            else:
                self.infoText('Não há itens para cancelar')

        elif self._state_screen in [in_ficha, in_troca_ficha]:
            row_count = self.ui.tableItensFicha.rowCount()
            if row_count > 0:
                self._tab_order = [self.ui.tableItensFicha]
                self.ui.tableItensFicha.selectRow(row_count - 1)
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.tableItensFicha)
                self._state_item = item_cancel
            else:
                self.infoText('Não há itens para cancelar')

        self.check_button_status()
        return cancela

    @Log
    def KEY_ESCAPE_Click(self):
        '''
        -> STATUS CUPOM
        coupon_closed, coupon_opened, coupon_add_discount,
        coupon_add_surchage, coupon_add_payment, coupon_finish
        -> STATUS SCREEN
        in_login, in_menu, in_coupon, in_close_turn, in_ss,
        in_search_client, in_add_payment
        -> STATUS ITEM
        item_none, item_add_quantity, item_add_discount, item_cancel
        '''
        if self.bloqueia_retorno:
            return 'Sair Bloqueado, aguarde'

        self.bloqueia_retorno = True
        if self.ui.stackedActions.currentIndex() == stk_tef:
            ## MENU TEF ###
            # Cancela a transacao TEF no caso de retorno com 'ESC', No caso do Socio, nao cacela o
            # desconto se sair da tela de pagamento do TEF, cancela tambem quando volta do menu do
            # TEF
            if ((self.recarga_tef or self.socio_torcedor) and not self.controle_cancelamento_tef) \
                    or not self.mov_capa:
                if self._CONTROL_FOCUS.objectName() == self.ui.edtSigTefTexto.objectName():
                    self.ui.edtSigTefTexto.clear()
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                    self.ui.edtSigTefTexto.setVisible(False)
                    self.ui.lblSigTefInfo.setVisible(False)
                    self.sig_tef.finalizar_transacao(0, self.sig_tef.numero_cupom_fiscal.value,
                                                       self.sig_tef.data_fiscal.value,
                                                       self.sig_tef.horario.value)

                elif self._CONTROL_FOCUS.objectName() == self.ui.tableSigTefMenu.objectName():
                    cancela_menu_tef = False
                    if self.socio_torcedor:
                        cancela_menu_tef = True

                    self.sig_tef.finalizar_transacao(0, self.sig_tef.numero_cupom_fiscal.value,
                                                       self.sig_tef.data_fiscal.value,
                                                       self.sig_tef.horario.value,
                                                       cancela_menu_tef=cancela_menu_tef)

                if self.recarga_tef: # desistiu de usar o Recarga
                    self.recarga_tef = False

                if self.tela_socio_torcedor: # desistiu de usrar o Socio Torcedor
                    self.tela_socio_torcedor = False
                    self.socio_torcedor = False

            elif self.mov_capa:
                self.cancel_pinpad = True
                self.sig_tef.continue_(0)
                self.bloqueia_retorno = False
                return "Cancelou transacao"

            self._set_screen_state(self._state_screen)
            self._consulta_cheque = False
            #self.setControlFocus(self._CONTROL_FOCUS, self.ui.menuTef)
            self.check_button_status()
            self.bloqueia_retorno = False
            return 'Menu TEF'

        if self._CONTROL_FOCUS == self.ui.cbChTipoCartao:
            self.KEY_CARTAO_Click()
            self.bloqueia_retorno = False
            return

        if self._CONTROL_FOCUS == self.ui.cbFinTicket:
            self.KEY_TICKET_Click()
            self.bloqueia_retorno = False
            return

        if self._state_screen in [in_login, in_menu, in_menu_fiscal]:
            self.close()
        elif self._state_screen == in_item_menu_fiscal:
            if self.tdm:
                self._set_screen_state(in_menu)
                self.tdm = False
            else:
                self._set_screen_state(in_menu_fiscal)

        elif self._state_screen in [in_ficha, in_troca_ficha]: # Fichas
            if self._CONTROL_FOCUS == self.ui.spinQtdItemFicha or \
               self._CONTROL_FOCUS == self.ui.tableItensFicha or \
               self._CONTROL_FOCUS == self.ui.spinDescontoFicha:
                if self._CONTROL_FOCUS == self.ui.spinQtdItemFicha:
                    self.ui.spinQtdItemFicha.setValue(1)
                elif self._CONTROL_FOCUS == self.ui.spinDescontoFicha:
                    self.ui.spinDescontoFicha.setValue(0)

                self.ui.tableItensFicha.clearSelection()
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtItemFicha)
                self._state_item = item_none
            elif self._CONTROL_FOCUS == self.ui.edtDataEntregaFicha:
                self.ui.edtDataEntregaFicha.clear()
                self.obs_produto = ''
                self.ui.lbl_data_entrega_ficha.setVisible(False)
                self.ui.lbl_cliente_ficha.setVisible(False)
                self.ui.edtDataEntregaFicha.setVisible(False)
                self.ui.edtClienteFicha.setVisible(False)
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtItemFicha)
            elif self._CONTROL_FOCUS == self.ui.edtClienteFicha:
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtDataEntregaFicha)

            else:
                self.ui.lblTotalFichas.setText((u'%.2f' % 0))
                self._state_coupon = coupon_closed
                self.ui.edtClienteFicha.clear()
                if config['apenas_ficha']:
                    self._clear_globals(0)
                    self.KEY_FICHA()
                else:
                    self._set_screen_state(in_coupon)

        elif self._state_screen == in_coupon:
            if self.ui.lblCodigo.text() in [u'Código do Vendedor', u'Código do Cliente',
                                            u'Mensagem no Cupom Fiscal', u'Doação']:
                if self.ui.lblCodigo.text() == u'Doação':
                    self._clear_globals(0)
                    self.atualiza_valor_a_pagar()

                self.ui.frmInfoItens.setVisible(True)
                self.ui.lblCodigo.setText(u'Código do Item')
                self.ui.edtCodigo.clear()
                self.ui.edtCodigo.setMaxLength(14)
                if config['usar_ficha']:
                    self.ui.edtCodigo.setMaxLength(15)

            if self._state_coupon in [coupon_closed, coupon_finish]:
                if self.ui.stackedActions.currentIndex() == stk_page_payments:
                    self.ui.label_2.setText('''<span style=" font-family:'Ubuntu'; font-size:20pt;
                    font-weight:600; color:#00007f;">::: AGUARDE :::</span>''')
                    self._clear_globals(coupon_closed)
                    self._set_screen_state(in_coupon)

                if self.ui.lblCodigo.text() == u'Código do Cliente':
                    self.infoText('ESC = BUSCA CLIENTE')
                    self.ui.frmInfoItens.setVisible(True)
                    self.ui.lblCodigo.setText(u'Código do Item')
                    self.ui.edtCodigo.clear()
                elif self._state_item == item_balanca:
                    self._state_item = item_none
                    self.ui.edtCodigo.clear()
                    self.ui.spinQtda.setDecimals(0)
                    self.ui.spinQtda.setValue(1)
                    self._state_item = item_none
                    self.check_button_status()

            elif self._state_coupon == coupon_opened:
                if self._state_item == item_balanca:
                    self._state_item = item_none
                    self.ui.edtCodigo.clear()
                    self.ui.spinQtda.setDecimals(0)
                    self.ui.spinQtda.setValue(1)
                    self._state_item = item_none
                    self.check_button_status()
                    self.infoText('Cancelado leitura da balança', True)

            if self._state_coupon in [coupon_add_discount, coupon_add_surchage] or \
                            self._state_item in [item_add_discount, item_add_surchage]:
                self.infoText('Cancelou Desconto/Acréscimo')
                self.vlr_item_dav = False
                self._state_coupon = self._state_coupon_old
                self._clear_add_item_st()

            if self._state_item == item_add_quantity:
                self.infoText('Cancelou alterar Quantidade')
                self._clear_add_item_st()

        elif self._state_screen == in_add_payment:
            if self.vr_doacao:
                self._clear_globals(0)
                self.atualiza_valor_a_pagar()

            self.ui.edtOCodCliente.setText('')
            if self._coupon_totalize or (self.mov_capa and self.mov_capa.supcap_totalizado):
                self.infoText('ESC = CUPOM TOTALIZADO')
            elif self.ficha:
                self.ui.stackedActions.setCurrentIndex(stk_page_ficha)
                self._state_screen = in_ficha
                self.ui.spinQtdItemFicha.setValue(1)
                self.ui.edtItemFicha.clear()
                self._coupon_payment_method = None
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtItemFicha)
            else:
                self._coupon_payment_method = None
                self.ui.stackedActions.setCurrentIndex(stk_page_coupom)
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                self._tab_order = [self.ui.edtCodigo]
                self._state_screen = in_coupon
                self.check_button_status()
                self.desconto_socio_calculado = False

        elif self._state_screen in [in_dav, in_pve, in_ss, in_close_turn]:
            if self.user_logged:
                if self._CONTROL_FOCUS == self.ui.tableBuscaDav:
                    self.bloqueia_retorno = False
                    return self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtBuscaClienteDav)
                if self._state_screen in [in_close_turn, in_ss]:
                    self._set_screen_state(in_menu)
                else:
                    self.desbloqueia_importacao()
                    self._set_screen_state(in_coupon)
            else:
                self._set_screen_state(in_login)
        elif self._state_screen == in_restaurante:
            if self._state_coupon == itens_mesa_transf:
                self._tab_order = []
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.tableMesa)
            if self._state_item == item_cancel:
                self._tab_order = []
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.tableMesa)
                self._state_item = item_none

        elif self._state_screen == in_search_client:
            if self._state_coupon in (coupon_ficha, coupon_troca_ficha):
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtItemFicha)
                self.ui.stackedActions.setCurrentIndex(stk_page_ficha)
                self._state_screen = self._previous_state
                self._tab_order = [self.ui.edtItemFicha]
                self.check_button_status()

            elif self._state_coupon == coupon_closed or (self.mov_capa and self.mov_capa.supcap_totalizado):
                self._set_screen_state(self._previous_state)
            else:
                self._coupon_payment_method = None
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
                self.ui.stackedActions.setCurrentIndex(stk_page_coupom)
                self._state_screen = in_coupon
                self._tab_order = [self.ui.edtCodigo]
                self.check_button_status()
                self.desconto_socio_calculado = False

        elif self._state_screen == in_end_sale:
            self._clear_globals(coupon_closed)
            tbl_parametros = CaixaCfgParametros.get_by()
            if tbl_parametros:
                saldo_cx = self.obj_abertura_fechamento_turno.get_saldo_turno(self.mov_caixa,
                                                                         config['loja'],
                                                                         config['caixa'],
                                                                         (self.user_logged.id,
                                                                          self.user_logged.name))
                limite_sangria = tbl_parametros.supcfp_limite_sangria
                if not limite_sangria:
                    limite_sangria = Decimal('0')
                if not saldo_cx:
                    saldo_cx = Decimal('0')

                if saldo_cx > limite_sangria:
                    self.show_info_proccess(True, '%s' % config['frase_sangria'])
                    cont = 0
                    while cont < 15:
                        self.show_info_proccess(True, '%s' % config['frase_sangria'])
                        sleep(0.3)
                        cont += 1

                    self.show_info_proccess(False, '')

            if self._IdMesa:
                self._IdMesa = False
                self._set_screen_state(in_restaurante)
            elif self.ficha:
                self._set_screen_state(in_ficha)
            else:
                self._set_screen_state(in_coupon)
            self.infoText('::: CAIXA ABERTO :::')

        if self._CONTROL_FOCUS == self.ui.tableItens:
            self.ui.tableItens.clearSelection()
            self._tab_order = []
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
            self._state_item = item_none
            self.ui.tableItens.scrollToBottom()
        self.check_button_status()
        self.bloqueia_retorno = False
        return STATE_SCREEN_NAMES[self._state_screen]

    @Log
    def KEY_RETURN_Click(self):
        campos_bloqueados_tef = [self.ui.spinValorTicket, self.ui.spinCartaoValor, self.ui.tableSigTefMenu,
                                 self.ui.spinSigTefValor, self.ui.edtSigTefTexto, self.ui.menuTef,
                                 self.ui.spinSSDinheiro, self.ui.spinSSOutros]
        if self._CONTROL_FOCUS in campos_bloqueados_tef and not self.ficha:
            if self.bloqueia_tecla:
                return 'Teclado bloqueado'
            if self._CONTROL_FOCUS == self.ui.spinSSDinheiro and config['sangriamanual']:
                return self._on_key_click(QtCore.Qt.Key_Return)

            # bloqueia o duplo 'enter' do usuario quando o tef etiver executando suas mensagens
            self.bloqueia_tecla = True
            self.check_button_status()
            Log.info('bloqueia_tecla: True')

        return self._on_key_click(QtCore.Qt.Key_Return)

    @Log
    def KEY_LEFT_Click(self):
        return self._on_key_click(QtCore.Qt.Key_Left)

    @Log
    def KEY_UP_Click(self):
        if self._CONTROL_FOCUS in [self.ui.edtUser]:
            return

        if self._CONTROL_FOCUS in [self.ui.mainMenu, self.ui.menuTef]:
            return self._on_key_click(QtCore.Qt.Key_Up)
        else:
            tab_order = self._tab_order
            if len(tab_order) > 1:
                if type(self._CONTROL_FOCUS) in [QtGui.QComboBox, QtGui.QTableWidget]:
                    if type(self._CONTROL_FOCUS) == QtGui.QComboBox:
                        index = self._CONTROL_FOCUS.currentIndex()
                        if self._CONTROL_FOCUS == self.ui.cbChTipoCartao:
                            tablefin = self.ui.tableFinalizadora
                            tablefin.selectRow(index - 1)
                        elif self._CONTROL_FOCUS == self.ui.cbFinTicket:
                            tablefin = self.ui.tableFinalizadoraTicket
                            tablefin.selectRow(index - 1)
                        elif self._CONTROL_FOCUS == self.ui.cbTipoTef:
                            self.ui.cbChTipoCartao.setCurrentIndex(-1)
                        elif self._CONTROL_FOCUS == self.ui.cbTipoTicket:
                            self.ui.cbFinTicket.setCurrentIndex(-1)
                        else:
                            self.infoText(self._CONTROL_FOCUS.itemText(index - 1))
                    else:
                        index = self._CONTROL_FOCUS.currentRow()
                    if index > 0:
                        return self._on_key_click(QtCore.Qt.Key_Up)
                    elif index == 0 and type(self._CONTROL_FOCUS) == QtGui.QTableWidget:
                        tab_order.remove(self._CONTROL_FOCUS)
                        return self.setControlFocus(self._CONTROL_FOCUS, tab_order[-1], True)
                tab_order.remove(self._CONTROL_FOCUS)
                self.setControlFocus(self._CONTROL_FOCUS, tab_order[-1], True)
            else:
                if type(self._CONTROL_FOCUS) in [QtGui.QComboBox, QtGui.QTableWidget]:
                    if type(self._CONTROL_FOCUS) == QtGui.QComboBox:
                        index = self._CONTROL_FOCUS.currentIndex()
                        if self._CONTROL_FOCUS == self.ui.cbChTipoCartao:
                            tablefin = self.ui.tableFinalizadora
                            tablefin.selectRow(index - 1)
                        elif self._CONTROL_FOCUS == self.ui.cbFinTicket:
                            tablefin = self.ui.tableFinalizadoraTicket
                            tablefin.selectRow(index - 1)
                        elif self._CONTROL_FOCUS == self.ui.cbTipoTef:
                            if config['config_pinpad_active']: # muda o indice apenas se o Pinpad estiver habilitado
                                self.ui.cbChTipoCartao.setCurrentIndex(-1)
                            else:
                                return
                        elif self._CONTROL_FOCUS == self.ui.cbTipoTicket:
                            self.ui.cbFinTicket.setCurrentIndex(-1)
                        else:
                            self.infoText(self._CONTROL_FOCUS.itemText(index - 1))
                    else:
                        index = self._CONTROL_FOCUS.currentRow()
                    if index > 0:
                        return self._on_key_click(QtCore.Qt.Key_Up)
                    elif index == 0 and type(self._CONTROL_FOCUS) == QtGui.QTableWidget:
                        return self._on_key_click(QtCore.Qt.Key_Up)
                self.setControlFocus(self._CONTROL_FOCUS, tab_order[-1], True)
            return

    @Log
    def KEY_RIGHT_Click(self):
        return self._on_key_click(QtCore.Qt.Key_Right)

    @Log
    def KEY_DOWN_Click(self):
        if type(self._CONTROL_FOCUS) == QtGui.QComboBox:
            index = self._CONTROL_FOCUS.currentIndex()
            if self._CONTROL_FOCUS == self.ui.cbChTipoCartao:
                tablefin = self.ui.tableFinalizadora
                tablefin.selectRow(index + 1)
            elif self._CONTROL_FOCUS == self.ui.cbFinTicket:
                tablefin = self.ui.tableFinalizadoraTicket
                tablefin.selectRow(index + 1)
            else:
                self.infoText(self._CONTROL_FOCUS.itemText(index + 1))
        elif type(self._CONTROL_FOCUS) == QtGui.QLineEdit:
            tableBuscaProduto = self.ui.tableBuscaProduto
            if self._CONTROL_FOCUS == self.ui.edtBuscaCliente:
                self.setControlFocus(self._CONTROL_FOCUS, tableBuscaProduto)
        return self._on_key_click(QtCore.Qt.Key_Down)

    @Log
    def KEY_CLIENTE_Click(self):
        if self.ui.lblCodigo.text() == u'Doação':
            return message_dlg(self, MSG_INFO,
                               "Atenção!\n\nNão é permitida consulta de itens na Doação!")

        self._previous_state = self._state_screen
        if not self._state_screen == in_coupon and not self._state_screen == in_add_payment:
            self._set_screen_state(in_search_client)
        else:
            if self._state_coupon == coupon_opened and not self.cliente.name:
                self._set_screen_state(in_search_client)
            elif (self._state_screen == in_add_payment and self.cliente.name):
                if self._coupon_payment_method == pm_outros:
                    self.ui.edtOCodCliente.setText(self.cliente.supcli_cnpj_cpf)
                elif self._coupon_payment_method == pm_cheque:
                    self.ui.edtChCliente.setText(self.cliente.supcli_cnpj_cpf)
                return message_dlg(self, MSG_INFO, 'Não é possível selecionar um cliente.\n\n\
                O Cliente %s já foi identificado no cupom fiscal.' % str(self.cliente.name))
            else:
                if self.cliente.name:
                    if message_dlg(self, MSG_QUESTION, 'Deseja remover o Cliente atual e selecionar'
                                                       ' outro Cliente?') == YES_ROLE:
                        self.cliente = NotClient
                        self.cliente_total_aberto = 0.0
                        self.mov_capa.supcap_cliente = ''
                        self.desconto_funcionario = Decimal(0)
                        self.dict_itens_desconto_funcionario = {}
                        self.ui.lblNomeCliente.clear()
                        self.atualiza_valor_a_pagar()
                    else:
                        return
                self._set_screen_state(in_search_client)
        self.check_button_status()

    @Log
    def KEY_CONSULTA_Click(self):
        if self.ui.lblCodigo.text() == u'Doação':
            return message_dlg(self, MSG_INFO,
                               "Atenção!\n\nNão é permitida consulta de itens na Doação!")

        if not config['consultaprodutos']:
            return message_dlg(self, MSG_INFO, "Não é permitido a "
                                               "Consulta de Produtos")

        # nao permite consulta com o cupom finalizado(Ex: deu desconto na capa)
        if self._state_coupon == coupon_finish:
            self.ui.edtCodigo.setText('')
            self.ativa_beep()
            return message_dlg(self, MSG_INFO, 'O cupom já foi finalizado!' +
                               'Escolha uma forma de pagamento')

        self.consulta_produto = True

        self._previous_state = self._state_screen
        tableBuscaProduto = self.ui.tableBuscaProduto
        self._tab_order = [self.ui.edtBuscaCliente]

        # Definir Largura das Colunas
        tableBuscaProduto.setColumnWidth(0, 141)  # código
        tableBuscaProduto.setColumnWidth(1, 570)  # nome do cliente
        # Verifica se hà dados
        self.clear_widget_table(tableBuscaProduto)
        tableBuscaProduto.setColumnWidth(0, 120)
        tableBuscaProduto.setColumnWidth(1, 260)
        tableBuscaProduto.setColumnWidth(2, 80)
        tableBuscaProduto.setColumnWidth(3, 85)
        tableBuscaProduto.setColumnWidth(4, 30)
        tableBuscaProduto.setColumnWidth(5, 40)
        tableBuscaProduto.setColumnWidth(6, 90)
        self.infoText('Digite o Código/Nome do Produto')
        self.ui.lbl_consulta.setText('.: Consulta Produto :.')
        self._state_screen = in_search_client
        self.ui.stackedActions.setCurrentIndex(stk_search_client)
        self.ui.stackedBuscaCliPro.setCurrentIndex(1)
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtBuscaCliente)
        self.ui.edtBuscaCliente.clear()
        self.check_button_status()
        return True

    @Log
    def KEY_MENSAGEM_Click(self):
        if self._state_screen == in_coupon:
            self.ui.lblCodigo.setText(u'Mensagem no Cupom Fiscal')
            self.ui.edtCodigo.setMaxLength(47)
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
            self.ui.edtCodigo.clear()
            self.ui.edtCodigo.setText(self._message_in_coupon)
        self.check_button_status()
        return True

    @Log
    def KEY_IMPRIME_DAV(self):
        if not self.btn_imprime_dav_down:
            self.ui.btnKEY_ImprimeDav.setDown(True)
            self.btn_imprime_dav_down = True
        else:
            self.ui.btnKEY_ImprimeDav.setDown(False)
            self.btn_imprime_dav_down = False

        return True

    @Log
    def KEY_VENDEDOR_Click(self):
        if self._state_screen == in_coupon:
            self.ui.lblCodigo.setText(u'Código do Vendedor')
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
            self.ui.edtCodigo.clear()
        self.check_button_status()
        return True

    @Log
    def finalizadora_tipo_existe(self, tipo):
        fin = CaixaCadFinalizadora.query. \
            filter(CaixaCadFinalizadora.supfin_tipo_finalizadora == tipo).order_by('name')
        pos = fin.filter(CaixaCadFinalizadora.supfin_usa_tef == 'F')
        f_count = fin.count()
        if tipo == 'H' and f_count:
            self.ui.cbChTipoCheque.clear()
            for f in fin:
                self.ui.cbChTipoCheque.addItem(f.name)
            self.ui.cbChTipoCheque.setCurrentIndex(0)
        elif tipo == 'O' and f_count:
            self.ui.cbOTipoPgto.clear()
            for f in fin:
                self.ui.cbOTipoPgto.addItem(f.name)
            self.ui.cbOTipoPgto.setCurrentIndex(0)
        elif tipo == 'C' and f_count:
            self.ui.cbChTipoCartao.clear()
            rows = 0
            tablefin = self.ui.tableFinalizadora
            tablefin.setEnabled(False)
            self.clear_widget_table(tablefin)

            for p in pos:
                if p.supfin_codigo == 9999: # nao mostra a finalizadora padrao no POS
                    continue

                self.ui.cbChTipoCartao.addItem(p.name)
                tablefin.insertRow(rows)
                name_fin = QtGui.QTableWidgetItem(p.name)
                name_fin.setTextAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
                name_fin.setFont(self._font)
                tablefin.setItem(rows, 0, name_fin)
                rows += 1

            tamanho_table = rows * 44
            if tamanho_table >= 400:
                tamanho_table = 400

            tablefin.setMaximumSize(tablefin.maximumWidth(), tamanho_table)
            tablefin.setMinimumSize(tablefin.minimumWidth(), tamanho_table)
            tablefin.selectRow(0)
            tablefin.resizeColumnsToContents()
            tablefin.horizontalHeader().setStretchLastSection(True)

            self.ui.cbChTipoCartao.setCurrentIndex(-1)
        elif tipo == 'T' and f_count:
            self.ui.cbFinTicket.clear()

            rows = 0
            tablefin = self.ui.tableFinalizadoraTicket
            tablefin.setEnabled(False)
            self.clear_widget_table(tablefin)
            for p in pos:
                if p.supfin_codigo == 9999: # nao mostra a finalizadora padrao no POS
                    continue

                self.ui.cbFinTicket.addItem(p.name)
                tablefin.insertRow(rows)
                name_fin = QtGui.QTableWidgetItem(p.name)
                name_fin.setTextAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
                name_fin.setFont(self._font)
                tablefin.setItem(rows, 0, name_fin)
                rows += 1

            tamanho_table = rows * 58
            if tamanho_table >= 400:
                tamanho_table = 400

            tablefin.setMaximumSize(tablefin.maximumWidth(), tamanho_table)
            tablefin.setMinimumSize(tablefin.minimumWidth(), tamanho_table)
            tablefin.selectRow(0)
            tablefin.resizeColumnsToContents()
            tablefin.horizontalHeader().setStretchLastSection(True)
            self.ui.cbFinTicket.setCurrentIndex(-1)
        return f_count

    def calcula_descontos_promocoes(self, dinheiro=False):
        """
        Todos os desconto de promocoes devem ser chamados nesta funcao
        :return:
        """
        # nao calcula descontos de promocoes para Doacao e Ficha
        if self.vr_doacao:
            return 'Doacao'

        elif self.ficha:
            return 'Ficha'

        self.obj_cad_promo_capa.calcula_descontos_promo(dinheiro)
        if self.sig_tef.usando_sitef and (self.socio_torcedor or self._tef_nsu_socio_torcedor)\
                and not self.desconto_socio_calculado:
            self.obj_cupom_fiscal.get_desconto_socio_torcedor()

        if self._state_screen == in_add_payment:
            self.desconto_socio_calculado = True

        self.obj_cad_funcionarios.get_desconto_funcionario()

        self.atualiza_valor_a_pagar()
        self.atualiza_desconto()
        return True

    def limpa_variaveis_abertura_cupom(self):
        self._tempo_ocioso = 0
        self.mov_corpos = []
        self.desconto_promo = Decimal(0)
        self.desconto_total_itens = Decimal(0)
        self.desconto_socio_torcedor = Decimal(0).quantize(TWOPLACES) # zera o desconto para o novo cupom
        self.desconto_funcionario = Decimal(0)
        self._payment_receipt_g = []

    @Log
    def atualiza_valores_ficha(self):
        self.ui.lbl_ficha.setText(u'.: Emissão de fichas :.')
        self.ui.lbl_desconto_ficha.setVisible(False)
        self.ui.spinDescontoFicha.setVisible(False)
        self.ui.spinDescontoFicha.setEnabled(False)
        self.vr_ficha = Decimal(self.ui.lblTotalFichas.text().replace(',', '.')
                                 ).quantize(TWOPLACES)
        self._coupon_amount = self.vr_ficha
        self._coupon_subtotal = self._coupon_amount
        self.atualiza_valor_a_pagar()
        self.atualiza_desconto()
        return True

    def verifica_certificado(self):
        if config['apenas_ficha']:
            return True

        if not self.loja.supcfc_data_validade:
            message_dlg(self, MSG_INFO, 'Atenção!\n\nSem Certificado Digital cadastrado!')
            return False

        elif (self.loja.supcfc_data_validade - datetime.now()).days < 0:
            message_dlg(self, MSG_INFO, 'Atenção!\n\n Certificado Digital expirado!')
            return False

        return True

    def verica_valor_sem_cliente(self):
        # Nao faz o teste para venda da ficha, mas faz para faturar a ficha
        if self._state_coupon in (coupon_ficha, coupon_troca_ficha):
            return True

        # "Nao permite venda acima de R$ 3.000,00 sem cliente em Minas Gerais"
        if not self.mov_capa or self.mov_capa.supcap_total_cupom < 3000:
            return True

        if not self.cliente.supcli_cnpj_cpf:
            message_dlg(self, MSG_INFO, 'Atenção!\n\nInforme um cliente para este valor de venda!')
            return False

        msg = []
        if not self.cliente.supcli_endereco:
            msg.append('Logradouro')
        if not self.cliente.supcli_endereco_numero:
            msg.append('Número')
        if not self.cliente.supcli_cidade:
            msg.append('Cidade')
        if not self.cliente.supcli_cod_municipio or not int(self.cliente.supcli_cod_municipio):
            msg.append('Código IBGE da cidade')
        if not self.cliente.supcli_estado:
            msg.append('Estado')

        if msg:
            message_dlg(self, MSG_INFO, 'Atenção!\n\nCadastro de cliente incompleto, dados faltantes:\n%s' % ',\n'.join(msg))
            return False

        return True

    @Log
    def KEY_DINHEIRO_Click(self):
        if not self.verica_valor_sem_cliente():
            return True

        if self.bloqueia_tecla:
            return 'Tecla Bloqueada'

        if not self.finalizadora_tipo_existe('D'):
            return message_dlg(self, MSG_ERROR, "Não existem nehuma Finalizadora "
                                                "do tipo 'Dinheiro' Cadastrada no Banco de Dados.")

        if self._state_screen == in_end_sale or self._coupon_payment_method == pm_dinheiro:
            return True

        if self.ficha:
            self.atualiza_valores_ficha()
            # caso o valor seja negativo(volta dinheiro pro cliente) o 'pagamento'
            # eh apenas em outros
            if self.vr_ficha < 0:
                return self.KEY_OUTROS_Click()

        self.calcula_descontos_promocoes(True)
        self._coupon_payment_method = pm_dinheiro
        self._set_screen_state(in_add_payment)
        self.check_button_status()
        return True

    @Log
    def KEY_TICKET_Click(self):
        if not self.verica_valor_sem_cliente():
            return True

        if self.bloqueia_tecla:
            return 'Tecla Bloqueada'

        self.ui.cbFinTicket.setCurrentIndex(-1)
        if not self.finalizadora_tipo_existe('T'):
            return message_dlg(self, MSG_ERROR, "Não existem nehuma Finalizadora do tipo 'Ticket' "
                                                "Cadastrada no Banco de Dados.")

        if self._state_screen == in_end_sale or self._coupon_payment_method == pm_ticket:
            return True

        if self.ficha:
            self.atualiza_valores_ficha()
            # caso o valor seja negativo(volta dinheiro pro cliente) o 'pagamento'
            # eh apenas em outros
            if self.vr_ficha < 0:
                return self.KEY_OUTROS_Click()

        self.calcula_descontos_promocoes()
        self._coupon_payment_method = pm_ticket
        self._set_screen_state(in_add_payment)
        self.check_button_status()
        return True

    @Log
    def KEY_ESTACIONAMENTO_Click(self):
        self.estacionamento = True
        self.codigo_estacionamento = ''
        try:
            if message_dlg(self, MSG_NONE,
                               'Informe o código do cartão de estacionamento!') == YES_ROLE:

                self.estacionamento = False
        except Exception as e:
            Log.error(str(e))
            message_dlg(self, MSG_ERROR, str(e))

        self.estacionamento = False
        return True

    @Log
    def KEY_DOACAO(self):
        self.limpa_variaveis_abertura_cupom()
        if self._state_screen == in_coupon:
            self.vr_doacao = Decimal(0)
            self.ui.lblCodigo.setText(u'Doação')
            self.infoText(u'Doação')
            self.ui.edtCodigo.setMaxLength(47)
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
            self.ui.edtCodigo.clear()
            self.ui.edtCodigo.setText(self._message_in_coupon)
        self.check_button_status()
        return True

    @Log
    def KEY_OUTROS_Click(self):
        if not self.verica_valor_sem_cliente():
            return True

        if self.bloqueia_tecla:
            return 'Tecla Bloqueada'

        if not self.finalizadora_tipo_existe('O'):
            return message_dlg(self, MSG_ERROR, "Não existem nehuma Finalizadora do tipo 'Outros' "
                                                "Cadastrada no Banco de Dados.")
        if self._state_screen == in_end_sale or self._coupon_payment_method == pm_outros:
            return True

        if self.ficha:
            self.atualiza_valores_ficha()

        self.calcula_descontos_promocoes()
        self._coupon_payment_method = pm_outros
        self._set_screen_state(in_add_payment)
        self.check_button_status()
        return True

    @Log
    def KEY_CHEQUE_Click(self):
        if not self.verica_valor_sem_cliente():
            return True

        if self.bloqueia_tecla:
            return 'Tecla Bloqueada'

        if not self.finalizadora_tipo_existe('H'):
            return message_dlg(self, MSG_ERROR, "Não existem nehuma Finalizadora "
                                                "do tipo 'Cheque' Cadastrada no Banco de Dados.")
        if (self._state_screen == in_end_sale or self._coupon_payment_method == pm_cheque) and \
            not self._IdDav: # no dav não deve fazer este teste
            return True

        if self.ficha:
            self.atualiza_valores_ficha()
            # caso o valor seja negativo(volta dinheiro pro cliente) o 'pagamento'
            # eh apenas em outros
            if self.vr_ficha < 0:
                return self.KEY_OUTROS_Click()

        self.calcula_descontos_promocoes()
        self._coupon_payment_method = pm_cheque
        self._set_screen_state(in_add_payment)
        self.check_button_status()
        return True

    @Log
    def KEY_CARTAO_Click(self):
        if not self.verica_valor_sem_cliente():
            return True

        self.ui.cbChTipoCartao.setCurrentIndex(-1)
        if not self.finalizadora_tipo_existe('C'):
            return message_dlg(self, MSG_ERROR, "Não existem nehuma Finalizadora "
                                                "do tipo 'Cartão' Cadastrada no Banco de Dados.")

        if self._state_screen == in_end_sale or self._coupon_payment_method == pm_cartao:
            return True

        if self.ficha:
            self.atualiza_valores_ficha()
            # caso o valor seja negativo(volta dinheiro pro cliente) o 'pagamento'
            # eh apenas em outros
            if self.vr_ficha < 0:
                return self.KEY_OUTROS_Click()

        self.calcula_descontos_promocoes()
        self._coupon_payment_method = pm_cartao
        self._set_screen_state(in_add_payment)
        self.check_button_status()
        return True

    @Log
    def KEY_DESCONTO_Click(self):
        if self._state_screen == in_troca_ficha:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinDescontoFicha)
            return True

        if self._indicador_desc_acre == item_add_surchage and not self.emitindo_dav:
            return message_dlg(self, MSG_INFO, 'Já foi efetuado acréscimo nesse cupom !')

        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
        self._CONTROL_FOCUS.clear()
        self.infoText('Digite o Código do item para adicionar o Desconto!')
        self._state_item = item_add_discount
        self._state_coupon_old = self._state_coupon
        self.check_button_status()

        return True

    @Log
    def KEY_ACRESCIMO_Click(self):
        if self._indicador_desc_acre == item_add_discount and not self.emitindo_dav:
            return message_dlg(self, MSG_INFO, 'Já foi efetuado desconto nesse cupom !')

        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
        self._CONTROL_FOCUS.clear()
        self.infoText('Digite o Código do item para adicionar o Acréscimo!')
        self._state_item = item_add_surchage
        self._state_coupon_old = self._state_coupon
        self.check_button_status()
        return True

    @Log
    def KEY_COMANDA_Click(self):
        if not self.conectado:
            message_dlg(self, MSG_INFO, 'Atenção!\n\nSem conexão com o Supervisor!')
            return False

        self._comanda_ativa = True if not self._comanda_ativa else False

        if self._comanda_ativa:
            self.infoText("Digite código da comanda.")
            self.ui.btnKEY_COMANDA.setStyleSheet(
                "border-image: url(:/buttons/images/buttons/blue_pressed.png);")
        else:
            self.ui.btnKEY_COMANDA.setStyleSheet(
                "/buttons/images/buttons/disabled.png;")

        self.check_button_status()
        return self._comanda_ativa

    @Log
    def bloqueia_botoes(self):
        '''
        Bloqueia os botoes a baixo
        :return:
        '''
        self.ui.btnKEY_QTDA.setEnabled(False)
        self.ui.btnKEY_BALANCA.setEnabled(False)
        self.ui.btnKEY_CONSULTA.setEnabled(False)
        self.ui.btnKEY_VENDEDOR.setEnabled(False)
        self.ui.btnKEY_CLIENTE.setEnabled(False)
        self.ui.btnKEY_ACRESCIMO.setEnabled(False)
        self.ui.btnKEY_DESCONTO.setEnabled(False)
        self.ui.btnKEY_RECARGA_CEL.setEnabled(False)
        self.ui.btnKEY_SOCIO_TORCEDOR.setEnabled(False)
        self.ui.btnKEY_MENU.setEnabled(False)
        self.ui.btnKEY_CONSULTA_CH.setEnabled(False)
        self.ui.btnKEY_MENSAGEM.setEnabled(False)
        self.ui.btnKEY_DAV.setEnabled(False)
        return True

    @Log
    def KEY_RECARGA_CEL_Click(self):
        self.limpa_variaveis_abertura_cupom()
        self.recarga_tef = True
        self.bloqueia_botoes()
        self.run_tef(0, 0, tipo='D', modalidade=300)
        if self._CONTROL_FOCUS.objectName() != self.ui.tableSigTefMenu.objectName() and \
                self._state_screen != in_coupon:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.tableSigTefMenu)

        else: # nao conectou no tef
            self.recarga_tef = False
            self.sig_tef.limpa_variaveis()

        return True

    @Log
    def KEY_SOCIO_TORCEDOR_Click(self):
        self.bloqueia_botoes()
        self.socio_torcedor = True
        self.tela_socio_torcedor = True
        self.socio_tef = True # habilita o campos para cpf
        self.run_tef(0, 0, tipo='C', modalidade=61)
        if self._CONTROL_FOCUS.objectName() != self.ui.edtSigTefTexto.objectName() and \
                                                                    self._state_screen != in_coupon:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtSigTefTexto)

        # nao conectou no tef
        elif self._CONTROL_FOCUS.objectName() != self.ui.edtSigTefTexto.objectName():
            self.socio_torcedor = False
            self.tela_socio_torcedor = False
            self.socio_tef = False
            self.sig_tef.limpa_variaveis()


        return True

    @Log
    def KEY_CONSUTLA_CH_Click(self):
        """
        Função a qual identifica que a ação a ser tomada junto ao tef eh para efetuar uma consulta
        cheque.
        Sendo assim eh manipulado os componetes para iniciar o consulta tef.
        """
        self.ui.spinSigTefValor.setVisible(True)
        self.ui.edtSigTefTexto.setVisible(False)
        self.ui.tableSigTefMenu.setVisible(False)
        self.ui.lblSigTefInfo.setText('Informe valor do cheque.')
        self.ui.lblSigTefInfo.setVisible(True)
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinSigTefValor)

        # Limpando campos antes de chamar a tela.
        self.ui.edtAgencia.clear()
        self.ui.edtBanco.clear()
        self.ui.edtNroCheque.clear()
        self.ui.edtCompensacao.clear()
        self.ui.edtContaCorrente.clear()
        self.ui.edtC1.clear()
        self.ui.edtC2.clear()
        self.ui.edtC3.clear()

        self.ui.stackedActions.setCurrentIndex(stk_tef)
        self._consulta_cheque = True

        return self._consulta_cheque

    @Log
    def _returnPressed_spinSigTefValor(self):
        value = '%.2f' % self.ui.spinSigTefValor.valueFromText(self.ui.spinSigTefValor.text())
        buffer = value.replace('.', ',')
        self.ui.spinSigTefValor.setValue(0.00)
        self.ui.spinSigTefValor.setVisible(False)
        if self._consulta_cheque:
            if Decimal(value) <= 0:
                self.ui.spinSigTefValor.setVisible(True)
                return message_dlg(self, MSG_INFO, 'Valor do cheque deve ser maior do que zero.')
            self.ui.lblSigTefInfo.setVisible(False)
            now = datetime.now()
            data_fiscal = now.strftime('%Y%m%d')
            horario = now.strftime('%H%M%S')
            self._state_screen = in_coupon
            self.sig_tef.usando_sitef = True
            self.sig_tef.tipo_finalizadora = None
            self.sig_tef.inicia_sitef(Decimal(value), 0, data_fiscal, horario,
                                      self.user_logged.name)
        else:
            self.sig_tef.continue_(buffer)

    @Log
    def _returnPressed_edtSigTefTexto(self):
        buffer = self.ui.edtSigTefTexto.text()
        self.ui.edtSigTefTexto.clear()
        self.ui.edtSigTefTexto.setVisible(False)
        self.ui.lblSigTefInfo.setVisible(False)

        Log.info(str(self.sig_tef.tipo_campo.value))
        if self.sig_tef.tipo_campo.value == 505 and buffer.isdigit():
            self.sig_tef.mod_parcelas = int(buffer)

        self.sig_tef.continue_(buffer)
        # se for sorcio o cpf, seleciona o cliente correspondente ao cpf se o mesmo for cliente
        if self._CONTROL_FOCUS.objectName() == 'edtCodigo':
            self._search_cliente(buffer)

        return buffer

    @Log
    def _returnPressed_tableSigTefMenu(self, row):
        buffer = self.ui.tableSigTefMenu.item(row, 0).text()
        self.ui.tableSigTefMenu.setVisible(False)
        self.ui.lblSigTefInfo.setVisible(False)
        self.sig_tef.continue_(buffer)
        return buffer

    @Log
    def _returnPressed_cbMFVendaPeriodo(self):
        idx = self.ui.cbMFVendaPeriodo.currentIndex()
        if idx == 0:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbMFRegistro60I)
            self.ui.cbMFRegistro60I.setEnabled(True)
            self.ui.cbMFRegistro60R.setEnabled(True)
        else:
            if message_dlg(self, MSG_QUESTION,
                           'Deseja gerar o arquivo: %s'
                                   % str(self.ui.cbMFVendaPeriodo.currentText())) == YES_ROLE:
                self.show_info_proccess(True, 'Gerando Arquivo')
                QtGui.QApplication.processEvents()
                d, m, y = map(int, self.ui.edtMFIni.text().split('/'))
                start = datetime(y, m, d).date()
                d, m, y = map(int, self.ui.edtMFFim.text().split('/'))
                end = datetime(y, m, d).date()
                try:
                    items = ("Perfil A", "Perfil B")

                    item, ok = QtGui.QInputDialog.getItem(None, "QInputDialog.getItem()",
                                                          "Selecione o Perfil:", items, 0, False)
                    perfil = 'B'
                    if item == "Perfil A":
                        perfil = 'A'

                    reg = gerar_cotep_09_08(self, self.loja, self.caixa_id, start, end, perfil)
                    paf_ecf = get_info_pafecf('sw.ini')
                    dt = datetime.now()
                    filename = '%s%s.txt' % (paf_ecf.laudo, dt.strftime('%d%m%Y%H%M%S'))
                    filename = os.path.join(ROOT_PATH, filename)
                    f = open(filename, 'w')
                    f.write(reg)
                    f.close()
                    assinatura_ead(filename)
                    self.show_info_proccess(False, '')
                    message_dlg(self, MSG_INFO, 'Arquivo gerado:', file=filename)
                except Exception as e:
                    self.tratar_except()
            self.show_info_proccess(False, '')
            self._set_screen_state(in_menu_fiscal)
        return self.ui.cbMFVendaPeriodo.currentText()

    @Log
    def _returnPressed_edtBuscaClienteDav(self):
        return self._set_screen_state(in_dav)

    @Log
    def _returnPressed_cbMFRegistro60I(self):
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbMFRegistro60R)
        return self.ui.cbMFRegistro60I.currentText()

    @Log
    def _returnPressed_cbMFRegistro60R(self):
        if message_dlg(self, MSG_QUESTION, 'Deseja gerar o arquivo: %s'
                % self.ui.cbMFVendaPeriodo.currentText().encode('utf-8')) == YES_ROLE:
            self.show_info_proccess(True, 'Gerando Arquivo')
            QtGui.QApplication.processEvents()
            d, m, y = map(int, self.ui.edtMFIni.text().split('/'))
            start = datetime(y, m, d).date()
            d, m, y = map(int, self.ui.edtMFFim.text().split('/'))
            end = datetime(y, m, d).date()
            try:
                reg = gerar_ac_09_08(self.loja.supcfc_loja, self.caixa_id, start, end,
                                     self.ui.cbMFRegistro60I.currentIndex(),
                                     self.ui.cbMFRegistro60R.currentIndex())
                paf_ecf = get_info_pafecf('sw.ini')
                dt = datetime.now()
                filename = '%s%s.txt' % (paf_ecf.laudo, dt.strftime('%d%m%Y%H%M%S'))
                filename = os.path.join(ROOT_PATH, filename)
                f = open(filename, 'w')
                f.write(reg)
                f.close()
                assinatura_ead(filename)
                self.show_info_proccess(False, '')
                message_dlg(self, MSG_INFO, 'Arquivo gerado:', file=filename)

            except Exception as e:
                self.tratar_except()
        self.show_info_proccess(False, '')
        self._set_screen_state(in_menu_fiscal)
        return self.ui.cbMFRegistro60R.currentText()

    @Log
    def _returnPressed_cbMFSubcategoria(self):
        idx = self.ui.cbMFSubcategoria.currentIndex()
        if idx == 0:
            try:
                if message_dlg(self, MSG_QUESTION,
                               'Deseja gerar o arquivo Arquivo Eletrônico do Estoque') == YES_ROLE:
                    self.show_info_proccess(True, 'Gerando arquivo.')
                    QtGui.QApplication.processEvents()
                    loja = get_loja()
                    p1 = gerar_registro_ident_estab('E1')
                    p2 = gerar_registro_relacao_merc(loja.supcfc_cnpj, 'E2', loja.supcfc_loja)
                    p9 = gerar_registro_totalizacao(loja.supcfc_cnpj,
                                                    loja.supcfc_inscricao_estadual,
                                                    p2[1], 0, 'E9')
                    filename = os.path.join(ROOT_PATH, 'EstoqueTotal.txt')
                    f = open(filename, 'w')
                    f.write(p1 + p2[0] + p9)
                    f.close()
                    assinatura_ead(filename)
                    self.show_info_proccess(False, '')
                    message_dlg(self, MSG_INFO, 'Arquivo gerado:', file=filename)
            except Exception:
                self.tratar_except()
            self.show_info_proccess(False, '')
            self._set_screen_state(in_menu_fiscal)
        else:
            self.ui.frameMFSubcategoria.setVisible(False)
            self.ui.frameMFMovimento.setVisible(True)
            self.clear_widget_table(self.ui.tableMFBuscaProduto)
            self.clear_widget_table(self.ui.tableMFProdutos)
            self._tab_order = [self.ui.edtMFBuscaProduto]
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtMFBuscaProduto)
        return self.ui.cbMFSubcategoria.currentText()

    @Log
    def KEY_MENU_Click(self):
        if self._state_coupon not in (coupon_closed, mesa_closed, mesa_opened):
            message_dlg(self, MSG_INFO, 'Finalize o cupom para poder acessar o menu.')
            return 'Cupom aberto'
        else:
            self.cliente = NotClient
            self.cliente_total_aberto = 0.0
            self._message_in_coupon = None
            self.vendedor_codigo = 0
            self._nome_vendedor = ''
            self.ui.lblNomeCliente.clear()
            self._set_screen_state(in_menu)
        self.check_button_status()
        return True

    @Log
    def KEY_CANCELAR_NOTA(self):
        """
        Cancela a nota que esta sendo feita e ainda nao foi enviada ao SEFAZ
        :return: 
        """
        self.cancelar_nota_fiscal()
        return  True

    @Log
    def cancelar_nota_fiscal(self, automatico=False):
        '''Cancela nota fical quando ela não foi mandada para a SEFAZ'''

        if not automatico:
            liberar = True
            if self.user_logged.supope_operador_supervisor == 'O':
                dlg = password_dlg(self, msg_info='')
                if not dlg.retval:
                    liberar = False
            if self.user_logged and liberar:
                if self.mov_caixa:
                    if message_dlg(self, MSG_QUESTION,
                                   'Deseja Cancelar a Venda?') == NO_ROLE:
                        return False
                else:
                    message_dlg(self, MSG_INFO, 'E Necessário Abrir um Turno')
                    return False
            else:
                message_dlg(self, MSG_INFO, 'Efetue Login')
                return False

            if message_dlg(self, MSG_QUESTION, 'Atenção! \n\nDeseja cancelar esta nota? '
                                               'A operação é irreversível!', focus=1) == NO_ROLE:
                return False

        if self.mov_capa.supcap_state and self.mov_capa.supcap_chave_acesso:
            if message_dlg(self, MSG_QUESTION, 'Atenção! \n\n'
                                               'Nota com a chave de acesso "%s" já está emitida na SEFAZ'
                                               'Deseja cancelar na SEFAZ? A operação é irreversível!' %
                                               str(self.mov_capa.supcap_chave_acesso), focus=1) == NO_ROLE:
                self.obj_cupom_fiscal.imprimir_nota(self.mov_capa.supcap_nro_cupom)
                return False
            else:
                self.obj_cupom_fiscal.cancelar_nota_fiscal_sefaz(self.mov_capa)

            return True

        nfce = self.obj_cupom_fiscal.gerar_nfce()
        nroCupom = self.mov_capa.supcap_nro_cupom
        self.obj_mov_capa.write(self.mov_capa, {'supcap_state': '5',
                                                'supcap_data_fim': datetime.now(),
                                                'supcap_chave_acesso': nfce.chave,
                                                'supcap_ambiente': config['nfce_ambiente'],})
        MovCupomServidor().create({'supcse_capa_id': self.mov_capa.id})
        self._clear_globals(coupon_closed)
        self._set_screen_state(in_coupon)
        self._set_controls_default_values()
        self.check_button_status()
        if self.sig_tef.usando_sitef:
            self.socio_torcedor = False
            now = datetime.now()
            data_fiscal = now.strftime('%Y%m%d')
            horario = now.strftime('%H%M%S')
            nsu = self.obj_cor_nsu.query.all()
            if nsu:
                nsu = nsu[0]
                horario = nsu.supnsu_tef_hora_transacao
                data_fiscal = nsu.supnsu_tef_data_transacao

            self.sig_tef.finalizar_transacao(0, nroCupom, data_fiscal, horario)

        message_dlg(self, MSG_INFO, 'Nota %06d, cancelada com sucesso!' % nroCupom)
        return True

    @Log
    def KEY_BALANCA_Click(self):
        """ Tecla alterar quantidade do item """
        if config['balanca_active']:
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtCodigo)
            self._CONTROL_FOCUS.clear()
            self.infoText('Obtendo o Peso da Balança', True)
            self._state_item = item_balanca
            # ITEM DE BALANÇA
            self.ui.spinQtda.setDecimals(3)
            try:
                self.ui.spinQtda.setValue(self._obter_peso_balanca())
                self.infoText('Digite o Código do Produto Pesável.', True)
            except DriverError as e:
                self.ui.spinQtda.setDecimals(0)
                self.show_info_proccess(False)
                message_dlg(self, MSG_INFO, 'Balança desconectada')
            except InvalidState:
                self.ui.spinQtda.setDecimals(0)
                message_dlg(self, MSG_INFO, 'Não foi possível obter o peso da balança.')
            except AssertionError:
                self.show_info_proccess(False)
                pass
            return
        else:
            message_dlg(self, MSG_INFO, 'O sistema não está configurado p/ usar balança!\n\n'
                                        'Contacte o Administrador do Sistema.')
        return True

    @Log
    def mensagem_do_foco(self, objeto):
        '''
        Os campos que receberem foco e esfiverem no dicionario abaixo, terao sua
        respectivas mensagens enviadas ao display do teclado
        :param objeto: elemtento que esta recebendo o foco
        :return: True se conseguiu envia o texto e False se nao conseguiu
        '''
        msg_edtCodigo = 'Digite o codigo do item:\n' # campo codigo do item
        if self.ui.lblCodigo.text() == u'Código do Vendedor':
            msg_edtCodigo = 'Codigo do vendedor:\n'

        dict_campo_texto = {
            'mainMenu':          'Menu:\n',
            'edtUser':           'Digite o usuario:\n', # campo usuario
            'edtPwd':            'Digite a senha:\n', # campo senha
            'edtCodigo':          msg_edtCodigo,
            'edtBuscaCliente0':  'Digite o codigo/nome do Cliente:\n', # campo cliente na consulta de cliente
            'edtBuscaCliente1':  'Digite o codigo/nome do Produto:\n', # campo produto na consulta de produto
            'tableBuscaProduto': 'Selecione um item:(v | ^)\n', # campo tabela de itens na consulta de produtos
            'tableBuscaCliente': 'Selecione um cliente:(v | ^)\n', # campo tabela de clientes na consulta de clientes
            'spinQtda':          'Quantidade do item:', # quantidade do item na tela de cupom
            'cbTipoDesc':        'Tipo de desconto:(v | ^)\n%s'\
                                 % self.ui.cbTipoDesc.currentText(), # tipo de desconto R$ ou %
            'spinPgtoDinheiro':  'Pagamento em Dinheiro\nA receber: R$%s\n'\
                                 % self.ui.lblVlrReceber.text(), # campo valor no pgto em Dinheiro
            'tableBuscaCliente': 'Selecione um cliente:(v | ^)\n', # tabela na busca de clientes
            'tableBuscaProduto': 'Selecione um item:(v | ^)\n', # tabela na busca de clientes
            'spinDecItemValor':  '%s:\n%s' % (self.ui.lblDesconto.text(),
                                              self.ui.cbTipoDesc.currentText()), # campo de desconto na tela do cupom
            'listImportacao':    'Aguarde a importacao...', # lista de importacao na tela de importacao
            'tableItens':        'Cancelar Item:(v | ^)\n', # lista de item a serem cancelados, tela de cupom
            'cbTipoTef':         'Tipo:(v | ^)\n%s'\
                                 % (self.ui.cbTipoTef.currentText()), # tipo de cartao, tela de pg cartao
            'cbChTipoCartao':    'Finalizadora:(v | ^)\n%s'\
                                 % self.ui.cbChTipoCartao.currentText(), # campo finalizadora na tela de pg cartao
            'spinCartaoValor':   'Pagamento em Cartao\nA receber: R$%s\n'\
                                 % self.ui.lblVlrReceber.text(), # campo valor na tela de pgto cartao
            'cbTipoTicket':      'Tipo:(v | ^)\n%s'\
                                 % self.ui.cbTipoTicket.currentText(), # campo tipo na tela de pg ticket
            'cbFinTicket':       'Finalizadora:(v | ^)\n%s'\
                                 % self.ui.cbFinTicket.currentText(), # campo finalizadora na tela de pg ticket
            'spinValorTicket':   'Pagamento em Ticket\nA receber: R$%s\n'\
                                 % self.ui.lblVlrReceber.text(), # campo valor na tela de pgto ticket
            'cbOTipoPgto':       'Finalizadora:(v | ^)\n%s'\
                                 % self.ui.cbOTipoPgto.currentText(), # campo tipo na tela de pg outros
            'edtOCodCliente':    'Codigo/Nome do Cliente:\n%s'\
                                 % self.ui.edtOCodCliente.text(), # campo cliente na tala de pg outros
            'spinOValor':        'Pagamento em Outros\nA receber: R$%s\n'\
                                 % self.ui.lblVlrReceber.text(), # campo valor na tela de pg outros
            'cbChTipoCheque':    'Finalizadora:(v | ^)\n%s'\
                                 % self.ui.cbChTipoCheque.currentText(), # campo finalizadora na tela de pg cheque
            'edtChCliente':      'Cliente:\n%s'\
                                 % self.ui.edtChCliente.text(), # campo cliente na tela de pg cheque
            'cbChProp':          'Tipo:(v | ^)\n%s'\
                                 % str(remove_accents(self.ui.cbChProp.currentText())), # campo tipo na tela de pg cheque
            'spinChNroBanco':    'No. Banco:\n', # campo No.banco na tela de pg cheque
            'spinChNroAgencia':  'No. Agencia:\n', # campo No.agencia na tela de pg cheque
            'spinChNroCC':       'No. C. corrente:\n', # campo No. C. corrente na tela de pg cheque
            'spinChNroCheque':   'No. Cheque:\n', # campo No.Vheque na tela de pg cheque
            'edtChDocEmit':      'Doc. Emitente:\n', # campo Doc. Emitente na tela pg cheque
            'dateChVcto':        'Vencimento:\n', # campo Vencimento na tela pg cheque

            'spinChValor':        'Pagamento em Cheque\nA receber: R$%s\n'\
                                 % self.ui.lblVlrReceber.text(), # campo valor na tela de pg outros
            'spinSSDinheiro': 'Denheiro:\n',
            'spinSSCheque': 'Cheque:\n',
            'spinSSCartao': 'Cartao:\n',
            'spinSSTickets': 'Ticket:\n',
            'spinSSOutros': 'Outros:\n',
            'tableSigTefMenu': '%s(v|^)' % self.ui.lblSigTefInfo.text()[:35],
            'edtSigTefTexto': '%s' % self.ui.lblSigTefInfo.text()[:40],
            'spinSigTefValor': '%s' % self.ui.lblSigTefInfo.text()[:40],
                            }

        nome_campo = objeto.objectName()
        linha = 'smak' in config['teclado_selecionado'] and 2 or 3
        if nome_campo == 'edtBuscaCliente':
            if self.ui.stackedBuscaCliPro.currentIndex() == 0: # cliente
                nome_campo += '0'

            else: # produto
                nome_campo += '1'
        Log.info('Foco Campo:%s' % str(nome_campo))
        try:
            if dict_campo_texto.get(nome_campo):
                if not nome_campo == 'edtPwd':
                    texto = dict_campo_texto.get(nome_campo)
                    if 'smak' in config['teclado_selecionado'] and 'Tipo' in nome_campo \
                           or 'cbChProp' in nome_campo:
                        texto = texto.split('\n')
                        self.teclado.envia_display(texto[0])
                        self.teclado.envia_linha_display(texto[1], 2)
                    else:
                        self.teclado.envia_display(texto)

                if nome_campo in ('mainMenu'):
                        if objeto.currentItem() and objeto.currentItem().text():
                            texto = str(remove_accents(objeto.currentItem().text() or ''))
                            self.teclado.envia_linha_display(texto, 2)
                elif nome_campo == 'spinPgtoDinheiro':
                    texto = 'Valor pago:\n'
                    self.teclado.envia_linha_display(texto, linha)
                elif nome_campo == 'spinQtda':
                    self.teclado.envia_linha_display(self.ui.spinQtda.text(), linha)
                elif nome_campo == 'edtPwd':
                    texto = dict_campo_texto.get(nome_campo)
                    if 'smak' in config['teclado_selecionado']:
                        linha = 1
                    self.teclado.envia_linha_display(texto, linha)
                elif nome_campo == 'tableBuscaCliente':
                    row = self.ui.tableBuscaCliente.currentRow()
                    texto = self.ui.tableBuscaCliente.item(row, 1).text()
                    self.teclado.envia_linha_display(texto, 2)
                elif nome_campo == 'tableBuscaProduto':
                    row = self.ui.tableBuscaProduto.currentRow()
                    texto = self.ui.tableBuscaProduto.item(row, 1).text()
                    self.teclado.envia_linha_display(texto, 2)
                elif nome_campo == 'tableItens':
                    row = self.ui.tableItens.currentRow()
                    texto = self.ui.tableItens.item(row, 1).text() + ' ' + \
                            self.ui.tableItens.item(row, 2).text() + ' R$' + \
                            self.ui.tableItens.item(row, 6).text()
                    self.teclado.envia_linha_display(texto[:40], 2)
                elif nome_campo == 'spinCartaoValor':
                    texto = 'Valor pago:\n' + str(self.ui.spinCartaoValor.text())
                    self.teclado.envia_linha_display(texto, linha)
                elif nome_campo == 'spinValorTicket':
                    texto = 'Valor pago:\n' + str(self.ui.spinValorTicket.text())
                    self.teclado.envia_linha_display(texto, linha)
                elif nome_campo == 'spinOValor':
                    texto = 'Valor pago:\n' + str(self.ui.spinOValor.text())
                    self.teclado.envia_linha_display(texto, linha)
                elif nome_campo == 'spinChValor':
                    texto = 'Valor pago:\n' + str(self.ui.spinChValor.text())
                    self.teclado.envia_linha_display(texto, linha)
                elif nome_campo == 'tableSigTefMenu' and self.ui.tableItens.rowCount() > 1:
                    texto = self.ui.tableSigTefMenu.item(0, 1).text()
                    self.teclado.envia_linha_display(texto, 2)
                elif nome_campo == 'spinSigTefValor':
                    texto = '%s' % str(self.ui.spinSigTefValor.text())
                    self.teclado.envia_linha_display(texto, linha)
                elif nome_campo == 'listImportacao':
                    self.teclado.envia_linha_display(texto, 1)


            else: # limpa o display
                self.teclado.limpar_display()

            return True

        except Exception as e:
            Log.error(str(e))

        return False

    @Log
    def setControlFocus(self, old, new, is_escape=False):
        old_name, new_name = ('', '')
        if old:
            old_name = old.objectName()
        if new:
            new_name = new.objectName()
        if not is_escape:
            self._tab_order.append(new)
        if old:
            old.setStyleSheet("")

        if new in [self.ui.edtCodigo, self.ui.edtBuscaCliente, self.ui.edtPwd]:
            self._timer.start(100)

        self._CONTROL_FOCUS = new

        if type(new) in [QtGui.QLineEdit, QtGui.QDoubleSpinBox, QtGui.QComboBox, QtGui.QDateEdit]:
            new.setStyleSheet("background-color: rgba(153, 204, 204, 255); color: rgb(0, 0, 127)")

            if type(old) in [QtGui.QDoubleSpinBox, QtGui.QDateEdit]:
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Up,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Down,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_End,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)

            if type(new) in [QtGui.QDoubleSpinBox, QtGui.QDateEdit]:
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Up,
                                     QtCore.Qt.KeyboardModifiers())
                new.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Down,
                                     QtCore.Qt.KeyboardModifiers())
                new.keyPressEvent(ev)

        elif type(new) == QtGui.QTableWidget:
            new.setStyleSheet('background-color: rgb(255, 252, 205)')

        try:
            new.selectAll()
        except Exception as e:
            pass

        # pinpad
        if not config['config_pinpad_active']:
            self.ui.cbTipoTef.setCurrentIndex(1) # 1 = POS, 0 = TEF

        # Desbloqueia o teclado quando o foco estiver nos campos a baixo
        campos_bloqueados_tef = [self.ui.cbTipoTicket, self.ui.cbTipoTef, self.ui.tableSigTefMenu,
                                 self.ui.spinSigTefValor, self.ui.edtSigTefTexto, self.ui.mainMenu,
                                 self.ui.edtCodigo]
        if new in campos_bloqueados_tef:
            self.bloqueia_tecla = False
            Log.info('bloqueia_tecla: False')

        if self.teclado:
            self.mensagem_do_foco(new)

        return new.objectName()

    def keyPressEvent(self, ev):
        if not ev.key():
            return ''
        if self._executando_importacao:
            return
        if self.emitindo_dav:
            return

        if not self._keyboard_lock or self._CONTROL_FOCUS in (self.ui.edtCodigo,
                                                              self.ui.edtCodigoRes):
            ks = [K0, K1, K2, K3, K4, K5, K6, K7, K8, K9]
            if ev.key() in ks:
                self._CONTROL_FOCUS.keyPressEvent(ev)
                return self.get_key_name(ev.key())
            if not config.get('usarcodificado'):
                if ev.key() == QtCore.Qt.Key_Period.__int__():
                    self.KEY_COMMA_Click()
                if ev.key() in [QtCore.Qt.Key_Plus.__int__(), QtCore.Qt.Key_Minus.__int__()] \
                        and self._CONTROL_FOCUS == self.ui.spinQtda:
                    return
                elif ev.key() not in ks:
                    if ev.key() in [QtCore.Qt.Key_Enter.__int__(), QtCore.Qt.Key_Return.__int__()]:
                        if self._CONTROL_FOCUS == self.ui.edtCodigo:
                            self._returnPressed_edtCodigo()
                        else:
                            self.KEY_RETURN_Click()
                    else:
                        self._CONTROL_FOCUS.keyPressEvent(ev)
                    return
            else:
                if ev.key() == config['bt_entrar']:
                    if self._CONTROL_FOCUS == self.ui.edtCodigo:
                        self._returnPressed_edtCodigo()
                    else:
                        self.KEY_RETURN_Click()
                    return
            if ev.key() in [QtCore.Qt.Key_Enter.__int__(), QtCore.Qt.Key_Return.__int__()]:
                if self._CONTROL_FOCUS == self.ui.edtCodigo:
                    self._returnPressed_edtCodigo()
                else:
                    self.KEY_RETURN_Click()
                return

        return self.get_key_name(ev.key())

    @Log
    def closeEvent(self, ev):
        global RELOAD

        if self._state_coupon in (coupon_closed, mesa_opened, mesa_closed):
            msg = 'Deseja Encerrar o Sistema?'
            if RELOAD:
                msg = 'Deseja Recarregar o Sistema'
            if not self._menu_atualiza:
                if message_dlg(self, MSG_QUESTION, msg) == YES_ROLE:
                    self.teclado and self.teclado.limpar_display()
                    sleep(1)
                    self.teclado and self.teclado.fechar_teclado()
                    sleep(1)
                    Log.info('Fechou janela')
                    ev.accept()
                    if not RELOAD:
                        os.system('killall -9 python')
                        os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
                else:
                    RELOAD = False
                    Log.info('Ignorou fechar janela')
                    ev.ignore()
        else:
            ev.ignore()
            Log.info('Ignorou fechar janela')
        return ev

    @Log
    def _returnPressed_spinSSDinheiro(self):
        value = self.ui.spinSSDinheiro.valueFromText(self.ui.spinSSDinheiro.text())
        value = Decimal(str(value))
        if self.ui.spinSSCheque.isVisible():
            if config['sangriamanual']:
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinSSCheque)
            else:
                self.setControlFocus(self._CONTROL_FOCUS, self._CONTROL_FOCUS)
                self._returnPressed_spinSSOutros()
        else:
            if message_dlg(self, MSG_QUESTION, 'Deseja efetuar o Suprimento') == YES_ROLE:
                Suprimento(self).add_suprimento(self.loja.supcfc_loja, self.caixa_id, value)
                self._set_screen_state(in_menu)

        return value

    @Log
    def _returnPressed_spinSSCheque(self):
        if self.ui.spinSSCartao.isVisible():
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinSSCartao)
        else:
            self.infoText('SSCartao INVISIBLE')
        return self.ui.spinSSCheque.text()

    @Log
    def _returnPressed_spinSSCartao(self):
        if self.ui.spinSSTickets.isVisible():
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinSSTickets)
        else:
            self.infoText('spinSSTickets INVISIBLE')
        return self.ui.spinSSCartao.text()

    @Log
    def _returnPressed_spinSSTickets(self):
        if self.ui.spinSSOutros.isVisible():
            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinSSOutros)
        else:
            self.infoText('spinSSOutors INVISIBLE')
        return self.ui.spinSSTickets.text()

    @Log
    def _remove_till_cash(self, supfin_tipo_finalizadora, value):
        tipos = dict([
            ('D', config['sangria_dinheiro']),
            ('H', config['sangria_cheque']),
            ('C', config['sangria_cartao']),
            ('T', config['sangria_ticket']),
            ('O', config['sangria_outros'])
        ])
        nomes_f = dict([
            ('D', 'Dinheiro'),
            ('H', 'Cheque'),
            ('C', 'Cartao'),
            ('T', 'Ticket'),
            ('O', 'Outros')
        ])
        nome_f = nomes_f.get(supfin_tipo_finalizadora)
        emit_erro = False
        par_s = tipos.get(supfin_tipo_finalizadora).split(',')
        if not par_s:
            emit_erro = True
        elif len(par_s) != 2:
            emit_erro = True
        elif not par_s[0] or not par_s[1]:
            emit_erro = True
        if emit_erro:
            erro_msg = 'O Paramêtro para Sangria %s está configurado incorretamente no PDV-Config' \
                       '\n' % nome_f
            message_dlg(self, MSG_ERROR, 'Não foi possível efetuar a sangria de %s' % nome_f,
                        erro_msg)
            return False

        finalizadora = CaixaCadFinalizadoraNaEcf.get_by(supfec_codigo_na_ecf=par_s[1])
        if not finalizadora:
            self.show_info_proccess(False, '')
            msg = 'Finalizadora não cadastrada no Banco de Dados: %s=%r. ' \
                  'Verique os parametros de Sangria no PDV-Config' % (nome_f, par_s[1])
            message_dlg(self, MSG_ERROR, msg)
            return False
        else:
            nome_finalizadora = finalizadora.name

        try:
            # todo: Faz a sangria aqui
            pass
        except (PaymentAdditionError, CommandError):
            message_dlg(self, MSG_INFO,
                        'A forma de pagamento %r não está cadastrada na ECF!'
                        % nome_finalizadora)
            return False

        return True

    @Log
    def _returnPressed_spinSSOutros(self):
        self._CONTROL_FOCUS.keyPressEvent(QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_End,
                                                          QtCore.Qt.KeyboardModifiers(), text=''))

        if self._state_screen == in_ss:
            if message_dlg(self, MSG_QUESTION, 'Deseja efetuar a Sangria?') == YES_ROLE:
                Sangria(self).sangrar(self.loja.supcfc_loja, self.caixa_id,
                                      self.user_logged.id)

            self._set_screen_state(in_menu)

        elif self._state_screen == in_close_turn:
            if message_dlg(self, MSG_QUESTION, 'Deseja Encerrar o Turno?') == YES_ROLE:
                try:
                    Sangria(self).sangrar(self.loja.supcfc_loja, self.caixa_id,
                                          self.user_logged.id)

                    # Relatorio de Fechamento de Turno
                    retval = self.obj_abertura_fechamento_turno.fechar_turno(self.mov_caixa,
                                                                        config['loja'],
                                                                        config['caixa'],
                                                                        (self.user_logged.id,
                                                                         self.user_logged.name))
                except:
                    self.show_info_proccess(False)
                    self.tratar_except()

            if self._wait_logoff:
                self._set_screen_state(in_login)
                self.ui.lblNumCaixa.setText(u'%03d <b>Fechado' % self.caixa_id)
            else:
                self._set_screen_state(in_menu)
        return self.ui.spinSSOutros.text()

    @Log
    def _returnPressed_menutef(self, item=None, text=None):
        self.ui.tableSigTefMenu.setVisible(False)
        self.ui.lblSigTefInfo.setVisible(False)
        self.ui.edtSigTefTexto.setVisible(False)
        self.ui.spinSigTefValor.setVisible(False)
        self.ui.menuTef.clearFocus()

        menu = item.text()

        self.sig_tef.usando_sitef = True
        self.sig_tef.tipo_finalizadora = None
        now = datetime.now()
        data_fiscal = now.strftime('%Y%m%d')
        horario = now.strftime('%H%M%S')

        if menu == u'Funções Administrativas':
            modalidade = 110
        elif menu == u'Conecta Servidor':
            modalidade = 111
        elif menu == u'Reimprime Cupom':
            modalidade = 112
        elif menu == u'Pré-Autorização':
            modalidade = 115
        elif menu == u'Cancela TEF':
            modalidade = 200
        elif menu == u'Voltar':
            self._set_screen_state(in_menu)
            return

        self.ui.stackedActions.setCurrentIndex(stk_tef)
        self.infoText('Conectando SiTef...')
        self.show_info_proccess(True, 'Conectando SiTef...')
        QtGui.QApplication.processEvents()
        self.sig_tef.inicia_sitef(0.0, '', data_fiscal, horario, self.user_logged.name, modalidade)
        return menu

    @Log
    def emitir_nota_de_dav(self):
        dados_capa = exec_sql("""SELECT * FROM caixa_mov_capa ORDER BY id DESC LIMIT 1""", 'C')
        if not dados_capa or dados_capa[0].get('supcap_tipo') != 'D':
            return message_dlg(self, MSG_INFO, 'O último documento emitido não é DAV!')

        dados_capa = dados_capa[0]
        if message_dlg(self, MSG_QUESTION, 'Emitir nota do DAV %s' %
                                           dados_capa.get('supcap_nro_cupom')) == NO_ROLE:
            return True

        self.show_info_proccess(True, 'Emitindo NOTA...')
        nro_dav = dados_capa.get('supcap_nro_cupom')
        dav_id = dados_capa.pop('id')
        # Atualiza o DAV para cancelado
        self.mov_capa = self.obj_mov_capa.search({'id': dav_id})
        corpos = exec_sql("""SELECT * FROM caixa_mov_corpos WHERE supcor_capa_id = %s""" % dav_id, 'C')
        self.obj_cupom_fiscal.cancelar_dav(automatico=True)

        # criando capa da nota
        dados_capa.update({'supcap_nro_cupom': self.obj_cupom_fiscal.get_numero_nf(),
                           'supcap_tipo': 'V',
                           'supcap_state': ''})
        res = self.obj_mov_capa.create(dados_capa)
        self._coupon = self.mov_capa = res
        self.obj_cupom_fiscal.set_numero_nf()
        for corpo in corpos:
            del corpo['id']
            corpo['supcor_capa_id'] = self.mov_capa.id
            MovCorpos(self).create(corpo)

        for fechamento in exec_sql("""SELECT *
                                      FROM caixa_mov_fechamentos
                                      WHERE supfec_capa_id = %s
                                   """ % dav_id, 'C'):
            fechamento_id = fechamento.pop('id')
            fechamento['supfec_capa_id'] = self.mov_capa.id
            novo_fechamento_id = MovFechamentos(self).create(fechamento).id
            for vencimento in exec_sql("""SELECT *
                                          FROM caixa_mov_vencimentos
                                          WHERE supfve_fechamentos_id = %s""" % fechamento_id, 'C'):
                del vencimento['id']
                vencimento['supfve_fechamentos_id'] = novo_fechamento_id
                MovVencimentos().create(vencimento)

        self.obj_mov_capa.write(self.mov_capa, {'supcap_data_fim': datetime.now(),
                                                              'supcap_ambiente': config['nfce_ambiente']})
        # Enviando nota para o SEFAZ
        if self.obj_cupom_fiscal.envia_nota_servidor():
            self.obj_cupom_fiscal.imprimir_nota(self.mov_capa.supcap_nro_cupom)

        MovCupomServidor().create({'supcse_capa_id': res.id})
        self.show_info_proccess(False, '')
        message_dlg(self, MSG_INFO, 'Nota emitida!')
        return True

    @Log
    def _returnPressed_mainmenu(self, item=None, text=None):
        item_text = self.ui.mainMenu.currentItem().text().encode('utf-8')
        Log.info('Acesso ao Menu Principal: {0:s}'.format(item_text))
        self.ui.mainMenu.clearFocus()
        self.teclado and self.teclado.limpar_display()

        try:
            if item_text not in ['Encerrar Turno', 'LogOff', 'Abrir Turno', 'Redução Z',
                                 'TEF Admin.',
                                 'Menu Fiscal'] and self.mov_caixa:

                if self.mov_caixa.supcax_data_hora_abertura.date() != datetime.today().date():
                    if item_text == 'Redução X':
                        pass
                    else:
                        message_dlg(self, MSG_INFO, 'Encerre o Turno***')
                        return

            if item_text == 'Login':
                self.ui.edtUser.clear()
                self.ui.edtPwd.clear()
                self.ui.stackedActions.setCurrentIndex(stk_page_login)
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtUser)
                self._state_screen = in_login

            elif item_text == 'LogOff':
                if message_dlg(self, MSG_QUESTION, 'Deseja efetuar LogOff') == YES_ROLE:
                    if self.mov_caixa:
                        if message_dlg(self, MSG_QUESTION, 'Deseja encerrar o turno?') == YES_ROLE:
                            if self.user_logged.supope_operador_supervisor == 'O' \
                                    and config['sangria_x_z_suprimento']:
                                if not password_dlg(self, msg_info='LogOff').retval:
                                    return

                            self._state_screen = in_close_turn
                            self._add_remove_cash_till('SA')
                            self._wait_logoff = True
                        else:
                            self._set_screen_state(in_login)
                    else:
                        self._set_screen_state(in_login)

            elif item_text == 'Abrir Turno':
                if self.user_logged:
                    if self.obj_abertura_fechamento_turno.abrir_turno(self.loja.supcfc_loja,
                                                                 self.caixa_id,
                                                                 self.user_logged.supope_codigo):
                        self._set_screen_state(in_coupon)
                else:
                    message_dlg(self, MSG_INFO, 'Efetue login para poder Abrir um Turno')
                return

            elif item_text == 'Encerrar Turno':
                if self.z_pendente:
                    if message_dlg(self, MSG_QUESTION,
                                   'Deseja Emitir Redução-Z Pendente?') == YES_ROLE:
                        ReducaoXZ(self).gerar_reducao_xz(self.loja.supcfc_loja, self.caixa_id, 'Z',
                                                         self.user_logged.supope_codigo)
                        if not self._IdPve and not self.consulta_produto:
                            self.show_info_proccess(True, "Checando PVE's")
                            self.check_pves(days=2)
                            self.show_info_proccess(False)
                    else:
                        return
                else:
                    self._state_screen = in_close_turn
                    return self._add_remove_cash_till('SA')

            elif item_text == 'TEF Admin.':
                ## MENU TEF ###

                # VERIFICA SE AS LIBS EXISTEM DENTRO DO DIRETORIO sigtef/lib/
                lib_emv = os.path.exists(os.path.join(ROOT_PATH, 'sigtef/lib/libemv.so'))
                lib_seppemv = os.path.exists(os.path.join(ROOT_PATH, 'sigtef/lib/libseppemv.so'))
                lib_clisitef = os.path.exists(os.path.join(ROOT_PATH, 'sigtef/lib/libclisitef.so'))
                lib_clisitef32 = os.path.exists(
                    os.path.join(ROOT_PATH, 'sigtef/lib/libclisitef32.so'))

                if not all([lib_emv, lib_seppemv, lib_clisitef, lib_clisitef32]):
                    tef_libs_not_installed = ''
                    if not lib_emv:
                        tef_libs_not_installed += 'libemv.so, '
                    if not lib_seppemv:
                        tef_libs_not_installed += 'libseppemv.so, '
                    if not lib_clisitef:
                        tef_libs_not_installed += 'libclisitef.so, '
                    if not lib_clisitef32:
                        tef_libs_not_installed += 'libclisitef32.so'

                    tef_libs_dir = os.path.join(ROOT_PATH, 'sigtef/lib/')
                    Log.error("TEF não instalado: %s; Bibliotecas não encontradas: %s"
                              % (tef_libs_dir, tef_libs_not_installed))
                    message_dlg(self, MSG_ERROR, text="TEF não instalado")
                    return

                self.ui.stackedMenus.setCurrentIndex(1)
                self.setControlFocus(self._CONTROL_FOCUS, self.ui.menuTef)

            elif item_text == 'NFC-e':
                self._set_screen_state(in_coupon)

            elif item_text == 'Restaurante':
                self._set_screen_state(in_restaurante)

            elif item_text == 'Cancelar NFC-e':
                liberar = True
                liberou_cancelamento = False
                if self.user_logged.supope_operador_supervisor == 'O':
                    dlg = password_dlg(self, msg_info='')
                    if not dlg.retval:
                        liberar = False
                    else:
                        liberou_cancelamento = dlg.user.id
                else:
                    liberou_cancelamento = self.user_logged.id

                if self.user_logged and liberar:
                    if self.mov_caixa:
                        if message_dlg(self, MSG_QUESTION,
                                       'Deseja Cancelar a Última Nota Emitida') == YES_ROLE:
                            try:
                                self.obj_cupom_fiscal.cancelar_nota_fiscal_sefaz(liberou_cancelamento=liberou_cancelamento)
                            except:
                                self.tratar_except()
                    else:
                        message_dlg(self, MSG_INFO, 'E Necessário Abrir um Turno')
                else:
                    message_dlg(self, MSG_INFO, 'Efetue Login')

            elif item_text == 'Suprimento':
                self._state_screen = in_ss
                self._add_remove_cash_till('SU')

            elif item_text == 'Sangria':
                self.ui.spinSSCheque.setValue(0.0)
                self.ui.spinSSCartao.setValue(0.0)
                self.ui.spinSSTickets.setValue(0.0)
                self.ui.spinSSOutros.setValue(0.0)
                self._state_screen = in_ss
                self._add_remove_cash_till('SA')

            elif item_text == 'Redução X':
                liberar = True
                if not self.user_logged:
                    message_dlg(self, MSG_INFO, 'Efetue Login')
                else:
                    if self.par_sangria_x_z_suprimento and \
                                    self.user_logged.supope_operador_supervisor == 'O':
                        dlg = password_dlg(self, msg_info='')
                        if not dlg.retval:
                            liberar = False
                    if liberar:
                        if message_dlg(self, MSG_QUESTION,
                                       'Deseja emitir a Redução X?') == YES_ROLE:
                            ReducaoXZ(self).gerar_reducao_xz(self.loja.supcfc_loja, self.caixa_id,
                                                             'X', self.user_logged.supope_codigo)

            elif item_text == 'Recarregar o Sistema':
                global RELOAD
                RELOAD = True
                self.close()
            elif item_text == 'Desligar Sistema':
                if message_dlg(self, MSG_QUESTION, 'Deseja Desligar o sistema?') == YES_ROLE:
                    self.teclado and self.teclado.limpar_display()
                    sleep(1)
                    self.teclado and self.teclado.fechar_teclado()
                    sleep(1)
                    os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'shutdown -h now'))

            elif item_text == 'Atualizar Sistema':
                if message_dlg(self, MSG_QUESTION, 'Deseja atualizar o sistema?') == NO_ROLE:
                    return False
                if self.user_logged.supope_operador_supervisor == 'O':
                    dlgpwd = password_dlg(self, msg_info=u'Atualização de versão')
                    if dlgpwd.retval:
                        self._libera_restricao = dlgpwd.user.id
                    else:
                        msg = u'Operador: %s.\n\n' \
                              u'Não tem permissão para executar esta operação' \
                              % self.user_logged.name
                        return message_dlg(self, MSG_INFO, msg)
                self._menu_atualiza = True
                self.ui.text_log.clear()
                self.ui.progressBar.setValue(0)
                self.ui.btnKEY_CONSULTA.setDisabled(True)
                self.ui.btnKEY_CLIENTE.setDisabled(True)
                self.ui.stackedActions.setCurrentIndex(stk_update)
                QtGui.QApplication.processEvents()
                QtGui.QApplication.processEvents()
                try:
                    if SigFrenteUp(self).do_checks_version() is True:
                        message_dlg(self, MSG_INFO, "Clique em 'Ok' para concluir a atualização!")
                        self.ui.progressBar.setValue(100)
                        RELOAD = True
                        inc = 0
                        loop = 10
                        while True:
                            inc += 1
                            self.show_info_proccess(True, 'Atualização concluída! \n'
                                                          'Aguarde, o sistema irá reiniciar '
                                                          'em (%d-%d)' % (inc, loop))
                            QtGui.QApplication.processEvents()
                            sleep(1)
                            if inc == loop:
                                break

                        self.teclado and self.teclado.limpar_display()
                        sleep(1)
                        self.teclado and self.teclado.fechar_teclado()
                        sleep(1)
                        self.close()
                    else:
                        self.ui.btnKEY_CONSULTA.setDisabled(True)
                        self.ui.btnKEY_CLIENTE.setDisabled(True)
                        self.KEY_MENU_Click()
                except Exception as err:
                    self._CONSULTA.setDisabled(False)
                    self.ui.btnKEY_CLIENTE.setDisabled(False)
                    self.ui.btnKEY_MENU.setDisabled(False)
            elif item_text == 'Faturar DAV':
                # emite uma nota fiscal do ultimo DAV emitido e cancela o mesmo.
                self.emitir_nota_de_dav()

            return STATE_SCREEN_NAMES.get(self._state_screen)

        except:
            return self.tratar_except()
        return item_text

    @Log
    def _set_controls_default_values(self, ss_visible=False):
        try:
            self.ui.lblInfoEcf.setText(self.trUtf8('IP: %s |  Data carga: %s' % (get_ip(),
                                                                                   self.data_atualizacao)
                                                                                   ))
        except PrinterFileError:
            self.ui.lblInfoEcf.setText(self.trUtf8('IP: %s |  Data carga: %s' % (get_ip(),
                                                                                 self.data_atualizacao)))
        for t in dir(self.ui):
            try:
                if type(eval('self.ui.%s' % t)) == QtGui.QDoubleSpinBox:
                    QtGui.QDoubleSpinBox.setRange(0)
            except:
                pass

        self.ui.frmInfoItens.setVisible(True)
        self.ui.lblCodigo.setText(u'Código do Item')
        self.ui.edtCodigo.clear()

        path_logo = os.path.join(ROOT_PATH, config['local'], 'logo.png')
        self.ui.lblImage.setPixmap(QtGui.QPixmap(path_logo))
        self.ui.lblImage_2.setPixmap(QtGui.QPixmap(path_logo))
        self.ui.frmDescAcre.setVisible(False)
        self.ui.lblNomeCliente.clear()
        #CONTROLES DA SANGRIA/SUPRIMENTO
        self.ui.spinSSCheque.setVisible(ss_visible)
        self.ui.spinSSCartao.setVisible(ss_visible)
        self.ui.spinSSTickets.setVisible(ss_visible)
        self.ui.spinSSOutros.setVisible(ss_visible)
        self.ui.label_edit_13.setVisible(ss_visible)
        self.ui.label_edit_14.setVisible(ss_visible)
        self.ui.label_edit_15.setVisible(ss_visible)
        self.ui.label_edit_16.setVisible(ss_visible)
        di = datetime.today()
        for x in dir(self.ui):
            if x[0:2] == '__':
                pass
            else:
                widget = eval('self.ui.%s' % x)
                type_widget = type(widget)
                if type_widget in [QtGui.QLineEdit, QtGui.QDoubleSpinBox, QtGui.QDateEdit,
                                   QtGui.QComboBox]:
                    if type_widget == QtGui.QLineEdit:
                        widget.clear()
                    if type_widget in [QtGui.QDoubleSpinBox]:
                        widget.clear()
                        widget.setValue(0)
                    if type_widget in [QtGui.QDateEdit]:
                        widget.clear()
                    if type_widget == QtGui.QComboBox:
                        widget.setCurrentIndex(0)

        self.ui.cbChProp.clear()
        for t in ['Próprio', 'Terceiros']:
            self.ui.cbChProp.addItem(self.trUtf8(t))
        self.ui.cbChProp.setCurrentIndex(0)
        # Controles desconto item
        self.ui.cbTipoDesc.clear()
        for t in ['R$', '%']:
            self.ui.cbTipoDesc.addItem(t)
        self.ui.cbTipoDesc.setCurrentIndex(0)
        self.ui.spinDecItemValor.setDecimals(3)
        self.ui.spinDecItemValor.setValue(0)

        # Apaga lista de itens
        self.clear_widget_table(self.ui.tableItens)
        self.ui.spinQtda.setValue(1)
        self.ui.lblItems.setText(u'')
        return True

    @Log
    def clear_widget_table(self, table):
        # Apaga lista de itens
        rows_count = table.rowCount()
        table.horizontalHeader().setVisible(True)
        # Verifica se hà dados
        if rows_count:
            rows = range(0, rows_count + 1)
            rows.reverse()
            for r in rows:
                table.removeRow(r)
        return rows_count

    def tocar_beep(self):
        if not self.ativou_beep:
            return True
        try:
            self.ativou_beep = False
            os.system('beep -l200 -f750 -r4')
            Log.info('Beep executado')
        except Exception as e:
            Log.error('Erro no beep: %s' % e)

        return


    def ativa_beep(self):
        if not config['beep']:
            Log.info('Beep desativado')
            return True

        Log.info('Ativou Beep')
        if not 'ativou_beep' in self.__dict__:
            self.ativou_beep = True

            back = BackgroundTask
            back(0.3, self.tocar_beep).start()

        self.ativou_beep = True
        return True

    @Log
    def _returnPressed_edtCodigo(self):
        self.bloqueia_importacao()
        self.KTEXT = ''
        if config['apenas_ficha']:
                self.ui.edtCodigo.clear()
                self.desbloqueia_importacao()
                self.ativa_beep()
                return message_dlg(self, MSG_INFO, 'Atenção!\n\nNão é permitido emitir NFC-e '
                                                   'neste caixa, apenas fichas!')

        if not self.verifica_certificado():
            self.ui.edtCodigo.clear()
            self.desbloqueia_importacao()
            self.ativa_beep()
            return True

        if len(self.mov_corpos) >= 999:
            self.ui.edtCodigo.setText('')
            self.desbloqueia_importacao()
            self.ativa_beep()
            return message_dlg(self, MSG_INFO, 'Atenção!\n\nLimite máximo de 999 itens por cupom '
                                               'atingido!\nFeche o cupom e continue a venda em '
                                               'outro cupom! ')
        cod_item = self.ui.edtCodigo.text()
        # if config['usar_ficha'] and (not cod_item.startswith('99') or len(cod_item) < 15 or \
        #                              cod_item[-1] != modulo10(cod_item[:-1])):
        #     return message_dlg(self, MSG_INFO, 'Atenção!\n\nCódigo de ficha inválido!')

        if self._state_coupon == coupon_finish and cod_item:
            self.ui.edtCodigo.setText('')
            self.ativa_beep()
            return message_dlg(self, MSG_INFO, 'O cupom já foi finalizado!' +
                               'Escolha uma forma de pagamento')

        if self._state_coupon in [coupon_add_discount, coupon_add_surchage]:
            self._state_coupon = coupon_opened
        if self._comanda_ativa:
            self._comanda_ativa = False
            status_fatura_comanda = self.faturar_comanda(cod_item)
            if not status_fatura_comanda:
                self._comanda_ativa = True
            else:
                self.ui.btnKEY_COMANDA.setStyleSheet(
                    "/buttons/images/buttons/disabled.png;")
            return True

        if cod_item:
            cod_ficha = False
            desconto = Decimal(0)
            valor_ficha = Decimal(0)
            state_item = item_none
            if config['usar_ficha'] and cod_item.startswith('99'):
                cod_item, cod_ficha, valor_ficha, desconto = self.obj_ficha.get_dados_item_ficha(cod_item)
                self.ui.spinQtda.setValue(1)
                self.ui.spinDecItemValor.setValue(desconto)
                state_item = desconto and item_add_discount or state_item
                if not cod_item:
                    self.ui.edtCodigo.clear()
                    self.desbloqueia_importacao()
                    return False

            if self.ui.lblCodigo.text() == u'Código do Item':
                res_item = self._search_item(cod_item)
                if res_item and res_item.active == False:
                    self.ativa_beep()
                    message_dlg(self, MSG_INFO, 'Atenção!\n\n Item inativo, venda não permitida!'
                                                ' \n\n Informe o fato ao supervisor')
                    self.vlr_item_dav = False
                    self.ui.edtCodigo.clear()
                    self.desbloqueia_importacao()
                    return cod_item

                self.ui.edtCodigo.clear()

                if res_item:
                    self.ui.edtCodigo.setText(str(res_item.supite_codigo_item))
                    serie = False
                    if self._state_item == item_none:
                        if not serie:
                            if self.comanda_id:
                                self.vendedor_codigo = self.vendedor_comanda_ids
                                for x in self.vendedor_comanda_ids:
                                    if res_item == x[1][0]:
                                        self.vendedor_codigo = x[2]
                                        self._nome_vendedor = x[3]
                                if type(self.vendedor_codigo) is not int:
                                    self.vendedor_codigo = 0
                                    self._nome_vendedor = None

                            self.obj_cupom_fiscal.add_item(res_item,
                                                      Decimal(str(self.ui.spinQtda.value())),
                                                      vendedor_codigo=self.vendedor_codigo,
                                                      nome_vendedor=self._nome_vendedor,
                                                      discount_surcharge=[item_add_discount, desconto],
                                                      cod_ficha=cod_ficha)
                        else:
                            self.ui.edtCodigo.setText(str(res_item.supite_codigo_item))
                            if len(serie) == 1:
                                self.obj_cupom_fiscal.add_item(res_item,
                                                          Decimal(str(self.ui.spinQtda.value())),
                                                          vendedor_codigo=self.vendedor_codigo,
                                                          nome_vendedor=self._nome_vendedor,
                                                          serie=serie[0].NroSerie,
                                                          cod_ficha=cod_ficha)
                            else:
                                self.ui.frmSerie.setVisible(True)
                                self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbSerie)

                        return True

                    # DESCONTO/ACRÉSCIMO
                    if self._state_item in [item_add_discount, item_add_surchage]:
                        self.ui.frmDescAcre.setVisible(True)
                        if self._state_item == item_add_discount:
                            self.ui.lblDesconto.setText(u'Desconto no Item')
                        elif self._state_item == item_add_surchage:
                            self.ui.lblDesconto.setText(u'Acréscimo no Item')
                        if not serie:
                            self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinQtda)
                        else:
                            self.ui.frmSerie.setVisible(True)
                            self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbSerie)

                    # ITEM DE BALANÇA
                    if self._state_item == item_balanca:
                        if res_item.supite_pesavel == 'N':
                            self.ativa_beep()
                            msg = 'O Item: [%s - %s]. Não é um produto pesável.' % (
                                str(res_item.supite_codigo_item), res_item.name.rstrip(' '))
                            message_dlg(self, MSG_INFO, msg)
                            self.ui.edtCodigo.clear()
                            self.ui.spinQtda.setDecimals(0)
                            self.ui.spinQtda.setValue(1)
                            self.ui.lblItems.clear()
                            self.ui.lblVlrItem.setText(u'%1.2f' % 0.0)
                            self._state_item = item_none
                            self.check_button_status()
                            return
                        self.obj_cupom_fiscal.add_item(res_item,
                                                  Decimal(str(self.ui.spinQtda.value())),
                                                  vendedor_codigo=self.vendedor_codigo,
                                                  nome_vendedor=self._nome_vendedor,
                                                  cod_ficha=cod_ficha)
                else: # nao encontrou item
                    self.desbloqueia_importacao()

            elif self.ui.lblCodigo.text() == u'Código do Vendedor':
                codigo_invalido = False
                try:
                    if int(cod_item) > 10000000:
                        codigo_invalido = True
                except:
                    codigo_invalido = True

                if codigo_invalido:
                    self.ativa_beep()
                    msg = 'Código inválido do vendedor!'
                    message_dlg(self, MSG_INFO, msg)
                    self.ui.edtCodigo.clear()
                    self.desbloqueia_importacao()
                    return

                vendedor = CaixaCadVendedor.get_by(supven_codigo=int(cod_item))
                if vendedor:
                    if self.vendedor_codigo != vendedor.supven_codigo:
                        self.vendedor_codigo = vendedor.supven_codigo
                        lbl = self.user_logged.name + ' \ ' + vendedor.name
                        self._nome_vendedor = vendedor.name
                        self.ui.lblNomeUser.setText(lbl)
                        if not self.emitindo_dav:
                            message_dlg(self, MSG_INFO,
                                        '<h2>Vendedor selecionado:</h2></br></br><h2 align="center" '
                                        'style="text-align: center;"><strong>'
                                        '<span style="color:#FF0000;">%03d - %s</span></strong></h2>'
                                        % (vendedor.supven_codigo, vendedor.name))
                            self.teclado and self.teclado.envia_linha_display(
                                ('Ad. vendedor: %s' % vendedor.name)[:40], 2)
                    elif not self.emitindo_dav:
                        message_dlg(self, MSG_INFO, '<h2>Você já é o vendedor selecionado!</h2>')
                elif not self.emitindo_dav:
                    message_dlg(self, MSG_INFO, '<h2>Vendedor não cadastrado!</h2>')

                self.ui.lblCodigo.setText(u'Código do Item')
                self.ui.edtCodigo.clear()
                if self._coupon_totalize:
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.stackedActions)

            elif self.ui.lblCodigo.text() == u'Código do Cliente':
                self._search_cliente(self.ui.edtCodigo.text(), process_messages=True)
                self.ui.frmInfoItens.setVisible(True)
                self.ui.lblCodigo.setText(u'Código do Item')
                self.ui.edtCodigo.clear()
                if self._coupon_totalize:
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.stackedActions)
                self.ui.edtCodigo.setMaxLength(14)

            elif self.ui.lblCodigo.text() == u'Mensagem no Cupom Fiscal':
                self._message_in_coupon = self.ui.edtCodigo.text()
                self.ui.lblCodigo.setText(u'Código do Item')
                self.ui.edtCodigo.clear()
                if self._coupon_totalize:
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.stackedActions)
                self.ui.edtCodigo.setMaxLength(14)
            elif self.ui.lblCodigo.text() == u'Doação':
                self.vr_doacao = Decimal(self.ui.edtCodigo.text().replace(',', '.')
                                         ).quantize(TWOPLACES)
                if self.vr_doacao <= 0 :
                    self.ui.edtCodigo.clear()
                    return message_dlg(self, MSG_ERROR, u'Digite um valor válido!')

                self._coupon_amount = self.vr_doacao
                self._coupon_subtotal = self._coupon_amount
                self.atualiza_valor_a_pagar()
                self.atualiza_desconto()
                self.ui.lblCodigo.setText(u'Código do Item')
                self.ui.edtCodigo.clear()
                self.KEY_DINHEIRO_Click()
                self.ui.edtCodigo.setMaxLength(14)
        else:
            if self._state_item in [item_add_discount, item_add_surchage]:
                if self._coupon_amount > 0:
                    self.ui.frmDescAcre.setVisible(True)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.cbTipoDesc)
                    if self._state_item == item_add_discount:
                        self._state_coupon = coupon_add_discount
                        self.ui.lblDesconto.setText(u'Desconto no Cupom')
                    elif self._state_item == item_add_surchage:
                        self._state_coupon = coupon_add_surchage
                        self.ui.lblDesconto.setText(u'Acréscimo no Cupom')
                else:
                    if self._state_item == item_add_discount:
                        self.infoText('Não posso efetuar Desconto no Cupom')
                    elif self._state_item == item_add_surchage:
                        self.infoText('Não posso efetuar Acréscimo no Cupom')
                    self._clear_add_item_st()
            elif self.ui.lblCodigo.text() == u'Mensagem no Cupom Fiscal':
                self.infoText('↓↓↓ Digite a Mensagem ↓↓↓')
            else:
                msg = '↓↓↓ Digite o Código do Item ↓↓↓'
                if self.ui.lblCodigo.text() == u'Doação':
                    msg = u'↓↓↓ Digite o valor da Doação ↓↓↓'

                self.infoText(msg)
        return cod_item

    @Log
    def _search_cliente(self, cpf_cnpj, process_messages=False, troca_cli=True, msg_not_cli=True):
        """Busca um registro na tabela TblCad_Clientes a patir do código informado
        :param: cod_item: código do produto
        :rtype: <class 'models.caixa.TblCad_Itens'> or None
        """
        if len(cpf_cnpj) >= 11:
            _cpf_cnpj = cpf_cnpj.rjust(15, '0')  # 14 zeros a esquerda
            cliente = CaixaCadClientes.get_by(supcli_cnpj_cpf=_cpf_cnpj)
            msg = 'CPF/CNPJ nao localizado!'
            if not cliente:
                try:
                    msg = 'Codigo interno do cliente nao localizado!'
                    codigo = int(cpf_cnpj)
                    cliente = CaixaCadClientes.get_by(supcli_codigo_interno=codigo)
                except:
                    self.infoText(msg)
                    return
        else:
            try:
                msg = 'Codigo interno do cliente nao localizado!'
                codigo = int(cpf_cnpj)
                cliente = CaixaCadClientes.get_by(supcli_codigo_interno=codigo)
            except:
                self.infoText(msg)
                return False

        if cliente:
            res = cliente
            if res.supcli_cnpj_cpf[-14:] == self.loja.supcfc_cnpj[-14:]:
                message_dlg(self, MSG_INFO, 'O CNPJ do emissor de NFC-e não poder ser cliente!')
                return False

            # PROCESSAR MENSAGENS
            if process_messages and not self._IdDav:
                msg_str = '%s\n' % res.name
                if res.supcli_status == 'B':
                    msg_str += 'BLOQUEADO\n\n'
                else:
                    msg_str += 'LIBERADO\n\n'

                if msg_str and config['restricoesclientes']:
                    limite = ((res.supcli_limite_compras or 0) - (res.supcli_total_em_aberto or 0))
                    msg_str += 'Limite de Compras: R$ %.2f\n\n' % limite

                sql = """
                    SELECT *
                    FROM   caixa_cad_mensagens_clientes
                    WHERE  supclm_cnpj_cpf LIKE '%{cpf_cnpj}'
                """.format(cpf_cnpj=str(int(cliente.supcli_cnpj_cpf)))

                mensagens = exec_sql(sql, 'C', ret=True)
                msg_str += '*** MSG CLIENTE ***\n'

                for mensagem in mensagens:
                    if mensagem.get('supclm_tipo'):
                        msg_str += '%s\n' % mensagem.get('supclm_mensagem')

                message_dlg(self, MSG_INFO, '%s' % msg_str)

            if not self.cliente.name or troca_cli:
                self.cliente = res
                self.cliente_total_aberto = res.supcli_total_em_aberto
                if self.mov_capa:
                    self.mov_capa.supcap_cliente = res.supcli_cnpj_cpf

                self.ui.lblNomeCliente.setText(u'%s' % res.name)
                self.infoText(u'Cliente selecionado: %s' % res.name)
        else:
            if msg_not_cli and process_messages:
                message_dlg(self, MSG_INFO, msg)
            else:
                self.infoText(msg)
            res = None

        return res

    def ajusta_valor(self, qtd, vr_unitario, valor):
        total_item = ('%s' % (qtd * vr_unitario))
        total_item = Decimal(total_item[:total_item.index('.') + 3])
        if total_item == valor:
            return qtd, vr_unitario

        elif total_item < valor:
            self.vr_unitario_calc = vr_unitario
            return self.ajusta_valor(qtd, vr_unitario + Decimal('0.01'), valor)

        return False

    def ajusta_qtd(self, qtd, vr_unitario, valor):
        total_item = ('%s' % (qtd * vr_unitario))
        total_item = Decimal(total_item[:total_item.index('.') + 3])
        if total_item == valor:
            return qtd, vr_unitario

        elif total_item < valor:
            return self.ajusta_qtd(qtd + Decimal('0.001'), vr_unitario, valor)

        return False

    @Log
    def _busca_produto_balanca(self, cod, consulta=None):
        cod_prod = cod[1:config['digitos'] + 1]
        if config['codigocomdigito']:
            cod_prod = '%s%d' % (cod_prod, ean(int(cod_prod)))
        cod_prod = '%014d' % int(cod_prod)
        item = CaixaCadItens.get_by(supite_codigo_item=cod_prod, supite_loja=self.loja.supcfc_loja)
        if item and not consulta:
            if not item.supite_pesavel:
                message_dlg(self, MSG_ERROR, "Atenção!\n\nO item '%s' não é de balança" %cod_prod)
                return 99

            inteiro = cod[1 + config['digitos']:-3]
            casas = cod[-3:-1]
            vlr_str = '%d.%02d' % (int(inteiro), int(casas))
            if item.supite_fracionado or cod.startswith('2'):
                self.ui.spinQtda.setDecimals(3)
            else:
                self.ui.spinQtda.setDecimals(0)

            self.vr_unitario_calc = Decimal(0)
            total_item = Decimal(vlr_str)
            qtd = '%s' % float(total_item / item.supite_preco)
            qtd = Decimal(qtd[:qtd.index('.') + 4])
            qtda_t = self.ajusta_qtd(qtd, item.supite_preco, total_item)
            if not qtda_t:
                qtda_t = self.ajusta_valor(qtd, item.supite_preco + Decimal('0.01'), total_item)
                if not qtda_t:
                    qtda_t = qtd, self.vr_unitario_calc

            if len(qtda_t) > 1:
                if len('%s' % qtda_t[0]) > 8: # limite da ECF
                    message_dlg(self, MSG_ERROR, ("Atenção!\n\nA quantidade máxima aceita pela ECF "
                                                  "é 9999.99") % cod_prod)
                    return 99

                Log.info('Qtd. : %s' % qtda_t[0])
                Log.info('Preço do item alterado: %s > %s' % (item.supite_preco, qtda_t[1]))
                self.vlr_item_dav = qtda_t[1]

            self.ui.spinQtda.setValue(qtda_t[0])
            return item

        elif consulta:
            return exec_sql(consulta + """ and supite_codigo_item = '%s' """ \
                                   % cod_prod.rjust(14, '0'), 'C', ret=True)
        return False

    @Log
    def _search_item(self, cod_item):
        """Busca um registro na tabela TblCad_Itens a patir do código informado
        :param: cod_item: código do produto
        :rtype: <class 'models.caixa.TblCad_Itens'> or None
        """
        prod_balanca = False
        res = False
        if cod_item.startswith('2') and len(cod_item) == 13:
            prod_balanca = self._busca_produto_balanca(cod_item)

        if prod_balanca:
            if prod_balanca == 99:
                return False

            _cod = prod_balanca.supite_codigo_item
            item = prod_balanca
        else:
            _cod = cod_item.rjust(14, '0')  # 14 zeros a esquerda
            # os itens inativos podem ser vendidos se passa o codigo de barras deles
            item = CaixaCadItens.get_by(supite_codigo_item=_cod, supite_loja=self.loja.supcfc_loja)
            if item:
                if item.supite_pesavel and config['itempesaveletiquetabalanca']:
                    return message_dlg(self, MSG_INFO, 'Itens pesáveis somente com etiqueta!')

        if item:
            item_name = item.name if config['descricao_resumida'] else item.supite_descricao_completa
            self.ui.lblItems.setText(item_name)
            preco = item.supite_preco if not self.vlr_item_dav else self.vlr_item_dav
            self.ui.lblVlrItem.setText(u'%1.2f' % preco)
            if preco <= 0:
                if CadKits.get_kit_by_codigo_barra(_cod):
                    return item
                Log.info('Item %s sem preço de venda' % str(item.supite_codigo_item))
                message_dlg(self, MSG_INFO,
                            'Item %s sem preço de venda' % str(item.supite_codigo_item))
                return
            if item.supite_fracionado or (cod_item.startswith('2') and len(cod_item) == 13):
                self.ui.spinQtda.setDecimals(3)
            else:
                if not Decimal(self.ui.spinQtda.text().replace(',', '.'))._isinteger():
                    self.ativa_beep()
                    self.ui.lblItems.setText(u'')
                    self.ui.lblVlrItem.setText(u'%1.2f' % 0.00)
                    msg = 'A quantidade <h1><font color="red">{0!s}</font></h1> para o item  ' \
                          '<h2><font color="red">{1!s}</font></h2> não pode ser ' \
                          'fracionada.'.format(self.ui.spinQtda.text(), item_name)
                    self.vlr_item_dav = False
                    self.ui.spinQtda.setValue(1)
                    message_dlg(self, MSG_INFO, msg)
                    return False

                self.ui.spinQtda.setDecimals(0)
            res = item

            if config['local'] and config['check_dir_logo_itens']:
                for ext in ['.png', '.jpg', '.jpeg', '.gif']:
                    image = '%s/%s%s' % (config['local'], _cod, ext)
                    image_exist = os.path.exists(image)
                    if image_exist:
                        self.ui.lblImage.setPixmap(QtGui.QPixmap(image))
                        self.ui.lblImageProdFoot.setPixmap(QtGui.QPixmap(image))
                        break
                    else:
                        self.ui.lblImage.setPixmap(QtGui.QPixmap("%s/logo.png" % config['local']))
                        self.ui.lblImageProdFoot.setPixmap(
                            QtGui.QPixmap("%s/logo.png" % config['local']))

        else:
            self.ui.lblItems.setText(u'')
            self.ui.lblVlrItem.setText(u'%1.2f' % 0.00)
            # Quando não encontrar o codigo do item será exibida uma caixa de dialogo informando
            # o usuario ao invez de alterar o titulo da janela -- >
            # self.infoText(u'Código não cadastrado: %s' % cod_item)
            if self._IdPve:
                self.ativa_beep()
                msg = 'Código do item não cadastrado: <br><center><h1><font color="red">%s' \
                      '</font></h2>' % str(_cod)
                if self._state_coupon != coupon_closed and not self._coupon_totalize:
                    ecf_subtotal = self.send_command("get_coupon_subtotal()")
                    msg += '<br><br>A Pré-Venda será cancelada.'
                    message_dlg(self, MSG_NONE, msg)
                    self.KEY_DINHEIRO_Click()
                    self.ui.spinPgtoDinheiro.setValue(ecf_subtotal)
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.spinPgtoDinheiro)
                    self.KEY_RETURN_Click()
                    self.cancelar_nota_fiscal(automatico=True)
                else:
                    msg = 'O Cupom já foi totalizado, você não pode mais adicionar itens'
                    message_dlg(self, MSG_NONE, msg)
                    return
                self._clear_globals(coupon_closed)

            else:
                self.ativa_beep()
                message_dlg(self, MSG_NONE,
                            'Código do item não cadastrado: <br><center><h1><font color="red">' +
                            str(cod_item) + '</font></h2>')

            res = None

        # ao fechar uma mensagem, o display volta com o que estava escrito antes
        if self.teclado:
            self.setControlFocus(self._CONTROL_FOCUS, self._CONTROL_FOCUS)

        return res

    @Log
    def add_item_in_tableitens(self, cod_item, sequence, item_name, quantity, unit, price,
                               subtotal, desconto_acrescimo=[], venda_cancelamento='V',
                               insert=False, cod_ficha=False):

        tableItens = self.ui.tableItens
        desc = desconto_acrescimo[0] * -1 if desconto_acrescimo[0] < 0 else desconto_acrescimo[0]
        acre = desconto_acrescimo[1] * -1 if desconto_acrescimo[1] < 0 else desconto_acrescimo[1]

        if insert:
            self._coupon_subtotal += subtotal.quantize(TWOPLACES)
            rows_t = tableItens.rowCount()
            sequence -= 1
            if rows_t != sequence:
                sequence = rows_t

            indicador = ''
            if desc > 0:
                self._indicador_desc_acre = item_add_discount
                indicador = 'D'
            elif acre > 0:
                self._indicador_desc_acre = item_add_surchage
                indicador = 'A'
            if indicador:
                sess.begin()
                self._coupon.supcap_alteracao = indicador
                sess.commit()
        else:
            subtotal = Decimal(subtotal + acre - desc)
            desconto_acrescimo = list(desconto_acrescimo)
            desconto_acrescimo[0] *= -1
            tuple(desconto_acrescimo)
            sequence = tableItens.rowCount() # na recuperacao do cupom nao pode haver buraco na sequencia, caso um item for cancelado

        tableItens.insertRow(sequence)

        if venda_cancelamento == 'V':
            icon = self.icon_check
            checkstate = QtCore.Qt.CheckState.Checked
            self._coupon_amount += subtotal.quantize(TWOPLACES)
        else:
            icon = self.icon_delete
            checkstate = QtCore.Qt.CheckState.Unchecked

        # Cod Item
        newItem = QtGui.QTableWidgetItem('%s' % cod_item)
        newItem.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newItem.setIcon(icon)
        newItem.setFont(self._font)

        # Descrição
        newDesc = QtGui.QTableWidgetItem(item_name)
        newDesc.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newDesc.setFont(self._font)

        # Quantidade
        newQtda = QtGui.QTableWidgetItem('%1.3f' % quantity)
        newQtda.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newQtda.setFont(self._font)

        # Unidade
        newUnid = QtGui.QTableWidgetItem(unit)
        newUnid.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newUnid.setFont(self._font)

        # Valor Unitário
        newVUni = QtGui.QTableWidgetItem('%1.2f' % price)
        newVUni.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newVUni.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        newVUni.setFont(self._font)

        # Desconto
        desconto_acrescimo = desconto_acrescimo[0] if desconto_acrescimo[0] else desconto_acrescimo[1]
        newVDes = QtGui.QTableWidgetItem('%1.2f' % desconto_acrescimo)
        newVDes.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newVDes.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        newVDes.setFont(self._font)

        # Valor Total
        newVTot = QtGui.QTableWidgetItem('%1.2f' % subtotal)
        newVTot.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newVTot.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        newVTot.setFont(self._font)

        # Ativo
        newAtiv = QtGui.QTableWidgetItem(venda_cancelamento)
        newAtiv.setFlags(QtCore.Qt.ItemIsUserCheckable)
        newAtiv.setCheckState(checkstate)
        newAtiv.setFont(self._font)

        # WIDGET OS NEW ITENS NA TABLE ITENS
        tableItens.setItem(sequence, 0, newItem)
        tableItens.setItem(sequence, 1, newDesc)
        tableItens.setItem(sequence, 2, newQtda)
        tableItens.setItem(sequence, 3, newUnid)
        tableItens.setItem(sequence, 4, newVUni)
        tableItens.setItem(sequence, 5, newVDes)
        tableItens.setItem(sequence, 6, newVTot)
        tableItens.setItem(sequence, 7, newAtiv)
        if cod_ficha:
            newFicha = QtGui.QTableWidgetItem('%s' % cod_ficha)
            newFicha.setFlags(QtCore.Qt.ItemIsUserCheckable)
            newFicha.setCheckState(checkstate)
            newFicha.setFont(self._font)
            tableItens.setItem(sequence, 8, newFicha)

        tableItens.scrollToBottom()
        tableItens.resizeColumnsToContents()
        tableItens.horizontalHeader().setStretchLastSection(True)

        self.ui.lblVlrTotal.setText(u'%1.2f' % self._coupon_amount)
        self.atualiza_valor_a_pagar()
        self.ui.lblVlrTotal_2.setText(u'%1.2f' % self._coupon_amount)

        self.ui.spinQtda.setValue(1)

        self._item_discount = Decimal(0)
        self._item_quantity = Decimal(1)
        self._state_item = item_none
        self.ui.lblImage.setPixmap(QtGui.QPixmap("%s/logo.png" % config['local']))
        self.ui.lblImageProdFoot.setPixmap(QtGui.QPixmap("%s/logo.png" % config['local']))
        self._CONTROL_FOCUS = self.ui.edtCodigo

        return True

    @Log
    def _add_desc_acres_in_table(self, indicador, valor):
        tableItens = self.ui.tableItens
        sequence = tableItens.rowCount()
        # ADICIONA AO LIST WIDGET
        tableItens.insertRow(sequence)
        brush_red = QtGui.QBrush(QtCore.Qt.red)
        brush_green = QtGui.QBrush(QtCore.Qt.darkGreen)
        brush = brush_red if indicador == coupon_add_surchage else brush_green
        # Descrição
        descricao = u'ACRÉSCIMO' if indicador == coupon_add_surchage else 'DESCONTO'
        self._indicador_desc_acre = indicador
        newDesc = QtGui.QTableWidgetItem(descricao)
        newDesc.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newDesc.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        newDesc.setForeground(brush)
        # Desconto
        valor = valor if indicador == coupon_add_surchage else valor * -1
        newVDes = QtGui.QTableWidgetItem('%1.2f' % valor)
        newVDes.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newVDes.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        newVDes.setForeground(brush)
        # Valor Total
        newVTot = QtGui.QTableWidgetItem('%1.2f' % Decimal(self._coupon_subtotal + valor))
        newVTot.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
        newVTot.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        # WIDGET OS NEW ITENS NA TABLE ITENS
        tableItens.setItem(sequence, 1, newDesc)
        tableItens.setItem(sequence, 5, newVDes)
        tableItens.setItem(sequence, 6, newVTot)
        tableItens.scrollToBottom()
        return True

    @Log
    def resize_table_itens(self):
        tableItens = self.ui.tableItens
        w = tableItens.width()
        w_item = (21 * w) / 100
        w_desc = (37 * w) / 100
        w_qtda = (8 * w) / 100
        w_un = (4 * w) / 100
        w_vuni = (12 * w) / 100
        w_dsc = (7 * w) / 100
        w_vtotal = (11 * w) / 100
        print w_item, w_desc, w_qtda, w_un, w_vuni, w_dsc, w_vtotal, w
        tableItens.setColumnWidth(0, w_item)  # Items
        tableItens.setColumnWidth(1, w_desc)  # Descrição
        tableItens.setColumnWidth(2, w_qtda)  # Quantidade
        tableItens.setColumnWidth(3, w_un)  # Unidade
        tableItens.setColumnWidth(4, w_vuni)  # Valor Unitário
        tableItens.setColumnWidth(5, w_dsc)  # Desconto
        tableItens.setColumnWidth(6, w_vtotal)  # Valor Total
        return True

    @Log
    def _list_update(self, item_id, item, item_amount):
        """ Atualiza a lista de itens e os valores na tela """

        self._coupon_amount += item_amount

        tableItens = self.ui.tableItens
        tableItens.insertRow(item_id)
        self.last_item = item_id
        # No. Item
        newItem = QtGui.QTableWidgetItem('%03d' % item_id)
        newItem.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled)
        newItem.setIcon(self.icon_check)
        newItem.setFont(self._font)
        # Descrição
        item_name = item.name if config['descricao_resumida'] else item.supite_descricao_completa
        newDesc = QtGui.QTableWidgetItem(item_name)
        newDesc.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled)
        newDesc.setFont(self._font)
        # Quantidade
        newQtda = QtGui.QTableWidgetItem('%1.3f' % self._item_quantity)
        newQtda.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled)
        newQtda.setFont(self._font)
        # Unidade
        newUnid = QtGui.QTableWidgetItem(item.supite_unidade)
        newUnid.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled)
        newUnid.setFont(self._font)
        # Valor Unitário
        newVUni = QtGui.QTableWidgetItem('%1.2f' % item.supite_preco)
        newVUni.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled)
        newVUni.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        newVUni.setFont(self._font)
        # Desconto
        newVDes = QtGui.QTableWidgetItem('%1.2f' % self._item_discount)
        newVDes.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled)
        newVDes.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        newVDes.setFont(self._font)
        # Valor Total
        newVTot = QtGui.QTableWidgetItem('%1.2f' % item_amount)
        newVTot.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled)
        newVTot.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        newVTot.setFont(self._font)
        # Ativo
        newAtiv = QtGui.QTableWidgetItem('True')
        newAtiv.setFlags(
            QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled |
            QtCore.Qt.ItemIsUserCheckable)
        newAtiv.setCheckState(QtCore.Qt.CheckState.Checked)
        newAtiv.setFont(self._font)
        # WIDGET OS NEW ITENS NA TABLE ITENS
        tableItens.setItem(item_id, 0, newItem)
        tableItens.setItem(item_id, 1, newDesc)
        tableItens.setItem(item_id, 2, newQtda)
        tableItens.setItem(item_id, 3, newUnid)
        tableItens.setItem(item_id, 4, newVUni)
        tableItens.setItem(item_id, 5, newVDes)
        tableItens.setItem(item_id, 6, newVTot)
        tableItens.setItem(item_id, 7, newAtiv)
        tableItens.scrollToBottom()
        #Redefinir variaveis globais e controles
        self.ui.lblVlrTotal.setText(u'%1.2f' % self._coupon_amount)
        self.atualiza_valor_a_pagar()
        self.ui.lblVlrTotal_2.setText(u'%1.2f' % self._coupon_amount)

        self.ui.spinQtda.setValue(1)

        self._item_discount = Decimal(0)
        self._item_quantity = Decimal(1)
        self._state_item = item_none
        self._CONTROL_FOCUS = self.ui.edtCodigo
        return True

    @Log
    def _clear_globals(self, coupon_state):
        if self.user_logged:
            self.ui.lblNomeUser.setText(self.user_logged.name)
            self._operador_codigo = self.user_logged.supope_codigo
            self._operador_id = self.user_logged.id

        self.mov_capa = False
        self.nfce_qr_code = ''
        self.comanda_id = False
        self._IdsComandas = []
        self._consulta_cheque = False
        self._menu_atualiza = False
        self.cliente_identificado = False

        self.vlr_item_dav = False
        self._PveCancelamento = False
        self.ui.lblVlrItem.setText(u'%.2f' % 0)
        self.ui.lblInfoTicket.setText(u'%.2f' % 0)
        self.ui.lblInfoDinheiro.setText(u'%.2f' % 0)
        self.ui.lblInfoOutros.setText(u'%.2f' % 0)
        self.ui.lblInfoCartao.setText(u'%.2f' % 0)
        self.ui.lblInfoCheque.setText(u'%.2f' % 0)
        self.ui.lblInfoAcresTotalItens.setText(u'%.2f' % 0)
        self.ui.lblInfoDescTotalItens.setText(u'%.2f' % 0)
        self.ui.lblInfoDescCapa.setText(u'%.2f' % 0)
        self.ui.lblVlrTotal.setText(u'%.2f' % 0)
        self.ui.lblVlrTotalAPagar.setText(u'%.2f' % 0)
        self.ui.lblVlrTotal_2.setText(u'%1.2f' % 0)
        if not self.vr_doacao or not self.vr_ficha:
            self.ui.lblVlrTroco.setText(u'%.2f' % 0)
            self.ui.lblVlrReceber.setText(u'%.2f' % 0)

        self.ui.lblVlrPgto.setText(u'%.2f' % 0)
        self.ui.lblTotalFichas.setText(u'%.2f' % 0)
        self.ui.edtClienteFicha.clear()
        self.ui.lblOCliente.clear()
        self._coupon_subtotal = Decimal(0)
        self.vr_doacao = Decimal(0)
        self.vr_ficha = Decimal(0)
        self.docao_bloqueio = False

        # Redefine as variáveis globais
        self._state_coupon_old = False
        self._state_coupon = coupon_state
        self._state_item = item_none
        self._item_discount = Decimal(0)
        self._item_quantity = Decimal(1)
        self.last_item = 0
        self._coupon_amount = Decimal(0)
        self._coupon_payment_method = None
        self._coupon_totalize = None
        self._payment_amount = Decimal(0)
        self._indicador_desc_acre = ''
        self._pves_list = False

        # Variaveis utilizadas para controlar os itens que são vendido em promocao por cupom
        self._itens_promocao = False
        self.promocoes_aplicadas = {}
        self._id_promocao = None
        self._primeiro_desconto_qtd = False
        self._vlr_desc_promocao = None

        # controle socio torcedor
        self._tef_nsu_socio_torcedor = False
        if not self.consulta_produto and not self.socio_torcedor and self.mov_fechamentos:
            self.sig_tef.usando_sitef = False

        self.ui.cbSerie.clear()
        self.ui.frmSerie.setVisible(False)
        self.clear_widget_table(self.ui.tableItens)
        self.acao_pendente_z = False
        self.tela_socio_torcedor = False
        self.controle_cancelamento_tef = False

        return True

    @Log
    def _lock_keyboard(self, lock=False):
        self._keyboard_lock = lock
        #        lock = False if lock else True
        lock = True
        self.ui.frameKeyboard.setEnabled(True)
        return lock

    def get_acao_pendente(self):
        '''Antes de iniciar a improtacao verifica se tem algo a se importardo, se houver retorna True
        se nao houver retorna False
        '''
        res = False
        if not self.conectado:
            return res

        sql = '''SELECT id
                 FROM supervisor_mov_acao_pendente
                 WHERE supeape_loja = {loja}
                   AND supeape_pdv = {caixa}
                   AND supape_situacao = 'A'
                 LIMIT 1
              '''.format(loja=self.loja.supcfc_loja,
                         caixa=self.caixa_id)
        try:
            res = True if exec_sql(sql, 'S') else False
        except Exception as e:
            Log.error(str(e))

        return res

    def _timer_principal(self):
        if self._tempo_ocioso >= config['time_importacao'] > 0:
            if not self.get_acao_pendente():
                self._tempo_ocioso = 0
                return True

        # Todo: unificar as condicoes para 'self.importacao_bloqueada'
        if not self.importacao_bloqueada \
           and not self.emitindo_dav \
           and self._tempo_ocioso >= config['time_importacao'] > 0 \
           and self.status_conexao_supervisor \
           and self._close_window and self._state_coupon == coupon_closed \
           and self._state_screen in [in_login, in_coupon, in_end_sale, in_ficha] \
           and not self._consulta_cheque and not self.sig_tef.usando_sitef \
           and not self._comanda_ativa and not self.ui.edtCodigo.text()\
           and not self.ui.edtItemFicha.text():
            Log.info('Status de importação')
            if not self.obj_cupom_fiscal.get_cupom_aberto():
                self._tempo_ocioso = 0
                try:
                    QtCore.QTimer.singleShot(0, self.executar_importacao)
                except Exception as err:
                    Log.info(str(err))
                    return self.parar_importacao(str(err))
        elif self._close_window and self._state_coupon == coupon_closed:
            self._tempo_ocioso += 1
        else:
            self._tempo_ocioso = 0

        if self.status_conexao_supervisor:
            self._time_offline = 0
        else:
            self._time_offline += 1
        if self._limpar_info >= 3 and self.ui.lblInfo.text() and not self._executando_importacao:
            if self.teclado and self.limpa_display:
                self.limpa_display = False
                self.teclado.limpar_display()

            self.ui.lblInfo.clear()
            self.ui.lblItems.clear()
            self.ui.lblVlrItem.setText(u'0,00')
            self._limpar_info = 0
        else:
            self._limpar_info += 1
        status_conn = '  - Tempo ocioso: ' + str(self._tempo_ocioso)
        if self._time_offline:
            status_conn = ' - Tempo offline: ' + str(self._time_offline)
        self.ui.lblTime.setText(
            datetime.now().strftime('%d/%m/%Y %H:%M') + status_conn)
        return True

    def thread_tela_final(self):
        if self._state_screen == in_end_sale:
            self.KEY_ESCAPE_Click()

        self.timer_fim_de_venda.stop()

    def _timer_end_sale_exec(self):
        texto = "*** Pressione a tecla SAIR p/ Vender ***"
        if config['usagaveta']:
            texto += '  FECHE A GAVETA'

        self.infoText(texto)
        # para mostrar o troco, esta mensagem nao deve ser enviada pela funcao self.infoText
        if self.teclado:
            self.teclado.envia_linha_display(texto, 1)

        html_text = '''<p align="center" style=" margin-top:0px; margin-bottom:0px;
        margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">
        <span style=" font-family:'DejaVu Sans'; font-size:20pt; font-weight:600;
        color:#0000ff;">Obrigado e Volte Sempre!!!</span></p>'''
        self.ui.label_2.setText(html_text)
        self._set_screen_state(in_end_sale)


    def _active_scanner_timer(self):
        if self.estacionamento:
            return

        cod = ''
        try:
            if QtGui.QApplication.activeWindow():
                cod = self.barcode.get_code()

        except Exception as e:
            cod = ''

        if self._CONTROL_FOCUS in [self.ui.edtCodigo, self.ui.edtBuscaCliente, self.ui.edtUser]:
            if cod:
                old_focul = self._CONTROL_FOCUS
                self._CONTROL_FOCUS.setText(cod)
                QtGui.QApplication.processEvents()
                self.KEY_RETURN_Click()
                if old_focul == self.ui.edtBuscaCliente and config.get('usarcodificado', False):
                    self.setControlFocus(self._CONTROL_FOCUS, self.ui.edtBuscaCliente)

    @Log
    def show_info_proccess(self, visible, msg='Processo demorado', tef=False):
        self._lock_keyboard(visible)
        self._info_dialog._not_close = visible

        if tef and self._info_dialog.isVisible():
            QtGui.QApplication.processEvents()
            return self._info_dialog.set_info_text(self.trUtf8(msg))

        if visible:
            self._info_dialog.set_info_text(self.trUtf8(msg))
            if not self._info_dialog.isVisible():
                self._info_dialog.showNormal()
        else:
            self._info_dialog.close()

        QtGui.QApplication.processEvents()
        QtGui.QApplication.processEvents()
        return 'MSG: Fim processo demorado.'

    @Log
    def check_pves(self, days=1):
        self.infoText('Checando Pré-Vendas: %d Dia(s)' % days)
        try:
            sql = """SELECT id
                     FROM   supervisor_pve_capa
                     WHERE  suppva_data_hora <= '%s 23:59:59'
                            AND suppva_data_cupom is null
                """ % (datetime.now().date() - timedelta(days=days))
            pve_ids = exec_sql(sql, 'S', ret=True)

            retval = len(pve_ids)
            vlr_pvs = map(lambda v: v['id'], pve_ids)
            if pve_ids:
                for id in vlr_pvs:
                    self._PveCancelamento = True
                    capa = SupervisorPveCapa.get_by(id=id)
                    b_sess.begin()
                    capa.suppva_restricao = -1
                    b_sess.commit()
                    self._returnPressed_tableBuscaPve(None, str(id), True)
        except Exception:
            self.tratar_except()
            return False
        return retval

    @Log
    def tratar_except(self, tipo_msg=MSG_ERROR, mostrar_tela=True):
        self.ui.lblItems.clear()
        self.ui.lblVlrItem.setText(u'0,00')
        msg_err = ''
        s_lines = traceback.format_exc().splitlines()
        for s in s_lines:
            msg_err += self.trUtf8('%s\n' % s)
        self.infoText(str(s_lines[-1]))
        Log.error('TRATAR EXCEPT: %r' % msg_err)
        try:
            sess.rollback()
        except Exception as e:
            Log.error('TRATAR EXCEPT: %r' % e)
            return message_dlg(self, MSG_ERROR, 'Erro ao desfazer transação!'
                                                '\nInforme este fato ao Suporte Técnico!\n'
                                                '\n%s' % str(e))
        Log.error(s_lines[-1])
        if mostrar_tela:
            return message_dlg(self, tipo_msg, 'Houve um erro interno!'
                                               '\nInforme este fato ao Suporte Técnico!\n'
                                               '\n%s' % str(s_lines[-1]))

    @Log
    def parar_importacao(self, err=''):
        if self._state_screen == in_end_sale:
            self.KEY_ESCAPE_Click()
        else:
            self._set_screen_state(self._state_screen)
        self._lock_keyboard(False)
        self._executando_importacao = False
        return message_dlg(self, MSG_ERROR, 'Erro na Importação!\n\n'
                                            'Conexão com o Supervisor foi interrompida.\n'
                                            'Informe este fato ao Suporte Técnico!\n'
                                            '\n%s' % err)

    @Log
    def executar_importacao(self):
        if self._executando_importacao:
            Log.info('Importacao ja esta sendo executada')
            return False

        # controle para inabilitar os botoes quando estiver importando acoes pendentes
        screen_antigo = self._state_screen
        self._state_screen = in_importacao
        self.check_button_status()
        self._state_screen = screen_antigo
        self.ui.progressBarImport.setValue(0)
        inc = 0
        while True:
            if self.user_logged:
                self._lock_keyboard(False)
            inc += 1
            self.infoText('Tentando conectar no Supervisor (%d/5)...' % inc, display=True)
            QtGui.QApplication.processEvents()
            sleep(1)
            if self.status_conexao_supervisor:
                self.ui.stackedActions.setCurrentIndex(stk_importacao)
                break
            if inc == 5:
                self.infoText('Não foi possível conectar no Supervisor.')
                if not self.user_logged:
                    self._set_screen_state(in_login)
                    self._lock_keyboard(False)
                else:
                    self._clear_globals(coupon_closed)
                    self._set_screen_state(in_coupon)
                    self._lock_keyboard(False)
                self._executando_importacao = False
                break

        if not self.status_conexao_supervisor:
            return self.parar_importacao()

        self._executando_importacao = True
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.listImportacao)
        self._lock_keyboard(True)
        self._tempo_ocioso = 0
        self._close_window = False

        erro = False
        full = config['importacao_full']

        obj_importacao_automatica = ImportacaoAutomatica(self, full)
        try:
            # ao parar a exportacao, o cupom que estava sendo exportado nao foi comitado no supervisor
            # mas foi alterado no caixa como enviado. Ao voltar a exportacao o cupom foi enviado novamente
            # e no cupom servidor estava marcado como true, entao fica como cupom faltate no ERP pois ele
            # nao eh validado. 12/01/2016 chamado
            # self.thread_exportacao.cancel()
            obj_importacao_automatica.importar(config['loja'], self.caixa_id)
            self.set_data_atualizacao()
            self.ui.lblInfoEcf.setText(self.ui.lblInfoEcf.text()[:-16] + self.data_atualizacao)
        except Exception as err:
            self.parar_importacao(str(err))

        finally:
            obj_importacao_automatica.connection_caixa.close()
            obj_importacao_automatica.connection_supervisor.close()

        if erro:
            return False

        self.ui.progressBarImport.setValue(100)
        QtGui.QApplication.processEvents()
        sleep(1)

        if full:
            inc = 0
            loop = 10
            while True:
                inc += 1
                self.show_info_proccess(True, 'Importação Full concluída! \n'
                                              'Aguarde, o sistema irá reiniciar '
                                              'em (%d-%d)' % (inc, loop))
                QtGui.QApplication.processEvents()
                sleep(1)
                if inc == loop:
                    break

            self.teclado and self.teclado.limpar_display()
            sleep(1)
            self.teclado and self.teclado.fechar_teclado()
            sleep(1)
            return reload_app(ROOT_PATH)

        if self.acao_pendente_z:
            self.infoText(u'Emissão de Redução Z automática.', False)
            self.ui.listImportacao.item(12).setIcon(self.icon_arrow)
            QtGui.QApplication.processEvents()
            if self.user_logged:
                operador = self.user_logged.supope_codigo
            else:
                operador = CaixaCadOperadores.get_by(supope_operador_supervisor='S').supope_codigo

            if self.par_sangria_x_z_suprimento and \
                            operador == 'O':
                dlg = password_dlg(self, msg_info='')
                if not dlg.retval:
                    return False

            if ReducaoXZ(self).gerar_reducao_xz(self.loja.supcfc_loja, self.caixa_id, 'Z', operador):
                self.acao_pendente_z = False

        self.infoText('')
        if self._state_screen == in_end_sale:
            self.KEY_ESCAPE_Click()
        else:
            self._set_screen_state(self._state_screen)
        self._lock_keyboard(False)
        self._close_window = True
        self._executando_importacao = False

        return True


@Log
def verifica_numeracao_nfce(nro_nota, caixa, n_loja):
    dados_caixa = exec_sql("""SELECT supcap_nro_cupom AS nro_nota
                              FROM caixa_mov_capa AS cmc
                              INNER JOIN caixa_mov_cupom_servidor AS cmcs on supcse_capa_id = cmc.id
                              WHERE supcap_caixa = %s
                                AND supcap_loja = %s 
                                AND supcap_tipo IN ('V', 'C') """ % (caixa, n_loja), 'C')
    nro_registro = max(map(lambda x: x.get('nro_nota'), dados_caixa) or [0])
    try:
        dados_super = exec_sql("""SELECT supcap_nro_cupom AS nro_nota
                                  FROM supervisor_mov_capa 
                                  WHERE supcap_caixa = %s
                                    AND supcap_loja = %s 
                                    AND supcap_tipo IN ('V', 'C')
                                    AND supcap_state IS NOT NULL""" % (caixa, n_loja), 'S')
        dados_super = max(map(lambda x: x.get('nro_nota'), dados_super) or [0])
        if nro_registro < dados_super:
            nro_registro = dados_super
    except:
        pass

    if nro_registro and nro_nota <= nro_registro:
        message_dlg(None, MSG_ERROR, 'Atenção!\n\nNúmero da nota no PDVConfig está menor ou igual\n'
                                     'que o número da nota emitida por este caixa!\n'
                                     'Última nota: %s\nPDVConfig: %s' % (nro_registro, nro_nota))
        os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
        sys.exit(-1)

    return True

@Log
def verifica_numeracao_dav(caixa, n_loja):
    dados_caixa = exec_sql("""SELECT supcap_nro_cupom AS nro_nota
                              FROM caixa_mov_capa AS cmc
                              INNER JOIN caixa_mov_cupom_servidor AS cmcs on supcse_capa_id = cmc.id
                              WHERE supcap_caixa = %s
                                AND supcap_loja = %s 
                                AND supcap_tipo IN ('D') """ % (caixa, n_loja), 'C')
    nro_registro = max(map(lambda x: x.get('nro_nota'), dados_caixa) or [0])
    try:
        dados_super = exec_sql("""SELECT supcap_nro_cupom AS nro_nota
                                  FROM supervisor_mov_capa 
                                  WHERE supcap_caixa = %s
                                    AND supcap_loja = %s 
                                    AND supcap_tipo IN ('D')
                                    AND supcap_state IS NOT NULL""" % (caixa, n_loja), 'S')

        dados_super = max(map(lambda x: x.get('nro_nota'), dados_super) or [0])
        if nro_registro < dados_super:
            nro_registro = dados_super
    except:
        pass

    if nro_registro and config['nro_dav'] <= nro_registro:
        message_dlg(None, MSG_ERROR, 'Atenção!\n\nNúmero do DAV no PDVConfig está menor ou igual\n'
                                     'que o número do DAV emitido por este caixa!\n'
                                     'Última DAV: %s\nPDVConfig: %s' % (nro_registro, config['nro_dav']))
        os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
        sys.exit(-1)

    return True

@Log
def verifica_numeracao_cupom_gerencial(caixa, loja):
    dados_caixa = exec_sql("""SELECT MAX(supcap_nro_cupom) AS nro_nota
                              FROM caixa_mov_capa AS cmc
                              INNER JOIN caixa_mov_cupom_servidor AS cmcs on supcse_capa_id = cmc.id
                              WHERE supcap_caixa = %s
                                AND supcap_loja = %s 
                                AND supcap_tipo NOT IN ('V', 'D', 'C') """ % (caixa, loja), 'C')
    nro_registro = max(map(lambda x: x.get('nro_nota'), dados_caixa) or [0])
    try:
        dados_super = exec_sql("""SELECT MAX(supcap_nro_cupom) AS nro_nota
                                  FROM supervisor_mov_capa 
                                  WHERE supcap_caixa = %s
                                    AND supcap_loja = %s 
                                    AND supcap_nro_serie IS NULL
                                    AND supcap_state IS NULL""" % (caixa, loja), 'S')

        dados_super = max(map(lambda x: x.get('nro_nota'), dados_super) or [0])
        if nro_registro < dados_super:
            nro_registro = dados_super
    except:
        pass

    if nro_registro and config['coo_cup_gerencial'] <= nro_registro:
        message_dlg(None, MSG_ERROR, 'Atenção!\n\nNúmero do cupom gerencial no PDVConfig está menor ou igual\n'
                                     'que o número do cupom emitido por este caixa!\n'
                                     'Último Cupom: %s\nPDVConfig: %s' % (nro_registro, config['coo_cup_gerencial']))
        os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
        sys.exit(-1)
    return True

@Log
def verifica_hora_servidor_sefaz(loja, venda=False):
    if config['apenas_ficha']:
        return False

    ambiente = config['nfce_ambiente'] == 'P' and 1 or 2
    data_sefaz = False
    try:
        processo, _ = nfc_e(loja.supcfc_certificado_conteudo.decode('string_escape'),
                            loja.supcfc_chave_conteudo.decode('string_escape'),
                            ambiente,
                            loja.supcfc_estado).process.consultar_servico()
        data_sefaz = processo.resposta.dhRecbto.valor
    except Exception as e:
        Log.error('Falha ao buscar data da SEFAZ: %s' % e)

    data_agora = datetime.strptime(str(datetime.now())[:19], '%Y-%m-%d %H:%M:%S')
    data_agora = converte_tz(False, data_agora, 'America/Sao_Paulo', 'America/Sao_Paulo')
    if data_sefaz:
        if data_agora > data_sefaz:
            dif = data_agora - data_sefaz
        else:
            dif = data_sefaz - data_agora

        if data_agora.tzname() != data_sefaz.tzname():
            msg = 'Atenção!\n\nFuso horário da SEFAZ diferente do fuso do Computador!\n' + \
                  'Fuso SEFAZ: %s\nFuso Computador: %s' % (data_sefaz.tzname(), data_agora.tzname())
            message_dlg(None, MSG_ERROR, msg)
            os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
            sys.exit(-1)

        # alterar toda vez que ligar o pdv e for producao
        if config['nfce_ambiente'] == 'P' and dif.seconds > 1:  # 300 segundos = 5 minutos
            erro = ''
            if commands.getoutput('whoami') not in ['caixa', 'supersig', 'sig']:
                cmd = "%s '%s'" % (config['cmd_alterar_data'],
                                   data_sefaz.strftime('%Y-%m-%d %H:%M:%S'))
                sp = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE)
                erro = sp.stderr.read()
            else:
                cmd = "echo %s | sudo -S %s '%s'" % (config['senha_usuario'],
                                                     config['cmd_alterar_data'],
                                                     data_sefaz.strftime('%Y-%m-%d %H:%M:%S'))
                if 'incorrect password' in commands.getoutput(cmd):
                    erro = 'incorrect password attempt'

            if erro and not venda:
                message_dlg(None, MSG_ERROR,
                            'Não foi possível alterar a hora do Sistema.\n'
                            'Verifique as configurações de Data/Hora '
                            'do seu computador!\n\n%s' % erro)
                sys.exit(-1)

            if venda:
                Log.error("Não foi possível alterar a hora do sistema: %s" % erro)
                return False

            message_dlg(None, MSG_INFO, 'Atenção!\n\nData alterada para %s' %
                                                           data_sefaz.strftime('%d/%m/%Y %H:%M:%S'))
            return data_sefaz

            # if message_dlg(None, MSG_QUESTION, 'Atenção!\n\nData do cumputador diferente da SEFAZ\n'
            #                              'Data SEFAZ: %s\nData Computador: %s\n\nDeseja alterar Hora do sistema?' % (data_sefaz,
            #                                                                       data_agora)) != YES_ROLE:
            #     os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
            #     sys.exit(-1)
            # else:
            #     dlg = password_dlg(None, msg_info='Alterar Data do sistema para %s' % data_sefaz.strftime('%d-%m-%Y %H:%M '))
            #     if dlg.retval:
            #         erro = ''
            #         if commands.getoutput('whoami') not in ['caixa', 'supersig', 'sig']:
            #             cmd = "%s '%s'" % (config['cmd_alterar_data'],
            #                                data_sefaz.strftime('%Y-%m-%d %H:%M:%S'))
            #             sp = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
            #                                   stderr=subprocess.PIPE)
            #             erro = sp.stderr.read()
            #         else:
            #             cmd = "echo %s | sudo -S %s '%s'" % (config['senha_usuario'],
            #                                                  config['cmd_alterar_data'],
            #                                                  data_sefaz.strftime('%Y-%m-%d %H:%M:%S'))
            #             if 'incorrect password' in commands.getoutput(cmd):
            #                 erro = 'incorrect password attempt'
            #
            #         if erro:
            #             message_dlg(None, MSG_ERROR,
            #                         'Não foi possível alterar a hora do Sistema.\n'
            #                         'Verifique as configurações de Data/Hora '
            #                         'do seu computador!\n\n%s' % erro)
            #             sys.exit(-1)
            #
            #         return reload_app(ROOT_PATH)

    else:
        if venda:
            return False

        message_dlg(None, MSG_INFO, 'Atenção!'
                                    '<center><h1><font color="red">Sem comunicação com a SEFAZ</font></h1></center>'
                                    'Verifique se a hora do Computador está correta!\nData: %s'
                                    '<center><h2><font color="red">Entre em contato com o seu SUPERVISOR'
                                    ' antes de realizar VENDAS</font></h2></center>'
                                     % data_agora.strftime('%d/%m/%Y %H:%M:%S Fuso:%Z'))

    return data_sefaz

@Log
def run():
    import os
    from sigpdv.tools.config import config
    from sigpdv.control.utils import update_lista_app, reload_app
    from sigpdv.tools.utils import del_logs
    from time import sleep
    from sigpdv.tools.singleton import SingleInstance

    Log.info('================================================================================')
    Log.info('Versao do sistema: SigPDV: %s ' % get_version())
    Log.info('')
    Log.info('* Parametros do PDVConfig:')

    cfg_params = map(lambda x: '%s=%s' % (x[0], config[x[0]]), _OPTIONS)
    idx = 1
    prsm = []
    for p in cfg_params:
        prsm.append(p)
        if len(prsm) == 5:
            Log.info('[' + ' | '.join(prsm) + ']')
            prsm = []
        if len(prsm) < 5 and idx == len(cfg_params):
            Log.info('[' + ' | '.join(prsm) + ']')
        idx += 1

    Log.info('================================================================================')

    init_msg = []

    if len(sys.argv) > 1:
        os.system('kill %s' % sys.argv[1])
    app = QtGui.QApplication(sys.argv)
    # Não executar como root
    if os.name == 'posix':
        import pwd

        if pwd.getpwuid(os.getuid())[0] == 'roots':
            msg = u"Tentativa de executar o SiG Frente como root. Isso não é bom, abortando.\n"
            QtGui.QMessageBox.critical(None, 'SiG Frente', msg)
            sys.exit(1)

    pixmap = QtGui.QPixmap("splash.png")
    splash = QtGui.QSplashScreen(pixmap)
    splash.show()
    app.processEvents()
    wait = 0.5
    wait_exit = 0.5
    alignment = QtCore.Qt.AlignCenter
    msg_color = QtCore.Qt.white
    msg_err_color = QtCore.Qt.cyan
    msg = 'Arquivos Auxiliares:\n%s\n%s\n%s' % (
        os.path.join(ROOT_PATH, 'pdvsig.sig'),
        os.path.join(ROOT_PATH, 'gt.sig'),
        os.path.join(ROOT_PATH, 'lista_de_aplicativos.txt')
    )
    Log.info(msg)
    splash.showMessage(msg, alignment=alignment, color=msg_err_color)
    app.processEvents()

    try:
        msg = 'Verificando instância da aplicação'
        Log.info(msg)
        splash.showMessage(app.trUtf8(msg), alignment=alignment, color=msg_color)
        app.processEvents()
        SingleInstance('sigpdv')
        if not config['impressora']:
            raise Exception('Sem impressora cadastrada no PDVConfig')

    except IOError as e:
        msg = '%s' % e
        Log.error(msg)
        splash.showMessage(app.trUtf8(msg), alignment=alignment, color=msg_err_color)
        app.processEvents()
        init_msg.append(e)
        sleep(wait_exit)
        os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
        sys.exit(-1)

    try:
        msg = 'Apagando arquivos de log antigos'
        Log.info(msg)
        splash.showMessage(app.trUtf8(msg), alignment=alignment, color=msg_color)
        app.processEvents()
        if config['max_dias_log']: del_logs(config['max_dias_log'], os.path.join(ROOT_PATH, 'logs'))
    except Exception as e:
        msg = traceback.format_exc()
        Log.error(msg)
        splash.showMessage(app.trUtf8(msg), alignment=alignment, color=msg_err_color)
        app.processEvents()
        init_msg.append(e)
        sleep(wait_exit)

    try:
        msg = 'Inicializando o SigPDV...'
        Log.info(msg)
        splash.showMessage(app.trUtf8(msg), alignment=alignment, color=msg_color)
        app.processEvents()
        caixa = config['caixa']
        n_loja = config['loja']
        nro_nota = config['numero_nfce']
        Log.info("LOJA ::::::::::::::::::::::: %s" % n_loja)
        Log.info("CAIXA ::::::::::::::::::::::: %s" % caixa)
        update_lista_app(n_loja, caixa)
        loja = get_loja()
        if caixa <= 0:
            message_dlg(None, MSG_ERROR,
                        'Atenção!\n\nNúmero do caixa deve ser mair que 0(zero)')
            os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
            sys.exit(-1)

        verifica_numeracao_nfce(nro_nota, caixa, n_loja)
        verifica_numeracao_dav(caixa, n_loja)
        verifica_numeracao_cupom_gerencial(caixa, n_loja)
        verifica_hora_servidor_sefaz(loja)
        if not config['apenas_ficha']:
            if not loja.supcfc_data_validade:
                message_dlg(type=MSG_INFO, text='Atenção!\n\nSem Certificado Digital cadastrado!')

            elif (loja.supcfc_data_validade.date() - datetime.now().date()).days < 30:
                msg = 'Atenção!\n\n Certificado Digital expira em %s dias' % \
                      (loja.supcfc_data_validade.date() - datetime.now().date()).days
                message_dlg(type=MSG_INFO, text=msg)

            elif (loja.supcfc_data_validade - datetime.now()).days < 0:
                msg = 'Atenção!\n\n Certificado Digital expirado!'
                message_dlg(type=MSG_INFO, text=msg)

    except Exception as e:
        tipo_err = type(e)
        if tipo_err in (CommError, DriverError, PrinterFileError):
            message_dlg(type=MSG_ERROR, text='%s\n\n Erro ao inicializar a Aplicação' % e)
        elif tipo_err == EcfDateError:
            message_dlg(None, MSG_ERROR, str(e))
        elif tipo_err in (PendingReduceZ, AlmostOutofPaper, CouponOpenError):
            init_msg.append(e)
        else:
            init_msg.append(e)
        msg = '%s' % e
        Log.error(msg)
        splash.showMessage(app.trUtf8(msg), alignment=alignment, color=msg_err_color)
        app.processEvents()
        sleep(wait)

    try:
        host = config['db_sup_host']
        msg = 'Verificando conexão com servidor %s' % host
        Log.info(msg)
        splash.showMessage(app.trUtf8(msg), alignment=alignment, color=msg_color)
        app.processEvents()
        exec_sql('SELECT * FROM supervisor_mov_capa LIMIT 1', 'S')
        splash.showMessage('Conexao estabelecida %s!' % host, alignment=alignment, color=msg_color)
        status_conexao = True
    except Exception as e:
        status_conexao = False
        splash.showMessage('Sem conexão com Supervisor.', alignment=alignment, color=msg_err_color)
        app.processEvents()
        sleep(wait)
        init_msg.append(PdvConnectionError(msg + '\n\n' + str(e).replace('\t', '')
                                           .replace('None', '').replace('does not exist',
                                                                        'não existe')))

    try:
        splash.showMessage(app.trUtf8('Verificando informações do PAF-ECF'), alignment=alignment,
                           color=msg_color)

        app.processEvents()
        tbl_ecf = CaixaCfgEcfAutorizada.query.all()
        for reg in tbl_ecf:
            sess.begin()
            reg.delete()
            sess.commit()

        # sess.begin()
        # CaixaCfgEcfAutorizada(
        #     supecf_loja=config['loja'],
        #     supecf_caixa=config['caixa'],
        #     supecf_data_sb=ecf.CDataSB,
        #     supecf_fabricante=ecf.CFabricante,
        #     supecf_hora_sb=ecf.CHoraSB,
        #     supecf_modelo=ecf.CModelo,
        #     supecf_nserie=ecf.CNSerie,
        #     supecf_nacional=ecf.CNacional,
        #     supecf_versao=ecf.CVersao,
        #     supecf_empresa=ecf.Empresa,
        #     supecf_sign=ecf.Sign,
        #     supecf_md5=ecf.md5
        # )
        # sess.commit()
    except PrinterFileError as e:
        print 'erro', msg, 'PrinterFileError'
        Log.error(msg)
        window = PdvWindow()
        splash.finish(window)
    except Exception as e:
        print 'erro***', e
        splash.showMessage(str(e).decode('utf-8'), alignment=alignment, color=msg_err_color)
        app.processEvents()
        sleep(wait)
        init_msg.append(PdvConnectionError(str(e)))

    if config['check_atualiza']:
        splash.showMessage(app.trUtf8('Verificando atualizações disponíveis!'),
                           alignment=alignment, color=msg_color)
        if not SigFrenteUp():
            splash.showMessage(app.trUtf8('O Sistema já está atualizado!'),
                               alignment=alignment, color=msg_color)

    splash.showMessage('Iniciando SigFrente ' + get_version(), alignment=alignment, color=msg_color)
    window = PdvWindow(init_msg=init_msg, status_conexao=status_conexao)
    splash.finish(window)

    window.showFullScreen()
    try:
        update_lista_app(n_loja, caixa)
    except Exception as e:
        print 'update_lista_app', e
        init_msg.append(e)

    # Tradução
    qtTranslator = QtCore.QTranslator()
    if qtTranslator.load("qt_pt.qm"):
        app.installTranslator(qtTranslator)
        Log.info('installTranslator pt_BR')
    else:
        pass
        Log.warning('Arquivo de tradução não encontrado')
    app.exec_()

    if RELOAD:
        reload_app(ROOT_PATH)


if __name__ == "__main__":
    try:
        run()
    except Exception as e:
        try:
            QtGui.QApplication(sys.argv)
        except Exception:
            pass

        message_dlg(None, MSG_ERROR, str(e))
        os.system('echo %s | sudo -S %s' % (config['senha_usuario'] or 'sig', 'killall -9 sigfrente'))
        sys.exit(-1)





